/**
 * FLL - Level 3
 *
 * Project: IKI Read
 * API Version: 0.7
 * Licenses: lgpl-2.1-or-later
 *
 * Provides the common type structures.
 *
 * This is auto-included and should not need to be explicitly included.
 */
#ifndef _iki_read_common_type_h
#define _iki_read_common_type_h

#ifdef __cplusplus
extern "C" {
#endif

/**
 * Make the typedef for iki_read_main_t available to child structures.
 */
#ifndef _di_iki_read_main_t__
  typedef struct iki_read_main_t_ iki_read_main_t;
#endif // _di_iki_read_main_t__

/**
 * The IKI read main program cache.
 *
 * Properties:
 *   - buffer: A buffer used for loading the files and other miscellaneous tasks.
 *   - expand: A buffer used in certain cases for the expanded output that is to be printed.
 */
#ifndef _di_iki_read_setting_t_
  typedef struct {
    f_string_dynamic_t buffer;
    f_string_dynamic_t expand;
  } iki_read_cache_t;

  #define iki_read_cache_t_initialize \
    { \
      f_string_dynamic_t_initialize, \
      f_string_dynamic_t_initialize, \
    }
#endif // _di_iki_read_cache_t_

/**
 * The IKI write callbacks.
 *
 * Properties:
 *   - identify_alteration: Identify the location of an alteration.
 *
 *   - print_data: Print IKI/EKI data at the given index.
 *   - print_help: Print the main help message.
 *
 *   - process_buffer_ranges:       Process and print the given range.
 *   - process_buffer_ranges_whole: Process and print the given ranges in whole mode.
 *   - process_buffer_total:        Process and print the total.
 *
 *   - delete_data: Delete the IKI/EKI data.
 */
#ifndef _di_iki_read_callback_t_
  typedef struct {
    void (*identify_alteration)(iki_read_main_t * const main);

    void (*print_data)(fl_print_t * const print, const f_number_unsigned_t index);
    void (*print_help)(fl_print_t * const print);

    void (*process_buffer_ranges)(iki_read_main_t * const main, f_range_t * const range_buffer);
    void (*process_buffer_ranges_whole)(iki_read_main_t * const main, f_range_t * const range_buffer);
    void (*process_buffer_total)(iki_read_main_t * const main);

    void (*delete_data)(iki_read_main_t * const main);
  } iki_read_callback_t;

  #define iki_read_callback_t_initialize \
    { \
      0, \
      0, \
      0, \
      0, \
      0, \
      0, \
      0, \
    }
#endif // _di_iki_read_callback_t_

/**
 * The IKI read main program settings.
 *
 * This is passed to the program-specific main entry point to designate program settings.
 * These program settings are often processed from the program arguments (often called the command line arguments).
 *
 * Properties:
 *   - flag: Flags passed to the main function.
 *
 *   - status_signal: A status used eclusively by the threaded signal handler.
 *   - state:         The state data used when processing the FSS data.
 *
 *   - at:     The position representing the "at" index.
 *   - line:   The position representing the "line" index.
 *   - select: The position representing the "select" index.
 *
 *   - name:   A string representing the IKI vocabulary name being selected.
 *   - files:  An array of all files to process (except for the input pipe).
 *
 *   - replace:    A map containing the Vocabulary name and its replacement as the value.
 *   - reassign:   A triple containing the Vocabulary name (a), Content to match (b), and the string used as the reassign (c).
 *   - substitute: A triple containing the Vocabulary name (a), Content to match (b), and the string used as the substitute (c).
 *   - wrap:       A triple containing the Vocabulary name (a), the string to prepend (b), and the string to append (c).
 *
 *   - map_replacess: Map the vocabulary location to a location containing the replace parameter value.
 *   - map_wrapss:    Map the vocabulary location to a location containing the wrap parameter value.
 */
#ifndef _di_iki_read_setting_t_
  typedef struct {
    uint32_t flag;

    f_status_t status_signal;
    f_state_t state;

    f_number_unsigned_t at;
    f_number_unsigned_t line;
    f_number_unsigned_t select;

    f_string_dynamics_t names;
    f_string_dynamics_t files;

    f_string_maps_t replace;
    f_string_triples_t reassign;
    f_string_triples_t substitute;
    f_string_triples_t wrap;

    f_number_unsignedss_t map_replacess;
    f_number_unsignedss_t map_wrapss;
  } iki_read_setting_t;

  #define iki_read_setting_t_initialize \
    { \
      iki_read_main_flag_none_d, \
      F_okay, \
      macro_f_state_t_initialize_1(iki_read_allocation_large_d, iki_read_allocation_small_d, F_okay, 0, 0, 0, 0, 0, 0, 0), \
      0, \
      0, \
      0, \
      f_string_dynamics_t_initialize, \
      f_string_dynamics_t_initialize, \
      f_string_maps_t_initialize, \
      f_string_triples_t_initialize, \
      f_string_triples_t_initialize, \
      f_string_triples_t_initialize, \
      f_number_unsignedss_t_initialize, \
      f_number_unsignedss_t_initialize, \
    }
#endif // _di_iki_read_setting_t_

/**
 * The main program data as a single structure.
 *
 * Properties:
 *   - cache:    The cache data.
 *   - callback: The callbacks.
 *   - data:     A pointer for the structure representing IKI data (or EKI data).
 *   - program:  The main program data.
 *   - setting:  The settings data.
 */
#ifndef _di_iki_read_main_t_
  struct iki_read_main_t_ {
    iki_read_cache_t cache;
    iki_read_callback_t callback;
    void * data;
    fll_program_data_t program;
    iki_read_setting_t setting;
  };

  #ifndef iki_read_main_t_initialize
    #define iki_read_main_t_initialize \
      { \
        iki_read_cache_t_initialize, \
        iki_read_callback_t_initialize, \
        0, \
        fll_program_data_t_initialize, \
        iki_read_setting_t_initialize, \
      }
  #endif // iki_read_main_t_initialize
#endif // _di_iki_read_main_t_

/**
 * Delete the program cache data.
 *
 * @param cache
 *   The program cache data.
 *
 *   Must not be NULL.
 *
 *   This does not alter setting.state.status.
 *
 * @see f_memory_array_resize()
 * @see f_memory_arrays_resize()
 */
#ifndef _di_iki_read_setting_delete_
  extern void iki_read_cache_delete(iki_read_cache_t * const cache);
#endif // _di_iki_read_cache_delete_

/**
 * De-allocate main program data.
 *
 * @param main
 *   The main program data.
 *
 *   Must not be NULL.
 *
 *   This does not alter main.setting.state.status.
 *
 * @see fll_program_data_delete()
 * @see iki_read_setting_delete()
 */
#ifndef _di_iki_read_main_delete_
  extern void iki_read_main_delete(iki_read_main_t * const main);
#endif // _di_iki_read_main_delete_

/**
 * Delete the program main setting data.
 *
 * @param setting
 *   The program main setting data.
 *
 *   Must not be NULL.
 *
 *   This does not alter setting.state.status.
 *
 * @see f_iki_data_delete()
 * @see f_memory_array_resize()
 * @see f_memory_arrays_resize()
 */
#ifndef _di_iki_read_setting_delete_
  extern void iki_read_setting_delete(iki_read_setting_t * const setting);
#endif // _di_iki_read_setting_delete_

#ifdef __cplusplus
} // extern "C"
#endif

#endif // _iki_read_common_type_h
