#!/bin.bash
# license: lgpl-2.1-or-later
# programmer: Kevin Day
#
# Helper script for manually generating tests.
#
# This is a simple script that only accepts these arguments in this specific order:
#   1) The standard, using the 4-digit standard hexidecimal number (may also prepend "test-" to use 'fss_read -A').
#   2) The source file to read.
#   3) The destination directory to write to.
#

generate_main() {
  local file_source="${2}"
  local path_destination="${3}"
  local failure=0
  local test_base=
  local standard=
  local test_current=
  local test_set="${1}"
  local test_command=

  standard=$(echo -n ${test_set} | sed -e 's|^.*-||g')

  if [[ $standard == "" ]] ; then
    echo "ERROR: No standard specified."

    return 1
  fi

  if [[ $file_source == "" ]] ; then
    echo "ERROR: No source file specified."

    return 1
  fi

  if [[ $path_destination == "" ]] ; then
    echo "ERROR: No destination path specified."

    return 1
  fi

  file_source=$(echo ${file_source} | sed -e 's|^//*|/|' -e 's|//*|/|' -e 's|/*$||')
  path_destination=$(echo ${path_destination} | sed -e 's|^//*|/|' -e 's|/*$|/|')

  if [[ ! -f ${file_source} ]] ; then
    echo "ERROR: The source file '${file_source}' either does not exist or is not a file."

    return 1
  fi

  if [[ ! -d ${path_destination} ]] ; then
    echo "ERROR: The destination path '${path_destination}' either does not exist or is not a directory."

    return 1
  fi

  test_base=$(echo ${file_source} | sed -e 's|.*/||g' -e 's|\..*$||')
  test_current="${path_destination}${test_base}-"

  if [[ ${test_set} == "iki" || ${test_set} == "test-iki" ]] ; then
    test_command="iki"
  elif [[ ${test_set} == "eki" || ${test_set} == "test-eki" ]] ; then
    test_command="eki"
  else
    echo "ERROR: Unknown test set '${test_set}'."

    return 1
  fi

  # This follows the basic pattern of the FSS Read tests, but it does not need separate sets like FSS Read.
  generate_operate_iki

  if [[ ${failure} -eq 1 ]] ; then
    return 1
  fi

  return 0
}

generate_operate_iki() {
  local last_command=

  #### CONTENT ####
  generate_operate_run +n -c         ${file_source} > ${test_current}content.expect &&
  generate_operate_run +n -cl 0      ${file_source} > ${test_current}content-line-0.expect &&
  generate_operate_run +n -cl 1      ${file_source} > ${test_current}content-line-1.expect &&
  generate_operate_run +n -cl 2      ${file_source} > ${test_current}content-line-2.expect &&
  generate_operate_run +n -cl 3      ${file_source} > ${test_current}content-line-3.expect &&
  generate_operate_run +n -cl 4      ${file_source} > ${test_current}content-line-4.expect &&
  generate_operate_run +n -cl 5      ${file_source} > ${test_current}content-line-5.expect &&
  generate_operate_run +n -cl 6      ${file_source} > ${test_current}content-line-6.expect &&
  generate_operate_run +n -cl 7      ${file_source} > ${test_current}content-line-7.expect &&
  generate_operate_run +n -cl 8      ${file_source} > ${test_current}content-line-8.expect &&
  generate_operate_run +n -cl 9      ${file_source} > ${test_current}content-line-9.expect &&
  generate_operate_run +n -cl 10     ${file_source} > ${test_current}content-line-10.expect &&
  generate_operate_run +n -cl 20     ${file_source} > ${test_current}content-line-20.expect &&
  generate_operate_run +n -cl 100    ${file_source} > ${test_current}content-line-100.expect &&
  generate_operate_run +n -cla 0 0   ${file_source} > ${test_current}content-line-0-at-0.expect &&
  generate_operate_run +n -cla 1 0   ${file_source} > ${test_current}content-line-1-at-0.expect &&
  generate_operate_run +n -cla 2 0   ${file_source} > ${test_current}content-line-2-at-0.expect &&
  generate_operate_run +n -cla 3 0   ${file_source} > ${test_current}content-line-3-at-0.expect &&
  generate_operate_run +n -cla 4 0   ${file_source} > ${test_current}content-line-4-at-0.expect &&
  generate_operate_run +n -cla 5 0   ${file_source} > ${test_current}content-line-5-at-0.expect &&
  generate_operate_run +n -cla 6 0   ${file_source} > ${test_current}content-line-6-at-0.expect &&
  generate_operate_run +n -cla 7 0   ${file_source} > ${test_current}content-line-7-at-0.expect &&
  generate_operate_run +n -cla 8 0   ${file_source} > ${test_current}content-line-8-at-0.expect &&
  generate_operate_run +n -cla 9 0   ${file_source} > ${test_current}content-line-9-at-0.expect &&
  generate_operate_run +n -cla 10 0  ${file_source} > ${test_current}content-line-10-at-0.expect &&
  generate_operate_run +n -cla 20 0  ${file_source} > ${test_current}content-line-20-at-0.expect &&
  generate_operate_run +n -cla 100 0 ${file_source} > ${test_current}content-line-100-at-0.expect &&
  generate_operate_run +n -cla 0 1   ${file_source} > ${test_current}content-line-0-at-1.expect &&
  generate_operate_run +n -cla 1 1   ${file_source} > ${test_current}content-line-1-at-1.expect &&
  generate_operate_run +n -cla 2 1   ${file_source} > ${test_current}content-line-2-at-1.expect &&
  generate_operate_run +n -cla 3 1   ${file_source} > ${test_current}content-line-3-at-1.expect &&
  generate_operate_run +n -cla 4 1   ${file_source} > ${test_current}content-line-4-at-1.expect &&
  generate_operate_run +n -cla 5 1   ${file_source} > ${test_current}content-line-5-at-1.expect &&
  generate_operate_run +n -cla 6 1   ${file_source} > ${test_current}content-line-6-at-1.expect &&
  generate_operate_run +n -cla 7 1   ${file_source} > ${test_current}content-line-7-at-1.expect &&
  generate_operate_run +n -cla 8 1   ${file_source} > ${test_current}content-line-8-at-1.expect &&
  generate_operate_run +n -cla 9 1   ${file_source} > ${test_current}content-line-9-at-1.expect &&
  generate_operate_run +n -cla 10 1  ${file_source} > ${test_current}content-line-10-at-1.expect &&
  generate_operate_run +n -cla 20 1  ${file_source} > ${test_current}content-line-20-at-1.expect &&
  generate_operate_run +n -cla 100 1 ${file_source} > ${test_current}content-line-100-at-1.expect &&

  generate_operate_run +n -cn a         ${file_source} > ${test_current}content-name-a.expect &&
  generate_operate_run +n -cln 0 a      ${file_source} > ${test_current}content-line-0-name-a.expect &&
  generate_operate_run +n -cln 1 a      ${file_source} > ${test_current}content-line-1-name-a.expect &&
  generate_operate_run +n -cln 2 a      ${file_source} > ${test_current}content-line-2-name-a.expect &&
  generate_operate_run +n -cln 3 a      ${file_source} > ${test_current}content-line-3-name-a.expect &&
  generate_operate_run +n -cln 4 a      ${file_source} > ${test_current}content-line-4-name-a.expect &&
  generate_operate_run +n -cln 5 a      ${file_source} > ${test_current}content-line-5-name-a.expect &&
  generate_operate_run +n -cln 6 a      ${file_source} > ${test_current}content-line-6-name-a.expect &&
  generate_operate_run +n -cln 7 a      ${file_source} > ${test_current}content-line-7-name-a.expect &&
  generate_operate_run +n -cln 8 a      ${file_source} > ${test_current}content-line-8-name-a.expect &&
  generate_operate_run +n -cln 9 a      ${file_source} > ${test_current}content-line-9-name-a.expect &&
  generate_operate_run +n -cln 10 a     ${file_source} > ${test_current}content-line-10-name-a.expect &&
  generate_operate_run +n -cln 20 a     ${file_source} > ${test_current}content-line-20-name-a.expect &&
  generate_operate_run +n -cln 100 a    ${file_source} > ${test_current}content-line-100-name-a.expect &&
  generate_operate_run +n -clan 0 0 a   ${file_source} > ${test_current}content-line-0-at-0-name-a.expect &&
  generate_operate_run +n -clan 1 0 a   ${file_source} > ${test_current}content-line-1-at-0-name-a.expect &&
  generate_operate_run +n -clan 2 0 a   ${file_source} > ${test_current}content-line-2-at-0-name-a.expect &&
  generate_operate_run +n -clan 3 0 a   ${file_source} > ${test_current}content-line-3-at-0-name-a.expect &&
  generate_operate_run +n -clan 4 0 a   ${file_source} > ${test_current}content-line-4-at-0-name-a.expect &&
  generate_operate_run +n -clan 5 0 a   ${file_source} > ${test_current}content-line-5-at-0-name-a.expect &&
  generate_operate_run +n -clan 6 0 a   ${file_source} > ${test_current}content-line-6-at-0-name-a.expect &&
  generate_operate_run +n -clan 7 0 a   ${file_source} > ${test_current}content-line-7-at-0-name-a.expect &&
  generate_operate_run +n -clan 8 0 a   ${file_source} > ${test_current}content-line-8-at-0-name-a.expect &&
  generate_operate_run +n -clan 9 0 a   ${file_source} > ${test_current}content-line-9-at-0-name-a.expect &&
  generate_operate_run +n -clan 10 0 a  ${file_source} > ${test_current}content-line-10-at-0-name-a.expect &&
  generate_operate_run +n -clan 20 0 a  ${file_source} > ${test_current}content-line-20-at-0-name-a.expect &&
  generate_operate_run +n -clan 100 0 a ${file_source} > ${test_current}content-line-100-at-0-name-a.expect &&
  generate_operate_run +n -clan 0 1 a   ${file_source} > ${test_current}content-line-0-at-1-name-a.expect &&
  generate_operate_run +n -clan 1 1 a   ${file_source} > ${test_current}content-line-1-at-1-name-a.expect &&
  generate_operate_run +n -clan 2 1 a   ${file_source} > ${test_current}content-line-2-at-1-name-a.expect &&
  generate_operate_run +n -clan 3 1 a   ${file_source} > ${test_current}content-line-3-at-1-name-a.expect &&
  generate_operate_run +n -clan 4 1 a   ${file_source} > ${test_current}content-line-4-at-1-name-a.expect &&
  generate_operate_run +n -clan 5 1 a   ${file_source} > ${test_current}content-line-5-at-1-name-a.expect &&
  generate_operate_run +n -clan 6 1 a   ${file_source} > ${test_current}content-line-6-at-1-name-a.expect &&
  generate_operate_run +n -clan 7 1 a   ${file_source} > ${test_current}content-line-7-at-1-name-a.expect &&
  generate_operate_run +n -clan 8 1 a   ${file_source} > ${test_current}content-line-8-at-1-name-a.expect &&
  generate_operate_run +n -clan 9 1 a   ${file_source} > ${test_current}content-line-9-at-1-name-a.expect &&
  generate_operate_run +n -clan 10 1 a  ${file_source} > ${test_current}content-line-10-at-1-name-a.expect &&
  generate_operate_run +n -clan 20 1 a  ${file_source} > ${test_current}content-line-20-at-1-name-a.expect &&
  generate_operate_run +n -clan 100 1 a ${file_source} > ${test_current}content-line-100-at-1-name-a.expect &&

  generate_operate_run +n -cn b         ${file_source} > ${test_current}content-name-b.expect &&
  generate_operate_run +n -cln 0 b      ${file_source} > ${test_current}content-line-0-name-b.expect &&
  generate_operate_run +n -cln 1 b      ${file_source} > ${test_current}content-line-1-name-b.expect &&
  generate_operate_run +n -cln 2 b      ${file_source} > ${test_current}content-line-2-name-b.expect &&
  generate_operate_run +n -cln 3 b      ${file_source} > ${test_current}content-line-3-name-b.expect &&
  generate_operate_run +n -cln 4 b      ${file_source} > ${test_current}content-line-4-name-b.expect &&
  generate_operate_run +n -cln 5 b      ${file_source} > ${test_current}content-line-5-name-b.expect &&
  generate_operate_run +n -cln 6 b      ${file_source} > ${test_current}content-line-6-name-b.expect &&
  generate_operate_run +n -cln 7 b      ${file_source} > ${test_current}content-line-7-name-b.expect &&
  generate_operate_run +n -cln 8 b      ${file_source} > ${test_current}content-line-8-name-b.expect &&
  generate_operate_run +n -cln 9 b      ${file_source} > ${test_current}content-line-9-name-b.expect &&
  generate_operate_run +n -cln 10 b     ${file_source} > ${test_current}content-line-10-name-b.expect &&
  generate_operate_run +n -cln 20 b     ${file_source} > ${test_current}content-line-20-name-b.expect &&
  generate_operate_run +n -cln 100 b    ${file_source} > ${test_current}content-line-100-name-b.expect &&
  generate_operate_run +n -clan 0 0 b   ${file_source} > ${test_current}content-line-0-at-0-name-b.expect &&
  generate_operate_run +n -clan 1 0 b   ${file_source} > ${test_current}content-line-1-at-0-name-b.expect &&
  generate_operate_run +n -clan 2 0 b   ${file_source} > ${test_current}content-line-2-at-0-name-b.expect &&
  generate_operate_run +n -clan 3 0 b   ${file_source} > ${test_current}content-line-3-at-0-name-b.expect &&
  generate_operate_run +n -clan 4 0 b   ${file_source} > ${test_current}content-line-4-at-0-name-b.expect &&
  generate_operate_run +n -clan 5 0 b   ${file_source} > ${test_current}content-line-5-at-0-name-b.expect &&
  generate_operate_run +n -clan 6 0 b   ${file_source} > ${test_current}content-line-6-at-0-name-b.expect &&
  generate_operate_run +n -clan 7 0 b   ${file_source} > ${test_current}content-line-7-at-0-name-b.expect &&
  generate_operate_run +n -clan 8 0 b   ${file_source} > ${test_current}content-line-8-at-0-name-b.expect &&
  generate_operate_run +n -clan 9 0 b   ${file_source} > ${test_current}content-line-9-at-0-name-b.expect &&
  generate_operate_run +n -clan 10 0 b  ${file_source} > ${test_current}content-line-10-at-0-name-b.expect &&
  generate_operate_run +n -clan 20 0 b  ${file_source} > ${test_current}content-line-20-at-0-name-b.expect &&
  generate_operate_run +n -clan 100 0 b ${file_source} > ${test_current}content-line-100-at-0-name-b.expect &&
  generate_operate_run +n -clan 0 1 b   ${file_source} > ${test_current}content-line-0-at-1-name-b.expect &&
  generate_operate_run +n -clan 1 1 b   ${file_source} > ${test_current}content-line-1-at-1-name-b.expect &&
  generate_operate_run +n -clan 2 1 b   ${file_source} > ${test_current}content-line-2-at-1-name-b.expect &&
  generate_operate_run +n -clan 3 1 b   ${file_source} > ${test_current}content-line-3-at-1-name-b.expect &&
  generate_operate_run +n -clan 4 1 b   ${file_source} > ${test_current}content-line-4-at-1-name-b.expect &&
  generate_operate_run +n -clan 5 1 b   ${file_source} > ${test_current}content-line-5-at-1-name-b.expect &&
  generate_operate_run +n -clan 6 1 b   ${file_source} > ${test_current}content-line-6-at-1-name-b.expect &&
  generate_operate_run +n -clan 7 1 b   ${file_source} > ${test_current}content-line-7-at-1-name-b.expect &&
  generate_operate_run +n -clan 8 1 b   ${file_source} > ${test_current}content-line-8-at-1-name-b.expect &&
  generate_operate_run +n -clan 9 1 b   ${file_source} > ${test_current}content-line-9-at-1-name-b.expect &&
  generate_operate_run +n -clan 10 1 b  ${file_source} > ${test_current}content-line-10-at-1-name-b.expect &&
  generate_operate_run +n -clan 20 1 b  ${file_source} > ${test_current}content-line-20-at-1-name-b.expect &&
  generate_operate_run +n -clan 100 1 b ${file_source} > ${test_current}content-line-100-at-1-name-b.expect &&

  generate_operate_run +n -cn is         ${file_source} > ${test_current}content-name-is.expect &&
  generate_operate_run +n -cln 0 is      ${file_source} > ${test_current}content-line-0-name-is.expect &&
  generate_operate_run +n -cln 1 is      ${file_source} > ${test_current}content-line-1-name-is.expect &&
  generate_operate_run +n -cln 2 is      ${file_source} > ${test_current}content-line-2-name-is.expect &&
  generate_operate_run +n -cln 3 is      ${file_source} > ${test_current}content-line-3-name-is.expect &&
  generate_operate_run +n -cln 4 is      ${file_source} > ${test_current}content-line-4-name-is.expect &&
  generate_operate_run +n -cln 5 is      ${file_source} > ${test_current}content-line-5-name-is.expect &&
  generate_operate_run +n -cln 6 is      ${file_source} > ${test_current}content-line-6-name-is.expect &&
  generate_operate_run +n -cln 7 is      ${file_source} > ${test_current}content-line-7-name-is.expect &&
  generate_operate_run +n -cln 8 is      ${file_source} > ${test_current}content-line-8-name-is.expect &&
  generate_operate_run +n -cln 9 is      ${file_source} > ${test_current}content-line-9-name-is.expect &&
  generate_operate_run +n -cln 10 is     ${file_source} > ${test_current}content-line-10-name-is.expect &&
  generate_operate_run +n -cln 20 is     ${file_source} > ${test_current}content-line-20-name-is.expect &&
  generate_operate_run +n -cln 100 is    ${file_source} > ${test_current}content-line-100-name-is.expect &&
  generate_operate_run +n -clan 0 0 is   ${file_source} > ${test_current}content-line-0-at-0-name-is.expect &&
  generate_operate_run +n -clan 1 0 is   ${file_source} > ${test_current}content-line-1-at-0-name-is.expect &&
  generate_operate_run +n -clan 2 0 is   ${file_source} > ${test_current}content-line-2-at-0-name-is.expect &&
  generate_operate_run +n -clan 3 0 is   ${file_source} > ${test_current}content-line-3-at-0-name-is.expect &&
  generate_operate_run +n -clan 4 0 is   ${file_source} > ${test_current}content-line-4-at-0-name-is.expect &&
  generate_operate_run +n -clan 5 0 is   ${file_source} > ${test_current}content-line-5-at-0-name-is.expect &&
  generate_operate_run +n -clan 6 0 is   ${file_source} > ${test_current}content-line-6-at-0-name-is.expect &&
  generate_operate_run +n -clan 7 0 is   ${file_source} > ${test_current}content-line-7-at-0-name-is.expect &&
  generate_operate_run +n -clan 8 0 is   ${file_source} > ${test_current}content-line-8-at-0-name-is.expect &&
  generate_operate_run +n -clan 9 0 is   ${file_source} > ${test_current}content-line-9-at-0-name-is.expect &&
  generate_operate_run +n -clan 10 0 is  ${file_source} > ${test_current}content-line-10-at-0-name-is.expect &&
  generate_operate_run +n -clan 20 0 is  ${file_source} > ${test_current}content-line-20-at-0-name-is.expect &&
  generate_operate_run +n -clan 100 0 is ${file_source} > ${test_current}content-line-100-at-0-name-is.expect &&
  generate_operate_run +n -clan 0 1 is   ${file_source} > ${test_current}content-line-0-at-1-name-is.expect &&
  generate_operate_run +n -clan 1 1 is   ${file_source} > ${test_current}content-line-1-at-1-name-is.expect &&
  generate_operate_run +n -clan 2 1 is   ${file_source} > ${test_current}content-line-2-at-1-name-is.expect &&
  generate_operate_run +n -clan 3 1 is   ${file_source} > ${test_current}content-line-3-at-1-name-is.expect &&
  generate_operate_run +n -clan 4 1 is   ${file_source} > ${test_current}content-line-4-at-1-name-is.expect &&
  generate_operate_run +n -clan 5 1 is   ${file_source} > ${test_current}content-line-5-at-1-name-is.expect &&
  generate_operate_run +n -clan 6 1 is   ${file_source} > ${test_current}content-line-6-at-1-name-is.expect &&
  generate_operate_run +n -clan 7 1 is   ${file_source} > ${test_current}content-line-7-at-1-name-is.expect &&
  generate_operate_run +n -clan 8 1 is   ${file_source} > ${test_current}content-line-8-at-1-name-is.expect &&
  generate_operate_run +n -clan 9 1 is   ${file_source} > ${test_current}content-line-9-at-1-name-is.expect &&
  generate_operate_run +n -clan 10 1 is  ${file_source} > ${test_current}content-line-10-at-1-name-is.expect &&
  generate_operate_run +n -clan 20 1 is  ${file_source} > ${test_current}content-line-20-at-1-name-is.expect &&
  generate_operate_run +n -clan 100 1 is ${file_source} > ${test_current}content-line-100-at-1-name-is.expect &&

  generate_operate_run +n -cn yup         ${file_source} > ${test_current}content-name-yup.expect &&
  generate_operate_run +n -cln 0 yup      ${file_source} > ${test_current}content-line-0-name-yup.expect &&
  generate_operate_run +n -cln 1 yup      ${file_source} > ${test_current}content-line-1-name-yup.expect &&
  generate_operate_run +n -cln 2 yup      ${file_source} > ${test_current}content-line-2-name-yup.expect &&
  generate_operate_run +n -cln 3 yup      ${file_source} > ${test_current}content-line-3-name-yup.expect &&
  generate_operate_run +n -cln 4 yup      ${file_source} > ${test_current}content-line-4-name-yup.expect &&
  generate_operate_run +n -cln 5 yup      ${file_source} > ${test_current}content-line-5-name-yup.expect &&
  generate_operate_run +n -cln 6 yup      ${file_source} > ${test_current}content-line-6-name-yup.expect &&
  generate_operate_run +n -cln 7 yup      ${file_source} > ${test_current}content-line-7-name-yup.expect &&
  generate_operate_run +n -cln 8 yup      ${file_source} > ${test_current}content-line-8-name-yup.expect &&
  generate_operate_run +n -cln 9 yup      ${file_source} > ${test_current}content-line-9-name-yup.expect &&
  generate_operate_run +n -cln 10 yup     ${file_source} > ${test_current}content-line-10-name-yup.expect &&
  generate_operate_run +n -cln 20 yup     ${file_source} > ${test_current}content-line-20-name-yup.expect &&
  generate_operate_run +n -cln 100 yup    ${file_source} > ${test_current}content-line-100-name-yup.expect &&
  generate_operate_run +n -clan 0 0 yup   ${file_source} > ${test_current}content-line-0-at-0-name-yup.expect &&
  generate_operate_run +n -clan 1 0 yup   ${file_source} > ${test_current}content-line-1-at-0-name-yup.expect &&
  generate_operate_run +n -clan 2 0 yup   ${file_source} > ${test_current}content-line-2-at-0-name-yup.expect &&
  generate_operate_run +n -clan 3 0 yup   ${file_source} > ${test_current}content-line-3-at-0-name-yup.expect &&
  generate_operate_run +n -clan 4 0 yup   ${file_source} > ${test_current}content-line-4-at-0-name-yup.expect &&
  generate_operate_run +n -clan 5 0 yup   ${file_source} > ${test_current}content-line-5-at-0-name-yup.expect &&
  generate_operate_run +n -clan 6 0 yup   ${file_source} > ${test_current}content-line-6-at-0-name-yup.expect &&
  generate_operate_run +n -clan 7 0 yup   ${file_source} > ${test_current}content-line-7-at-0-name-yup.expect &&
  generate_operate_run +n -clan 8 0 yup   ${file_source} > ${test_current}content-line-8-at-0-name-yup.expect &&
  generate_operate_run +n -clan 9 0 yup   ${file_source} > ${test_current}content-line-9-at-0-name-yup.expect &&
  generate_operate_run +n -clan 10 0 yup  ${file_source} > ${test_current}content-line-10-at-0-name-yup.expect &&
  generate_operate_run +n -clan 20 0 yup  ${file_source} > ${test_current}content-line-20-at-0-name-yup.expect &&
  generate_operate_run +n -clan 100 0 yup ${file_source} > ${test_current}content-line-100-at-0-name-yup.expect &&
  generate_operate_run +n -clan 0 1 yup   ${file_source} > ${test_current}content-line-0-at-1-name-yup.expect &&
  generate_operate_run +n -clan 1 1 yup   ${file_source} > ${test_current}content-line-1-at-1-name-yup.expect &&
  generate_operate_run +n -clan 2 1 yup   ${file_source} > ${test_current}content-line-2-at-1-name-yup.expect &&
  generate_operate_run +n -clan 3 1 yup   ${file_source} > ${test_current}content-line-3-at-1-name-yup.expect &&
  generate_operate_run +n -clan 4 1 yup   ${file_source} > ${test_current}content-line-4-at-1-name-yup.expect &&
  generate_operate_run +n -clan 5 1 yup   ${file_source} > ${test_current}content-line-5-at-1-name-yup.expect &&
  generate_operate_run +n -clan 6 1 yup   ${file_source} > ${test_current}content-line-6-at-1-name-yup.expect &&
  generate_operate_run +n -clan 7 1 yup   ${file_source} > ${test_current}content-line-7-at-1-name-yup.expect &&
  generate_operate_run +n -clan 8 1 yup   ${file_source} > ${test_current}content-line-8-at-1-name-yup.expect &&
  generate_operate_run +n -clan 9 1 yup   ${file_source} > ${test_current}content-line-9-at-1-name-yup.expect &&
  generate_operate_run +n -clan 10 1 yup  ${file_source} > ${test_current}content-line-10-at-1-name-yup.expect &&
  generate_operate_run +n -clan 20 1 yup  ${file_source} > ${test_current}content-line-20-at-1-name-yup.expect &&

  #### CONTENT - TOTAL ####
  generate_operate_run +n -ct         ${file_source} > ${test_current}content-total.expect &&
  generate_operate_run +n -clt 0      ${file_source} > ${test_current}content-line-0-total.expect &&
  generate_operate_run +n -clt 1      ${file_source} > ${test_current}content-line-1-total.expect &&
  generate_operate_run +n -clt 2      ${file_source} > ${test_current}content-line-2-total.expect &&
  generate_operate_run +n -clt 3      ${file_source} > ${test_current}content-line-3-total.expect &&
  generate_operate_run +n -clt 4      ${file_source} > ${test_current}content-line-4-total.expect &&
  generate_operate_run +n -clt 5      ${file_source} > ${test_current}content-line-5-total.expect &&
  generate_operate_run +n -clt 6      ${file_source} > ${test_current}content-line-6-total.expect &&
  generate_operate_run +n -clt 7      ${file_source} > ${test_current}content-line-7-total.expect &&
  generate_operate_run +n -clt 8      ${file_source} > ${test_current}content-line-8-total.expect &&
  generate_operate_run +n -clt 9      ${file_source} > ${test_current}content-line-9-total.expect &&
  generate_operate_run +n -clt 10     ${file_source} > ${test_current}content-line-10-total.expect &&
  generate_operate_run +n -clt 20     ${file_source} > ${test_current}content-line-20-total.expect &&
  generate_operate_run +n -clt 100    ${file_source} > ${test_current}content-line-100-total.expect &&
  generate_operate_run +n -clat 0 0   ${file_source} > ${test_current}content-line-0-at-0-total.expect &&
  generate_operate_run +n -clat 1 0   ${file_source} > ${test_current}content-line-1-at-0-total.expect &&
  generate_operate_run +n -clat 2 0   ${file_source} > ${test_current}content-line-2-at-0-total.expect &&
  generate_operate_run +n -clat 3 0   ${file_source} > ${test_current}content-line-3-at-0-total.expect &&
  generate_operate_run +n -clat 4 0   ${file_source} > ${test_current}content-line-4-at-0-total.expect &&
  generate_operate_run +n -clat 5 0   ${file_source} > ${test_current}content-line-5-at-0-total.expect &&
  generate_operate_run +n -clat 6 0   ${file_source} > ${test_current}content-line-6-at-0-total.expect &&
  generate_operate_run +n -clat 7 0   ${file_source} > ${test_current}content-line-7-at-0-total.expect &&
  generate_operate_run +n -clat 8 0   ${file_source} > ${test_current}content-line-8-at-0-total.expect &&
  generate_operate_run +n -clat 9 0   ${file_source} > ${test_current}content-line-9-at-0-total.expect &&
  generate_operate_run +n -clat 10 0  ${file_source} > ${test_current}content-line-10-at-0-total.expect &&
  generate_operate_run +n -clat 20 0  ${file_source} > ${test_current}content-line-20-at-0-total.expect &&
  generate_operate_run +n -clat 100 0 ${file_source} > ${test_current}content-line-100-at-0-total.expect &&
  generate_operate_run +n -clat 0 1   ${file_source} > ${test_current}content-line-0-at-1-total.expect &&
  generate_operate_run +n -clat 1 1   ${file_source} > ${test_current}content-line-1-at-1-total.expect &&
  generate_operate_run +n -clat 2 1   ${file_source} > ${test_current}content-line-2-at-1-total.expect &&
  generate_operate_run +n -clat 3 1   ${file_source} > ${test_current}content-line-3-at-1-total.expect &&
  generate_operate_run +n -clat 4 1   ${file_source} > ${test_current}content-line-4-at-1-total.expect &&
  generate_operate_run +n -clat 5 1   ${file_source} > ${test_current}content-line-5-at-1-total.expect &&
  generate_operate_run +n -clat 6 1   ${file_source} > ${test_current}content-line-6-at-1-total.expect &&
  generate_operate_run +n -clat 7 1   ${file_source} > ${test_current}content-line-7-at-1-total.expect &&
  generate_operate_run +n -clat 8 1   ${file_source} > ${test_current}content-line-8-at-1-total.expect &&
  generate_operate_run +n -clat 9 1   ${file_source} > ${test_current}content-line-9-at-1-total.expect &&
  generate_operate_run +n -clat 10 1  ${file_source} > ${test_current}content-line-10-at-1-total.expect &&
  generate_operate_run +n -clat 20 1  ${file_source} > ${test_current}content-line-20-at-1-total.expect &&
  generate_operate_run +n -clat 100 1 ${file_source} > ${test_current}content-line-100-at-1-total.expect &&

  generate_operate_run +n -cnt a         ${file_source} > ${test_current}content-name-a-total.expect &&
  generate_operate_run +n -clnt 0 a      ${file_source} > ${test_current}content-line-0-name-a-total.expect &&
  generate_operate_run +n -clnt 1 a      ${file_source} > ${test_current}content-line-1-name-a-total.expect &&
  generate_operate_run +n -clnt 2 a      ${file_source} > ${test_current}content-line-2-name-a-total.expect &&
  generate_operate_run +n -clnt 3 a      ${file_source} > ${test_current}content-line-3-name-a-total.expect &&
  generate_operate_run +n -clnt 4 a      ${file_source} > ${test_current}content-line-4-name-a-total.expect &&
  generate_operate_run +n -clnt 5 a      ${file_source} > ${test_current}content-line-5-name-a-total.expect &&
  generate_operate_run +n -clnt 6 a      ${file_source} > ${test_current}content-line-6-name-a-total.expect &&
  generate_operate_run +n -clnt 7 a      ${file_source} > ${test_current}content-line-7-name-a-total.expect &&
  generate_operate_run +n -clnt 8 a      ${file_source} > ${test_current}content-line-8-name-a-total.expect &&
  generate_operate_run +n -clnt 9 a      ${file_source} > ${test_current}content-line-9-name-a-total.expect &&
  generate_operate_run +n -clnt 10 a     ${file_source} > ${test_current}content-line-10-name-a-total.expect &&
  generate_operate_run +n -clnt 20 a     ${file_source} > ${test_current}content-line-20-name-a-total.expect &&
  generate_operate_run +n -clnt 100 a    ${file_source} > ${test_current}content-line-100-name-a-total.expect &&
  generate_operate_run +n -clant 0 0 a   ${file_source} > ${test_current}content-line-0-at-0-name-a-total.expect &&
  generate_operate_run +n -clant 1 0 a   ${file_source} > ${test_current}content-line-1-at-0-name-a-total.expect &&
  generate_operate_run +n -clant 2 0 a   ${file_source} > ${test_current}content-line-2-at-0-name-a-total.expect &&
  generate_operate_run +n -clant 3 0 a   ${file_source} > ${test_current}content-line-3-at-0-name-a-total.expect &&
  generate_operate_run +n -clant 4 0 a   ${file_source} > ${test_current}content-line-4-at-0-name-a-total.expect &&
  generate_operate_run +n -clant 5 0 a   ${file_source} > ${test_current}content-line-5-at-0-name-a-total.expect &&
  generate_operate_run +n -clant 6 0 a   ${file_source} > ${test_current}content-line-6-at-0-name-a-total.expect &&
  generate_operate_run +n -clant 7 0 a   ${file_source} > ${test_current}content-line-7-at-0-name-a-total.expect &&
  generate_operate_run +n -clant 8 0 a   ${file_source} > ${test_current}content-line-8-at-0-name-a-total.expect &&
  generate_operate_run +n -clant 9 0 a   ${file_source} > ${test_current}content-line-9-at-0-name-a-total.expect &&
  generate_operate_run +n -clant 10 0 a  ${file_source} > ${test_current}content-line-10-at-0-name-a-total.expect &&
  generate_operate_run +n -clant 20 0 a  ${file_source} > ${test_current}content-line-20-at-0-name-a-total.expect &&
  generate_operate_run +n -clant 100 0 a ${file_source} > ${test_current}content-line-100-at-0-name-a-total.expect &&
  generate_operate_run +n -clant 0 1 a   ${file_source} > ${test_current}content-line-0-at-1-name-a-total.expect &&
  generate_operate_run +n -clant 1 1 a   ${file_source} > ${test_current}content-line-1-at-1-name-a-total.expect &&
  generate_operate_run +n -clant 2 1 a   ${file_source} > ${test_current}content-line-2-at-1-name-a-total.expect &&
  generate_operate_run +n -clant 3 1 a   ${file_source} > ${test_current}content-line-3-at-1-name-a-total.expect &&
  generate_operate_run +n -clant 4 1 a   ${file_source} > ${test_current}content-line-4-at-1-name-a-total.expect &&
  generate_operate_run +n -clant 5 1 a   ${file_source} > ${test_current}content-line-5-at-1-name-a-total.expect &&
  generate_operate_run +n -clant 6 1 a   ${file_source} > ${test_current}content-line-6-at-1-name-a-total.expect &&
  generate_operate_run +n -clant 7 1 a   ${file_source} > ${test_current}content-line-7-at-1-name-a-total.expect &&
  generate_operate_run +n -clant 8 1 a   ${file_source} > ${test_current}content-line-8-at-1-name-a-total.expect &&
  generate_operate_run +n -clant 9 1 a   ${file_source} > ${test_current}content-line-9-at-1-name-a-total.expect &&
  generate_operate_run +n -clant 10 1 a  ${file_source} > ${test_current}content-line-10-at-1-name-a-total.expect &&
  generate_operate_run +n -clant 20 1 a  ${file_source} > ${test_current}content-line-20-at-1-name-a-total.expect &&
  generate_operate_run +n -clant 100 1 a ${file_source} > ${test_current}content-line-100-at-1-name-a-total.expect &&

  generate_operate_run +n -cnt b         ${file_source} > ${test_current}content-name-b-total.expect &&
  generate_operate_run +n -clnt 0 b      ${file_source} > ${test_current}content-line-0-name-b-total.expect &&
  generate_operate_run +n -clnt 1 b      ${file_source} > ${test_current}content-line-1-name-b-total.expect &&
  generate_operate_run +n -clnt 2 b      ${file_source} > ${test_current}content-line-2-name-b-total.expect &&
  generate_operate_run +n -clnt 3 b      ${file_source} > ${test_current}content-line-3-name-b-total.expect &&
  generate_operate_run +n -clnt 4 b      ${file_source} > ${test_current}content-line-4-name-b-total.expect &&
  generate_operate_run +n -clnt 5 b      ${file_source} > ${test_current}content-line-5-name-b-total.expect &&
  generate_operate_run +n -clnt 6 b      ${file_source} > ${test_current}content-line-6-name-b-total.expect &&
  generate_operate_run +n -clnt 7 b      ${file_source} > ${test_current}content-line-7-name-b-total.expect &&
  generate_operate_run +n -clnt 8 b      ${file_source} > ${test_current}content-line-8-name-b-total.expect &&
  generate_operate_run +n -clnt 9 b      ${file_source} > ${test_current}content-line-9-name-b-total.expect &&
  generate_operate_run +n -clnt 10 b     ${file_source} > ${test_current}content-line-10-name-b-total.expect &&
  generate_operate_run +n -clnt 20 b     ${file_source} > ${test_current}content-line-20-name-b-total.expect &&
  generate_operate_run +n -clnt 100 b    ${file_source} > ${test_current}content-line-100-name-b-total.expect &&
  generate_operate_run +n -clant 0 0 b   ${file_source} > ${test_current}content-line-0-at-0-name-b-total.expect &&
  generate_operate_run +n -clant 1 0 b   ${file_source} > ${test_current}content-line-1-at-0-name-b-total.expect &&
  generate_operate_run +n -clant 2 0 b   ${file_source} > ${test_current}content-line-2-at-0-name-b-total.expect &&
  generate_operate_run +n -clant 3 0 b   ${file_source} > ${test_current}content-line-3-at-0-name-b-total.expect &&
  generate_operate_run +n -clant 4 0 b   ${file_source} > ${test_current}content-line-4-at-0-name-b-total.expect &&
  generate_operate_run +n -clant 5 0 b   ${file_source} > ${test_current}content-line-5-at-0-name-b-total.expect &&
  generate_operate_run +n -clant 6 0 b   ${file_source} > ${test_current}content-line-6-at-0-name-b-total.expect &&
  generate_operate_run +n -clant 7 0 b   ${file_source} > ${test_current}content-line-7-at-0-name-b-total.expect &&
  generate_operate_run +n -clant 8 0 b   ${file_source} > ${test_current}content-line-8-at-0-name-b-total.expect &&
  generate_operate_run +n -clant 9 0 b   ${file_source} > ${test_current}content-line-9-at-0-name-b-total.expect &&
  generate_operate_run +n -clant 10 0 b  ${file_source} > ${test_current}content-line-10-at-0-name-b-total.expect &&
  generate_operate_run +n -clant 20 0 b  ${file_source} > ${test_current}content-line-20-at-0-name-b-total.expect &&
  generate_operate_run +n -clant 100 0 b ${file_source} > ${test_current}content-line-100-at-0-name-b-total.expect &&
  generate_operate_run +n -clant 0 1 b   ${file_source} > ${test_current}content-line-0-at-1-name-b-total.expect &&
  generate_operate_run +n -clant 1 1 b   ${file_source} > ${test_current}content-line-1-at-1-name-b-total.expect &&
  generate_operate_run +n -clant 2 1 b   ${file_source} > ${test_current}content-line-2-at-1-name-b-total.expect &&
  generate_operate_run +n -clant 3 1 b   ${file_source} > ${test_current}content-line-3-at-1-name-b-total.expect &&
  generate_operate_run +n -clant 4 1 b   ${file_source} > ${test_current}content-line-4-at-1-name-b-total.expect &&
  generate_operate_run +n -clant 5 1 b   ${file_source} > ${test_current}content-line-5-at-1-name-b-total.expect &&
  generate_operate_run +n -clant 6 1 b   ${file_source} > ${test_current}content-line-6-at-1-name-b-total.expect &&
  generate_operate_run +n -clant 7 1 b   ${file_source} > ${test_current}content-line-7-at-1-name-b-total.expect &&
  generate_operate_run +n -clant 8 1 b   ${file_source} > ${test_current}content-line-8-at-1-name-b-total.expect &&
  generate_operate_run +n -clant 9 1 b   ${file_source} > ${test_current}content-line-9-at-1-name-b-total.expect &&
  generate_operate_run +n -clant 10 1 b  ${file_source} > ${test_current}content-line-10-at-1-name-b-total.expect &&
  generate_operate_run +n -clant 20 1 b  ${file_source} > ${test_current}content-line-20-at-1-name-b-total.expect &&
  generate_operate_run +n -clant 100 1 b ${file_source} > ${test_current}content-line-100-at-1-name-b-total.expect &&

  generate_operate_run +n -cnt is         ${file_source} > ${test_current}content-name-is-total.expect &&
  generate_operate_run +n -clnt 0 is      ${file_source} > ${test_current}content-line-0-name-is-total.expect &&
  generate_operate_run +n -clnt 1 is      ${file_source} > ${test_current}content-line-1-name-is-total.expect &&
  generate_operate_run +n -clnt 2 is      ${file_source} > ${test_current}content-line-2-name-is-total.expect &&
  generate_operate_run +n -clnt 3 is      ${file_source} > ${test_current}content-line-3-name-is-total.expect &&
  generate_operate_run +n -clnt 4 is      ${file_source} > ${test_current}content-line-4-name-is-total.expect &&
  generate_operate_run +n -clnt 5 is      ${file_source} > ${test_current}content-line-5-name-is-total.expect &&
  generate_operate_run +n -clnt 6 is      ${file_source} > ${test_current}content-line-6-name-is-total.expect &&
  generate_operate_run +n -clnt 7 is      ${file_source} > ${test_current}content-line-7-name-is-total.expect &&
  generate_operate_run +n -clnt 8 is      ${file_source} > ${test_current}content-line-8-name-is-total.expect &&
  generate_operate_run +n -clnt 9 is      ${file_source} > ${test_current}content-line-9-name-is-total.expect &&
  generate_operate_run +n -clnt 10 is     ${file_source} > ${test_current}content-line-10-name-is-total.expect &&
  generate_operate_run +n -clnt 20 is     ${file_source} > ${test_current}content-line-20-name-is-total.expect &&
  generate_operate_run +n -clnt 100 is    ${file_source} > ${test_current}content-line-100-name-is-total.expect &&
  generate_operate_run +n -clant 0 0 is   ${file_source} > ${test_current}content-line-0-at-0-name-is-total.expect &&
  generate_operate_run +n -clant 1 0 is   ${file_source} > ${test_current}content-line-1-at-0-name-is-total.expect &&
  generate_operate_run +n -clant 2 0 is   ${file_source} > ${test_current}content-line-2-at-0-name-is-total.expect &&
  generate_operate_run +n -clant 3 0 is   ${file_source} > ${test_current}content-line-3-at-0-name-is-total.expect &&
  generate_operate_run +n -clant 4 0 is   ${file_source} > ${test_current}content-line-4-at-0-name-is-total.expect &&
  generate_operate_run +n -clant 5 0 is   ${file_source} > ${test_current}content-line-5-at-0-name-is-total.expect &&
  generate_operate_run +n -clant 6 0 is   ${file_source} > ${test_current}content-line-6-at-0-name-is-total.expect &&
  generate_operate_run +n -clant 7 0 is   ${file_source} > ${test_current}content-line-7-at-0-name-is-total.expect &&
  generate_operate_run +n -clant 8 0 is   ${file_source} > ${test_current}content-line-8-at-0-name-is-total.expect &&
  generate_operate_run +n -clant 9 0 is   ${file_source} > ${test_current}content-line-9-at-0-name-is-total.expect &&
  generate_operate_run +n -clant 10 0 is  ${file_source} > ${test_current}content-line-10-at-0-name-is-total.expect &&
  generate_operate_run +n -clant 20 0 is  ${file_source} > ${test_current}content-line-20-at-0-name-is-total.expect &&
  generate_operate_run +n -clant 100 0 is ${file_source} > ${test_current}content-line-100-at-0-name-is-total.expect &&
  generate_operate_run +n -clant 0 1 is   ${file_source} > ${test_current}content-line-0-at-1-name-is-total.expect &&
  generate_operate_run +n -clant 1 1 is   ${file_source} > ${test_current}content-line-1-at-1-name-is-total.expect &&
  generate_operate_run +n -clant 2 1 is   ${file_source} > ${test_current}content-line-2-at-1-name-is-total.expect &&
  generate_operate_run +n -clant 3 1 is   ${file_source} > ${test_current}content-line-3-at-1-name-is-total.expect &&
  generate_operate_run +n -clant 4 1 is   ${file_source} > ${test_current}content-line-4-at-1-name-is-total.expect &&
  generate_operate_run +n -clant 5 1 is   ${file_source} > ${test_current}content-line-5-at-1-name-is-total.expect &&
  generate_operate_run +n -clant 6 1 is   ${file_source} > ${test_current}content-line-6-at-1-name-is-total.expect &&
  generate_operate_run +n -clant 7 1 is   ${file_source} > ${test_current}content-line-7-at-1-name-is-total.expect &&
  generate_operate_run +n -clant 8 1 is   ${file_source} > ${test_current}content-line-8-at-1-name-is-total.expect &&
  generate_operate_run +n -clant 9 1 is   ${file_source} > ${test_current}content-line-9-at-1-name-is-total.expect &&
  generate_operate_run +n -clant 10 1 is  ${file_source} > ${test_current}content-line-10-at-1-name-is-total.expect &&
  generate_operate_run +n -clant 20 1 is  ${file_source} > ${test_current}content-line-20-at-1-name-is-total.expect &&
  generate_operate_run +n -clant 100 1 is ${file_source} > ${test_current}content-line-100-at-1-name-is-total.expect &&

  generate_operate_run +n -cnt yup         ${file_source} > ${test_current}content-name-yup-total.expect &&
  generate_operate_run +n -clnt 0 yup      ${file_source} > ${test_current}content-line-0-name-yup-total.expect &&
  generate_operate_run +n -clnt 1 yup      ${file_source} > ${test_current}content-line-1-name-yup-total.expect &&
  generate_operate_run +n -clnt 2 yup      ${file_source} > ${test_current}content-line-2-name-yup-total.expect &&
  generate_operate_run +n -clnt 3 yup      ${file_source} > ${test_current}content-line-3-name-yup-total.expect &&
  generate_operate_run +n -clnt 4 yup      ${file_source} > ${test_current}content-line-4-name-yup-total.expect &&
  generate_operate_run +n -clnt 5 yup      ${file_source} > ${test_current}content-line-5-name-yup-total.expect &&
  generate_operate_run +n -clnt 6 yup      ${file_source} > ${test_current}content-line-6-name-yup-total.expect &&
  generate_operate_run +n -clnt 7 yup      ${file_source} > ${test_current}content-line-7-name-yup-total.expect &&
  generate_operate_run +n -clnt 8 yup      ${file_source} > ${test_current}content-line-8-name-yup-total.expect &&
  generate_operate_run +n -clnt 9 yup      ${file_source} > ${test_current}content-line-9-name-yup-total.expect &&
  generate_operate_run +n -clnt 10 yup     ${file_source} > ${test_current}content-line-10-name-yup-total.expect &&
  generate_operate_run +n -clnt 20 yup     ${file_source} > ${test_current}content-line-20-name-yup-total.expect &&
  generate_operate_run +n -clnt 100 yup    ${file_source} > ${test_current}content-line-100-name-yup-total.expect &&
  generate_operate_run +n -clant 0 0 yup   ${file_source} > ${test_current}content-line-0-at-0-name-yup-total.expect &&
  generate_operate_run +n -clant 1 0 yup   ${file_source} > ${test_current}content-line-1-at-0-name-yup-total.expect &&
  generate_operate_run +n -clant 2 0 yup   ${file_source} > ${test_current}content-line-2-at-0-name-yup-total.expect &&
  generate_operate_run +n -clant 3 0 yup   ${file_source} > ${test_current}content-line-3-at-0-name-yup-total.expect &&
  generate_operate_run +n -clant 4 0 yup   ${file_source} > ${test_current}content-line-4-at-0-name-yup-total.expect &&
  generate_operate_run +n -clant 5 0 yup   ${file_source} > ${test_current}content-line-5-at-0-name-yup-total.expect &&
  generate_operate_run +n -clant 6 0 yup   ${file_source} > ${test_current}content-line-6-at-0-name-yup-total.expect &&
  generate_operate_run +n -clant 7 0 yup   ${file_source} > ${test_current}content-line-7-at-0-name-yup-total.expect &&
  generate_operate_run +n -clant 8 0 yup   ${file_source} > ${test_current}content-line-8-at-0-name-yup-total.expect &&
  generate_operate_run +n -clant 9 0 yup   ${file_source} > ${test_current}content-line-9-at-0-name-yup-total.expect &&
  generate_operate_run +n -clant 10 0 yup  ${file_source} > ${test_current}content-line-10-at-0-name-yup-total.expect &&
  generate_operate_run +n -clant 20 0 yup  ${file_source} > ${test_current}content-line-20-at-0-name-yup-total.expect &&
  generate_operate_run +n -clant 100 0 yup ${file_source} > ${test_current}content-line-100-at-0-name-yup-total.expect &&
  generate_operate_run +n -clant 0 1 yup   ${file_source} > ${test_current}content-line-0-at-1-name-yup-total.expect &&
  generate_operate_run +n -clant 1 1 yup   ${file_source} > ${test_current}content-line-1-at-1-name-yup-total.expect &&
  generate_operate_run +n -clant 2 1 yup   ${file_source} > ${test_current}content-line-2-at-1-name-yup-total.expect &&
  generate_operate_run +n -clant 3 1 yup   ${file_source} > ${test_current}content-line-3-at-1-name-yup-total.expect &&
  generate_operate_run +n -clant 4 1 yup   ${file_source} > ${test_current}content-line-4-at-1-name-yup-total.expect &&
  generate_operate_run +n -clant 5 1 yup   ${file_source} > ${test_current}content-line-5-at-1-name-yup-total.expect &&
  generate_operate_run +n -clant 6 1 yup   ${file_source} > ${test_current}content-line-6-at-1-name-yup-total.expect &&
  generate_operate_run +n -clant 7 1 yup   ${file_source} > ${test_current}content-line-7-at-1-name-yup-total.expect &&
  generate_operate_run +n -clant 8 1 yup   ${file_source} > ${test_current}content-line-8-at-1-name-yup-total.expect &&
  generate_operate_run +n -clant 9 1 yup   ${file_source} > ${test_current}content-line-9-at-1-name-yup-total.expect &&
  generate_operate_run +n -clant 10 1 yup  ${file_source} > ${test_current}content-line-10-at-1-name-yup-total.expect &&
  generate_operate_run +n -clant 20 1 yup  ${file_source} > ${test_current}content-line-20-at-1-name-yup-total.expect &&
  generate_operate_run +n -clant 100 1 yup ${file_source} > ${test_current}content-line-100-at-1-name-yup-total.expect &&
  generate_operate_run +n -clant 100 1 yup ${file_source} > ${test_current}content-line-100-at-1-name-yup-total.expect &&

  #### CONTENT - WHOLE ####
  generate_operate_run +n -cw         ${file_source} > ${test_current}content.expect &&
  generate_operate_run +n -cwl 0      ${file_source} > ${test_current}content-whole-line-0.expect &&
  generate_operate_run +n -cwl 1      ${file_source} > ${test_current}content-whole-line-1.expect &&
  generate_operate_run +n -cwl 2      ${file_source} > ${test_current}content-whole-line-2.expect &&
  generate_operate_run +n -cwl 3      ${file_source} > ${test_current}content-whole-line-3.expect &&
  generate_operate_run +n -cwl 4      ${file_source} > ${test_current}content-whole-line-4.expect &&
  generate_operate_run +n -cwl 5      ${file_source} > ${test_current}content-whole-line-5.expect &&
  generate_operate_run +n -cwl 6      ${file_source} > ${test_current}content-whole-line-6.expect &&
  generate_operate_run +n -cwl 7      ${file_source} > ${test_current}content-whole-line-7.expect &&
  generate_operate_run +n -cwl 8      ${file_source} > ${test_current}content-whole-line-8.expect &&
  generate_operate_run +n -cwl 9      ${file_source} > ${test_current}content-whole-line-9.expect &&
  generate_operate_run +n -cwl 10     ${file_source} > ${test_current}content-whole-line-10.expect &&
  generate_operate_run +n -cwl 20     ${file_source} > ${test_current}content-whole-line-20.expect &&
  generate_operate_run +n -cwl 100    ${file_source} > ${test_current}content-whole-line-100.expect &&
  generate_operate_run +n -cwla 0 0   ${file_source} > ${test_current}content-whole-line-0-at-0.expect &&
  generate_operate_run +n -cwla 1 0   ${file_source} > ${test_current}content-whole-line-1-at-0.expect &&
  generate_operate_run +n -cwla 2 0   ${file_source} > ${test_current}content-whole-line-2-at-0.expect &&
  generate_operate_run +n -cwla 3 0   ${file_source} > ${test_current}content-whole-line-3-at-0.expect &&
  generate_operate_run +n -cwla 4 0   ${file_source} > ${test_current}content-whole-line-4-at-0.expect &&
  generate_operate_run +n -cwla 5 0   ${file_source} > ${test_current}content-whole-line-5-at-0.expect &&
  generate_operate_run +n -cwla 6 0   ${file_source} > ${test_current}content-whole-line-6-at-0.expect &&
  generate_operate_run +n -cwla 7 0   ${file_source} > ${test_current}content-whole-line-7-at-0.expect &&
  generate_operate_run +n -cwla 8 0   ${file_source} > ${test_current}content-whole-line-8-at-0.expect &&
  generate_operate_run +n -cwla 9 0   ${file_source} > ${test_current}content-whole-line-9-at-0.expect &&
  generate_operate_run +n -cwla 10 0  ${file_source} > ${test_current}content-whole-line-10-at-0.expect &&
  generate_operate_run +n -cwla 20 0  ${file_source} > ${test_current}content-whole-line-20-at-0.expect &&
  generate_operate_run +n -cwla 100 0 ${file_source} > ${test_current}content-whole-line-100-at-0.expect &&
  generate_operate_run +n -cwla 0 1   ${file_source} > ${test_current}content-whole-line-0-at-1.expect &&
  generate_operate_run +n -cwla 1 1   ${file_source} > ${test_current}content-whole-line-1-at-1.expect &&
  generate_operate_run +n -cwla 2 1   ${file_source} > ${test_current}content-whole-line-2-at-1.expect &&
  generate_operate_run +n -cwla 3 1   ${file_source} > ${test_current}content-whole-line-3-at-1.expect &&
  generate_operate_run +n -cwla 4 1   ${file_source} > ${test_current}content-whole-line-4-at-1.expect &&
  generate_operate_run +n -cwla 5 1   ${file_source} > ${test_current}content-whole-line-5-at-1.expect &&
  generate_operate_run +n -cwla 6 1   ${file_source} > ${test_current}content-whole-line-6-at-1.expect &&
  generate_operate_run +n -cwla 7 1   ${file_source} > ${test_current}content-whole-line-7-at-1.expect &&
  generate_operate_run +n -cwla 8 1   ${file_source} > ${test_current}content-whole-line-8-at-1.expect &&
  generate_operate_run +n -cwla 9 1   ${file_source} > ${test_current}content-whole-line-9-at-1.expect &&
  generate_operate_run +n -cwla 10 1  ${file_source} > ${test_current}content-whole-line-10-at-1.expect &&
  generate_operate_run +n -cwla 20 1  ${file_source} > ${test_current}content-whole-line-20-at-1.expect &&
  generate_operate_run +n -cwla 100 1 ${file_source} > ${test_current}content-whole-line-100-at-1.expect &&

  generate_operate_run +n -cwn a         ${file_source} > ${test_current}content-whole-name-a.expect &&
  generate_operate_run +n -cwln 0 a      ${file_source} > ${test_current}content-whole-line-0-name-a.expect &&
  generate_operate_run +n -cwln 1 a      ${file_source} > ${test_current}content-whole-line-1-name-a.expect &&
  generate_operate_run +n -cwln 2 a      ${file_source} > ${test_current}content-whole-line-2-name-a.expect &&
  generate_operate_run +n -cwln 3 a      ${file_source} > ${test_current}content-whole-line-3-name-a.expect &&
  generate_operate_run +n -cwln 4 a      ${file_source} > ${test_current}content-whole-line-4-name-a.expect &&
  generate_operate_run +n -cwln 5 a      ${file_source} > ${test_current}content-whole-line-5-name-a.expect &&
  generate_operate_run +n -cwln 6 a      ${file_source} > ${test_current}content-whole-line-6-name-a.expect &&
  generate_operate_run +n -cwln 7 a      ${file_source} > ${test_current}content-whole-line-7-name-a.expect &&
  generate_operate_run +n -cwln 8 a      ${file_source} > ${test_current}content-whole-line-8-name-a.expect &&
  generate_operate_run +n -cwln 9 a      ${file_source} > ${test_current}content-whole-line-9-name-a.expect &&
  generate_operate_run +n -cwln 10 a     ${file_source} > ${test_current}content-whole-line-10-name-a.expect &&
  generate_operate_run +n -cwln 20 a     ${file_source} > ${test_current}content-whole-line-20-name-a.expect &&
  generate_operate_run +n -cwln 100 a    ${file_source} > ${test_current}content-whole-line-100-name-a.expect &&
  generate_operate_run +n -cwlan 0 0 a   ${file_source} > ${test_current}content-whole-line-0-at-0-name-a.expect &&
  generate_operate_run +n -cwlan 1 0 a   ${file_source} > ${test_current}content-whole-line-1-at-0-name-a.expect &&
  generate_operate_run +n -cwlan 2 0 a   ${file_source} > ${test_current}content-whole-line-2-at-0-name-a.expect &&
  generate_operate_run +n -cwlan 3 0 a   ${file_source} > ${test_current}content-whole-line-3-at-0-name-a.expect &&
  generate_operate_run +n -cwlan 4 0 a   ${file_source} > ${test_current}content-whole-line-4-at-0-name-a.expect &&
  generate_operate_run +n -cwlan 5 0 a   ${file_source} > ${test_current}content-whole-line-5-at-0-name-a.expect &&
  generate_operate_run +n -cwlan 6 0 a   ${file_source} > ${test_current}content-whole-line-6-at-0-name-a.expect &&
  generate_operate_run +n -cwlan 7 0 a   ${file_source} > ${test_current}content-whole-line-7-at-0-name-a.expect &&
  generate_operate_run +n -cwlan 8 0 a   ${file_source} > ${test_current}content-whole-line-8-at-0-name-a.expect &&
  generate_operate_run +n -cwlan 9 0 a   ${file_source} > ${test_current}content-whole-line-9-at-0-name-a.expect &&
  generate_operate_run +n -cwlan 10 0 a  ${file_source} > ${test_current}content-whole-line-10-at-0-name-a.expect &&
  generate_operate_run +n -cwlan 20 0 a  ${file_source} > ${test_current}content-whole-line-20-at-0-name-a.expect &&
  generate_operate_run +n -cwlan 100 0 a ${file_source} > ${test_current}content-whole-line-100-at-0-name-a.expect &&
  generate_operate_run +n -cwlan 0 1 a   ${file_source} > ${test_current}content-whole-line-0-at-1-name-a.expect &&
  generate_operate_run +n -cwlan 1 1 a   ${file_source} > ${test_current}content-whole-line-1-at-1-name-a.expect &&
  generate_operate_run +n -cwlan 2 1 a   ${file_source} > ${test_current}content-whole-line-2-at-1-name-a.expect &&
  generate_operate_run +n -cwlan 3 1 a   ${file_source} > ${test_current}content-whole-line-3-at-1-name-a.expect &&
  generate_operate_run +n -cwlan 4 1 a   ${file_source} > ${test_current}content-whole-line-4-at-1-name-a.expect &&
  generate_operate_run +n -cwlan 5 1 a   ${file_source} > ${test_current}content-whole-line-5-at-1-name-a.expect &&
  generate_operate_run +n -cwlan 6 1 a   ${file_source} > ${test_current}content-whole-line-6-at-1-name-a.expect &&
  generate_operate_run +n -cwlan 7 1 a   ${file_source} > ${test_current}content-whole-line-7-at-1-name-a.expect &&
  generate_operate_run +n -cwlan 8 1 a   ${file_source} > ${test_current}content-whole-line-8-at-1-name-a.expect &&
  generate_operate_run +n -cwlan 9 1 a   ${file_source} > ${test_current}content-whole-line-9-at-1-name-a.expect &&
  generate_operate_run +n -cwlan 10 1 a  ${file_source} > ${test_current}content-whole-line-10-at-1-name-a.expect &&
  generate_operate_run +n -cwlan 20 1 a  ${file_source} > ${test_current}content-whole-line-20-at-1-name-a.expect &&
  generate_operate_run +n -cwlan 100 1 a ${file_source} > ${test_current}content-whole-line-100-at-1-name-a.expect &&

  generate_operate_run +n -cwn b         ${file_source} > ${test_current}content-whole-name-b.expect &&
  generate_operate_run +n -cwln 0 b      ${file_source} > ${test_current}content-whole-line-0-name-b.expect &&
  generate_operate_run +n -cwln 1 b      ${file_source} > ${test_current}content-whole-line-1-name-b.expect &&
  generate_operate_run +n -cwln 2 b      ${file_source} > ${test_current}content-whole-line-2-name-b.expect &&
  generate_operate_run +n -cwln 3 b      ${file_source} > ${test_current}content-whole-line-3-name-b.expect &&
  generate_operate_run +n -cwln 4 b      ${file_source} > ${test_current}content-whole-line-4-name-b.expect &&
  generate_operate_run +n -cwln 5 b      ${file_source} > ${test_current}content-whole-line-5-name-b.expect &&
  generate_operate_run +n -cwln 6 b      ${file_source} > ${test_current}content-whole-line-6-name-b.expect &&
  generate_operate_run +n -cwln 7 b      ${file_source} > ${test_current}content-whole-line-7-name-b.expect &&
  generate_operate_run +n -cwln 8 b      ${file_source} > ${test_current}content-whole-line-8-name-b.expect &&
  generate_operate_run +n -cwln 9 b      ${file_source} > ${test_current}content-whole-line-9-name-b.expect &&
  generate_operate_run +n -cwln 10 b     ${file_source} > ${test_current}content-whole-line-10-name-b.expect &&
  generate_operate_run +n -cwln 20 b     ${file_source} > ${test_current}content-whole-line-20-name-b.expect &&
  generate_operate_run +n -cwln 100 b    ${file_source} > ${test_current}content-whole-line-100-name-b.expect &&
  generate_operate_run +n -cwlan 0 0 b   ${file_source} > ${test_current}content-whole-line-0-at-0-name-b.expect &&
  generate_operate_run +n -cwlan 1 0 b   ${file_source} > ${test_current}content-whole-line-1-at-0-name-b.expect &&
  generate_operate_run +n -cwlan 2 0 b   ${file_source} > ${test_current}content-whole-line-2-at-0-name-b.expect &&
  generate_operate_run +n -cwlan 3 0 b   ${file_source} > ${test_current}content-whole-line-3-at-0-name-b.expect &&
  generate_operate_run +n -cwlan 4 0 b   ${file_source} > ${test_current}content-whole-line-4-at-0-name-b.expect &&
  generate_operate_run +n -cwlan 5 0 b   ${file_source} > ${test_current}content-whole-line-5-at-0-name-b.expect &&
  generate_operate_run +n -cwlan 6 0 b   ${file_source} > ${test_current}content-whole-line-6-at-0-name-b.expect &&
  generate_operate_run +n -cwlan 7 0 b   ${file_source} > ${test_current}content-whole-line-7-at-0-name-b.expect &&
  generate_operate_run +n -cwlan 8 0 b   ${file_source} > ${test_current}content-whole-line-8-at-0-name-b.expect &&
  generate_operate_run +n -cwlan 9 0 b   ${file_source} > ${test_current}content-whole-line-9-at-0-name-b.expect &&
  generate_operate_run +n -cwlan 10 0 b  ${file_source} > ${test_current}content-whole-line-10-at-0-name-b.expect &&
  generate_operate_run +n -cwlan 20 0 b  ${file_source} > ${test_current}content-whole-line-20-at-0-name-b.expect &&
  generate_operate_run +n -cwlan 100 0 b ${file_source} > ${test_current}content-whole-line-100-at-0-name-b.expect &&
  generate_operate_run +n -cwlan 0 1 b   ${file_source} > ${test_current}content-whole-line-0-at-1-name-b.expect &&
  generate_operate_run +n -cwlan 1 1 b   ${file_source} > ${test_current}content-whole-line-1-at-1-name-b.expect &&
  generate_operate_run +n -cwlan 2 1 b   ${file_source} > ${test_current}content-whole-line-2-at-1-name-b.expect &&
  generate_operate_run +n -cwlan 3 1 b   ${file_source} > ${test_current}content-whole-line-3-at-1-name-b.expect &&
  generate_operate_run +n -cwlan 4 1 b   ${file_source} > ${test_current}content-whole-line-4-at-1-name-b.expect &&
  generate_operate_run +n -cwlan 5 1 b   ${file_source} > ${test_current}content-whole-line-5-at-1-name-b.expect &&
  generate_operate_run +n -cwlan 6 1 b   ${file_source} > ${test_current}content-whole-line-6-at-1-name-b.expect &&
  generate_operate_run +n -cwlan 7 1 b   ${file_source} > ${test_current}content-whole-line-7-at-1-name-b.expect &&
  generate_operate_run +n -cwlan 8 1 b   ${file_source} > ${test_current}content-whole-line-8-at-1-name-b.expect &&
  generate_operate_run +n -cwlan 9 1 b   ${file_source} > ${test_current}content-whole-line-9-at-1-name-b.expect &&
  generate_operate_run +n -cwlan 10 1 b  ${file_source} > ${test_current}content-whole-line-10-at-1-name-b.expect &&
  generate_operate_run +n -cwlan 20 1 b  ${file_source} > ${test_current}content-whole-line-20-at-1-name-b.expect &&
  generate_operate_run +n -cwlan 100 1 b ${file_source} > ${test_current}content-whole-line-100-at-1-name-b.expect &&

  generate_operate_run +n -cwn is         ${file_source} > ${test_current}content-whole-name-is.expect &&
  generate_operate_run +n -cwln 0 is      ${file_source} > ${test_current}content-whole-line-0-name-is.expect &&
  generate_operate_run +n -cwln 1 is      ${file_source} > ${test_current}content-whole-line-1-name-is.expect &&
  generate_operate_run +n -cwln 2 is      ${file_source} > ${test_current}content-whole-line-2-name-is.expect &&
  generate_operate_run +n -cwln 3 is      ${file_source} > ${test_current}content-whole-line-3-name-is.expect &&
  generate_operate_run +n -cwln 4 is      ${file_source} > ${test_current}content-whole-line-4-name-is.expect &&
  generate_operate_run +n -cwln 5 is      ${file_source} > ${test_current}content-whole-line-5-name-is.expect &&
  generate_operate_run +n -cwln 6 is      ${file_source} > ${test_current}content-whole-line-6-name-is.expect &&
  generate_operate_run +n -cwln 7 is      ${file_source} > ${test_current}content-whole-line-7-name-is.expect &&
  generate_operate_run +n -cwln 8 is      ${file_source} > ${test_current}content-whole-line-8-name-is.expect &&
  generate_operate_run +n -cwln 9 is      ${file_source} > ${test_current}content-whole-line-9-name-is.expect &&
  generate_operate_run +n -cwln 10 is     ${file_source} > ${test_current}content-whole-line-10-name-is.expect &&
  generate_operate_run +n -cwln 20 is     ${file_source} > ${test_current}content-whole-line-20-name-is.expect &&
  generate_operate_run +n -cwln 100 is    ${file_source} > ${test_current}content-whole-line-100-name-is.expect &&
  generate_operate_run +n -cwlan 0 0 is   ${file_source} > ${test_current}content-whole-line-0-at-0-name-is.expect &&
  generate_operate_run +n -cwlan 1 0 is   ${file_source} > ${test_current}content-whole-line-1-at-0-name-is.expect &&
  generate_operate_run +n -cwlan 2 0 is   ${file_source} > ${test_current}content-whole-line-2-at-0-name-is.expect &&
  generate_operate_run +n -cwlan 3 0 is   ${file_source} > ${test_current}content-whole-line-3-at-0-name-is.expect &&
  generate_operate_run +n -cwlan 4 0 is   ${file_source} > ${test_current}content-whole-line-4-at-0-name-is.expect &&
  generate_operate_run +n -cwlan 5 0 is   ${file_source} > ${test_current}content-whole-line-5-at-0-name-is.expect &&
  generate_operate_run +n -cwlan 6 0 is   ${file_source} > ${test_current}content-whole-line-6-at-0-name-is.expect &&
  generate_operate_run +n -cwlan 7 0 is   ${file_source} > ${test_current}content-whole-line-7-at-0-name-is.expect &&
  generate_operate_run +n -cwlan 8 0 is   ${file_source} > ${test_current}content-whole-line-8-at-0-name-is.expect &&
  generate_operate_run +n -cwlan 9 0 is   ${file_source} > ${test_current}content-whole-line-9-at-0-name-is.expect &&
  generate_operate_run +n -cwlan 10 0 is  ${file_source} > ${test_current}content-whole-line-10-at-0-name-is.expect &&
  generate_operate_run +n -cwlan 20 0 is  ${file_source} > ${test_current}content-whole-line-20-at-0-name-is.expect &&
  generate_operate_run +n -cwlan 100 0 is ${file_source} > ${test_current}content-whole-line-100-at-0-name-is.expect &&
  generate_operate_run +n -cwlan 0 1 is   ${file_source} > ${test_current}content-whole-line-0-at-1-name-is.expect &&
  generate_operate_run +n -cwlan 1 1 is   ${file_source} > ${test_current}content-whole-line-1-at-1-name-is.expect &&
  generate_operate_run +n -cwlan 2 1 is   ${file_source} > ${test_current}content-whole-line-2-at-1-name-is.expect &&
  generate_operate_run +n -cwlan 3 1 is   ${file_source} > ${test_current}content-whole-line-3-at-1-name-is.expect &&
  generate_operate_run +n -cwlan 4 1 is   ${file_source} > ${test_current}content-whole-line-4-at-1-name-is.expect &&
  generate_operate_run +n -cwlan 5 1 is   ${file_source} > ${test_current}content-whole-line-5-at-1-name-is.expect &&
  generate_operate_run +n -cwlan 6 1 is   ${file_source} > ${test_current}content-whole-line-6-at-1-name-is.expect &&
  generate_operate_run +n -cwlan 7 1 is   ${file_source} > ${test_current}content-whole-line-7-at-1-name-is.expect &&
  generate_operate_run +n -cwlan 8 1 is   ${file_source} > ${test_current}content-whole-line-8-at-1-name-is.expect &&
  generate_operate_run +n -cwlan 9 1 is   ${file_source} > ${test_current}content-whole-line-9-at-1-name-is.expect &&
  generate_operate_run +n -cwlan 10 1 is  ${file_source} > ${test_current}content-whole-line-10-at-1-name-is.expect &&
  generate_operate_run +n -cwlan 20 1 is  ${file_source} > ${test_current}content-whole-line-20-at-1-name-is.expect &&
  generate_operate_run +n -cwlan 100 1 is ${file_source} > ${test_current}content-whole-line-100-at-1-name-is.expect &&

  generate_operate_run +n -cwn yup         ${file_source} > ${test_current}content-whole-name-yup.expect &&
  generate_operate_run +n -cwln 0 yup      ${file_source} > ${test_current}content-whole-line-0-name-yup.expect &&
  generate_operate_run +n -cwln 1 yup      ${file_source} > ${test_current}content-whole-line-1-name-yup.expect &&
  generate_operate_run +n -cwln 2 yup      ${file_source} > ${test_current}content-whole-line-2-name-yup.expect &&
  generate_operate_run +n -cwln 3 yup      ${file_source} > ${test_current}content-whole-line-3-name-yup.expect &&
  generate_operate_run +n -cwln 4 yup      ${file_source} > ${test_current}content-whole-line-4-name-yup.expect &&
  generate_operate_run +n -cwln 5 yup      ${file_source} > ${test_current}content-whole-line-5-name-yup.expect &&
  generate_operate_run +n -cwln 6 yup      ${file_source} > ${test_current}content-whole-line-6-name-yup.expect &&
  generate_operate_run +n -cwln 7 yup      ${file_source} > ${test_current}content-whole-line-7-name-yup.expect &&
  generate_operate_run +n -cwln 8 yup      ${file_source} > ${test_current}content-whole-line-8-name-yup.expect &&
  generate_operate_run +n -cwln 9 yup      ${file_source} > ${test_current}content-whole-line-9-name-yup.expect &&
  generate_operate_run +n -cwln 10 yup     ${file_source} > ${test_current}content-whole-line-10-name-yup.expect &&
  generate_operate_run +n -cwln 20 yup     ${file_source} > ${test_current}content-whole-line-20-name-yup.expect &&
  generate_operate_run +n -cwln 100 yup    ${file_source} > ${test_current}content-whole-line-100-name-yup.expect &&
  generate_operate_run +n -cwlan 0 0 yup   ${file_source} > ${test_current}content-whole-line-0-at-0-name-yup.expect &&
  generate_operate_run +n -cwlan 1 0 yup   ${file_source} > ${test_current}content-whole-line-1-at-0-name-yup.expect &&
  generate_operate_run +n -cwlan 2 0 yup   ${file_source} > ${test_current}content-whole-line-2-at-0-name-yup.expect &&
  generate_operate_run +n -cwlan 3 0 yup   ${file_source} > ${test_current}content-whole-line-3-at-0-name-yup.expect &&
  generate_operate_run +n -cwlan 4 0 yup   ${file_source} > ${test_current}content-whole-line-4-at-0-name-yup.expect &&
  generate_operate_run +n -cwlan 5 0 yup   ${file_source} > ${test_current}content-whole-line-5-at-0-name-yup.expect &&
  generate_operate_run +n -cwlan 6 0 yup   ${file_source} > ${test_current}content-whole-line-6-at-0-name-yup.expect &&
  generate_operate_run +n -cwlan 7 0 yup   ${file_source} > ${test_current}content-whole-line-7-at-0-name-yup.expect &&
  generate_operate_run +n -cwlan 8 0 yup   ${file_source} > ${test_current}content-whole-line-8-at-0-name-yup.expect &&
  generate_operate_run +n -cwlan 9 0 yup   ${file_source} > ${test_current}content-whole-line-9-at-0-name-yup.expect &&
  generate_operate_run +n -cwlan 10 0 yup  ${file_source} > ${test_current}content-whole-line-10-at-0-name-yup.expect &&
  generate_operate_run +n -cwlan 20 0 yup  ${file_source} > ${test_current}content-whole-line-20-at-0-name-yup.expect &&
  generate_operate_run +n -cwlan 100 0 yup ${file_source} > ${test_current}content-whole-line-100-at-0-name-yup.expect &&
  generate_operate_run +n -cwlan 0 1 yup   ${file_source} > ${test_current}content-whole-line-0-at-1-name-yup.expect &&
  generate_operate_run +n -cwlan 1 1 yup   ${file_source} > ${test_current}content-whole-line-1-at-1-name-yup.expect &&
  generate_operate_run +n -cwlan 2 1 yup   ${file_source} > ${test_current}content-whole-line-2-at-1-name-yup.expect &&
  generate_operate_run +n -cwlan 3 1 yup   ${file_source} > ${test_current}content-whole-line-3-at-1-name-yup.expect &&
  generate_operate_run +n -cwlan 4 1 yup   ${file_source} > ${test_current}content-whole-line-4-at-1-name-yup.expect &&
  generate_operate_run +n -cwlan 5 1 yup   ${file_source} > ${test_current}content-whole-line-5-at-1-name-yup.expect &&
  generate_operate_run +n -cwlan 6 1 yup   ${file_source} > ${test_current}content-whole-line-6-at-1-name-yup.expect &&
  generate_operate_run +n -cwlan 7 1 yup   ${file_source} > ${test_current}content-whole-line-7-at-1-name-yup.expect &&
  generate_operate_run +n -cwlan 8 1 yup   ${file_source} > ${test_current}content-whole-line-8-at-1-name-yup.expect &&
  generate_operate_run +n -cwlan 9 1 yup   ${file_source} > ${test_current}content-whole-line-9-at-1-name-yup.expect &&
  generate_operate_run +n -cwlan 10 1 yup  ${file_source} > ${test_current}content-whole-line-10-at-1-name-yup.expect &&
  generate_operate_run +n -cwlan 20 1 yup  ${file_source} > ${test_current}content-whole-line-20-at-1-name-yup.expect &&

  #### CONTENT - WHOLE - TOTAL ####
  generate_operate_run +n -cwt         ${file_source} > ${test_current}content-whole-total.expect &&
  generate_operate_run +n -cwlt 0      ${file_source} > ${test_current}content-whole-line-0-total.expect &&
  generate_operate_run +n -cwlt 1      ${file_source} > ${test_current}content-whole-line-1-total.expect &&
  generate_operate_run +n -cwlt 2      ${file_source} > ${test_current}content-whole-line-2-total.expect &&
  generate_operate_run +n -cwlt 3      ${file_source} > ${test_current}content-whole-line-3-total.expect &&
  generate_operate_run +n -cwlt 4      ${file_source} > ${test_current}content-whole-line-4-total.expect &&
  generate_operate_run +n -cwlt 5      ${file_source} > ${test_current}content-whole-line-5-total.expect &&
  generate_operate_run +n -cwlt 6      ${file_source} > ${test_current}content-whole-line-6-total.expect &&
  generate_operate_run +n -cwlt 7      ${file_source} > ${test_current}content-whole-line-7-total.expect &&
  generate_operate_run +n -cwlt 8      ${file_source} > ${test_current}content-whole-line-8-total.expect &&
  generate_operate_run +n -cwlt 9      ${file_source} > ${test_current}content-whole-line-9-total.expect &&
  generate_operate_run +n -cwlt 10     ${file_source} > ${test_current}content-whole-line-10-total.expect &&
  generate_operate_run +n -cwlt 20     ${file_source} > ${test_current}content-whole-line-20-total.expect &&
  generate_operate_run +n -cwlt 100    ${file_source} > ${test_current}content-whole-line-100-total.expect &&
  generate_operate_run +n -cwlat 0 0   ${file_source} > ${test_current}content-whole-line-0-at-0-total.expect &&
  generate_operate_run +n -cwlat 1 0   ${file_source} > ${test_current}content-whole-line-1-at-0-total.expect &&
  generate_operate_run +n -cwlat 2 0   ${file_source} > ${test_current}content-whole-line-2-at-0-total.expect &&
  generate_operate_run +n -cwlat 3 0   ${file_source} > ${test_current}content-whole-line-3-at-0-total.expect &&
  generate_operate_run +n -cwlat 4 0   ${file_source} > ${test_current}content-whole-line-4-at-0-total.expect &&
  generate_operate_run +n -cwlat 5 0   ${file_source} > ${test_current}content-whole-line-5-at-0-total.expect &&
  generate_operate_run +n -cwlat 6 0   ${file_source} > ${test_current}content-whole-line-6-at-0-total.expect &&
  generate_operate_run +n -cwlat 7 0   ${file_source} > ${test_current}content-whole-line-7-at-0-total.expect &&
  generate_operate_run +n -cwlat 8 0   ${file_source} > ${test_current}content-whole-line-8-at-0-total.expect &&
  generate_operate_run +n -cwlat 9 0   ${file_source} > ${test_current}content-whole-line-9-at-0-total.expect &&
  generate_operate_run +n -cwlat 10 0  ${file_source} > ${test_current}content-whole-line-10-at-0-total.expect &&
  generate_operate_run +n -cwlat 20 0  ${file_source} > ${test_current}content-whole-line-20-at-0-total.expect &&
  generate_operate_run +n -cwlat 100 0 ${file_source} > ${test_current}content-whole-line-100-at-0-total.expect &&
  generate_operate_run +n -cwlat 0 1   ${file_source} > ${test_current}content-whole-line-0-at-1-total.expect &&
  generate_operate_run +n -cwlat 1 1   ${file_source} > ${test_current}content-whole-line-1-at-1-total.expect &&
  generate_operate_run +n -cwlat 2 1   ${file_source} > ${test_current}content-whole-line-2-at-1-total.expect &&
  generate_operate_run +n -cwlat 3 1   ${file_source} > ${test_current}content-whole-line-3-at-1-total.expect &&
  generate_operate_run +n -cwlat 4 1   ${file_source} > ${test_current}content-whole-line-4-at-1-total.expect &&
  generate_operate_run +n -cwlat 5 1   ${file_source} > ${test_current}content-whole-line-5-at-1-total.expect &&
  generate_operate_run +n -cwlat 6 1   ${file_source} > ${test_current}content-whole-line-6-at-1-total.expect &&
  generate_operate_run +n -cwlat 7 1   ${file_source} > ${test_current}content-whole-line-7-at-1-total.expect &&
  generate_operate_run +n -cwlat 8 1   ${file_source} > ${test_current}content-whole-line-8-at-1-total.expect &&
  generate_operate_run +n -cwlat 9 1   ${file_source} > ${test_current}content-whole-line-9-at-1-total.expect &&
  generate_operate_run +n -cwlat 10 1  ${file_source} > ${test_current}content-whole-line-10-at-1-total.expect &&
  generate_operate_run +n -cwlat 20 1  ${file_source} > ${test_current}content-whole-line-20-at-1-total.expect &&
  generate_operate_run +n -cwlat 100 1 ${file_source} > ${test_current}content-whole-line-100-at-1-total.expect &&

  generate_operate_run +n -cwnt a         ${file_source} > ${test_current}content-whole-name-a-total.expect &&
  generate_operate_run +n -cwlnt 0 a      ${file_source} > ${test_current}content-whole-line-0-name-a-total.expect &&
  generate_operate_run +n -cwlnt 1 a      ${file_source} > ${test_current}content-whole-line-1-name-a-total.expect &&
  generate_operate_run +n -cwlnt 2 a      ${file_source} > ${test_current}content-whole-line-2-name-a-total.expect &&
  generate_operate_run +n -cwlnt 3 a      ${file_source} > ${test_current}content-whole-line-3-name-a-total.expect &&
  generate_operate_run +n -cwlnt 4 a      ${file_source} > ${test_current}content-whole-line-4-name-a-total.expect &&
  generate_operate_run +n -cwlnt 5 a      ${file_source} > ${test_current}content-whole-line-5-name-a-total.expect &&
  generate_operate_run +n -cwlnt 6 a      ${file_source} > ${test_current}content-whole-line-6-name-a-total.expect &&
  generate_operate_run +n -cwlnt 7 a      ${file_source} > ${test_current}content-whole-line-7-name-a-total.expect &&
  generate_operate_run +n -cwlnt 8 a      ${file_source} > ${test_current}content-whole-line-8-name-a-total.expect &&
  generate_operate_run +n -cwlnt 9 a      ${file_source} > ${test_current}content-whole-line-9-name-a-total.expect &&
  generate_operate_run +n -cwlnt 10 a     ${file_source} > ${test_current}content-whole-line-10-name-a-total.expect &&
  generate_operate_run +n -cwlnt 20 a     ${file_source} > ${test_current}content-whole-line-20-name-a-total.expect &&
  generate_operate_run +n -cwlnt 100 a    ${file_source} > ${test_current}content-whole-line-100-name-a-total.expect &&
  generate_operate_run +n -cwlant 0 0 a   ${file_source} > ${test_current}content-whole-line-0-at-0-name-a-total.expect &&
  generate_operate_run +n -cwlant 1 0 a   ${file_source} > ${test_current}content-whole-line-1-at-0-name-a-total.expect &&
  generate_operate_run +n -cwlant 2 0 a   ${file_source} > ${test_current}content-whole-line-2-at-0-name-a-total.expect &&
  generate_operate_run +n -cwlant 3 0 a   ${file_source} > ${test_current}content-whole-line-3-at-0-name-a-total.expect &&
  generate_operate_run +n -cwlant 4 0 a   ${file_source} > ${test_current}content-whole-line-4-at-0-name-a-total.expect &&
  generate_operate_run +n -cwlant 5 0 a   ${file_source} > ${test_current}content-whole-line-5-at-0-name-a-total.expect &&
  generate_operate_run +n -cwlant 6 0 a   ${file_source} > ${test_current}content-whole-line-6-at-0-name-a-total.expect &&
  generate_operate_run +n -cwlant 7 0 a   ${file_source} > ${test_current}content-whole-line-7-at-0-name-a-total.expect &&
  generate_operate_run +n -cwlant 8 0 a   ${file_source} > ${test_current}content-whole-line-8-at-0-name-a-total.expect &&
  generate_operate_run +n -cwlant 9 0 a   ${file_source} > ${test_current}content-whole-line-9-at-0-name-a-total.expect &&
  generate_operate_run +n -cwlant 10 0 a  ${file_source} > ${test_current}content-whole-line-10-at-0-name-a-total.expect &&
  generate_operate_run +n -cwlant 20 0 a  ${file_source} > ${test_current}content-whole-line-20-at-0-name-a-total.expect &&
  generate_operate_run +n -cwlant 100 0 a ${file_source} > ${test_current}content-whole-line-100-at-0-name-a-total.expect &&
  generate_operate_run +n -cwlant 0 1 a   ${file_source} > ${test_current}content-whole-line-0-at-1-name-a-total.expect &&
  generate_operate_run +n -cwlant 1 1 a   ${file_source} > ${test_current}content-whole-line-1-at-1-name-a-total.expect &&
  generate_operate_run +n -cwlant 2 1 a   ${file_source} > ${test_current}content-whole-line-2-at-1-name-a-total.expect &&
  generate_operate_run +n -cwlant 3 1 a   ${file_source} > ${test_current}content-whole-line-3-at-1-name-a-total.expect &&
  generate_operate_run +n -cwlant 4 1 a   ${file_source} > ${test_current}content-whole-line-4-at-1-name-a-total.expect &&
  generate_operate_run +n -cwlant 5 1 a   ${file_source} > ${test_current}content-whole-line-5-at-1-name-a-total.expect &&
  generate_operate_run +n -cwlant 6 1 a   ${file_source} > ${test_current}content-whole-line-6-at-1-name-a-total.expect &&
  generate_operate_run +n -cwlant 7 1 a   ${file_source} > ${test_current}content-whole-line-7-at-1-name-a-total.expect &&
  generate_operate_run +n -cwlant 8 1 a   ${file_source} > ${test_current}content-whole-line-8-at-1-name-a-total.expect &&
  generate_operate_run +n -cwlant 9 1 a   ${file_source} > ${test_current}content-whole-line-9-at-1-name-a-total.expect &&
  generate_operate_run +n -cwlant 10 1 a  ${file_source} > ${test_current}content-whole-line-10-at-1-name-a-total.expect &&
  generate_operate_run +n -cwlant 20 1 a  ${file_source} > ${test_current}content-whole-line-20-at-1-name-a-total.expect &&
  generate_operate_run +n -cwlant 100 1 a ${file_source} > ${test_current}content-whole-line-100-at-1-name-a-total.expect &&

  generate_operate_run +n -cwnt b         ${file_source} > ${test_current}content-whole-name-b-total.expect &&
  generate_operate_run +n -cwlnt 0 b      ${file_source} > ${test_current}content-whole-line-0-name-b-total.expect &&
  generate_operate_run +n -cwlnt 1 b      ${file_source} > ${test_current}content-whole-line-1-name-b-total.expect &&
  generate_operate_run +n -cwlnt 2 b      ${file_source} > ${test_current}content-whole-line-2-name-b-total.expect &&
  generate_operate_run +n -cwlnt 3 b      ${file_source} > ${test_current}content-whole-line-3-name-b-total.expect &&
  generate_operate_run +n -cwlnt 4 b      ${file_source} > ${test_current}content-whole-line-4-name-b-total.expect &&
  generate_operate_run +n -cwlnt 5 b      ${file_source} > ${test_current}content-whole-line-5-name-b-total.expect &&
  generate_operate_run +n -cwlnt 6 b      ${file_source} > ${test_current}content-whole-line-6-name-b-total.expect &&
  generate_operate_run +n -cwlnt 7 b      ${file_source} > ${test_current}content-whole-line-7-name-b-total.expect &&
  generate_operate_run +n -cwlnt 8 b      ${file_source} > ${test_current}content-whole-line-8-name-b-total.expect &&
  generate_operate_run +n -cwlnt 9 b      ${file_source} > ${test_current}content-whole-line-9-name-b-total.expect &&
  generate_operate_run +n -cwlnt 10 b     ${file_source} > ${test_current}content-whole-line-10-name-b-total.expect &&
  generate_operate_run +n -cwlnt 20 b     ${file_source} > ${test_current}content-whole-line-20-name-b-total.expect &&
  generate_operate_run +n -cwlnt 100 b    ${file_source} > ${test_current}content-whole-line-100-name-b-total.expect &&
  generate_operate_run +n -cwlant 0 0 b   ${file_source} > ${test_current}content-whole-line-0-at-0-name-b-total.expect &&
  generate_operate_run +n -cwlant 1 0 b   ${file_source} > ${test_current}content-whole-line-1-at-0-name-b-total.expect &&
  generate_operate_run +n -cwlant 2 0 b   ${file_source} > ${test_current}content-whole-line-2-at-0-name-b-total.expect &&
  generate_operate_run +n -cwlant 3 0 b   ${file_source} > ${test_current}content-whole-line-3-at-0-name-b-total.expect &&
  generate_operate_run +n -cwlant 4 0 b   ${file_source} > ${test_current}content-whole-line-4-at-0-name-b-total.expect &&
  generate_operate_run +n -cwlant 5 0 b   ${file_source} > ${test_current}content-whole-line-5-at-0-name-b-total.expect &&
  generate_operate_run +n -cwlant 6 0 b   ${file_source} > ${test_current}content-whole-line-6-at-0-name-b-total.expect &&
  generate_operate_run +n -cwlant 7 0 b   ${file_source} > ${test_current}content-whole-line-7-at-0-name-b-total.expect &&
  generate_operate_run +n -cwlant 8 0 b   ${file_source} > ${test_current}content-whole-line-8-at-0-name-b-total.expect &&
  generate_operate_run +n -cwlant 9 0 b   ${file_source} > ${test_current}content-whole-line-9-at-0-name-b-total.expect &&
  generate_operate_run +n -cwlant 10 0 b  ${file_source} > ${test_current}content-whole-line-10-at-0-name-b-total.expect &&
  generate_operate_run +n -cwlant 20 0 b  ${file_source} > ${test_current}content-whole-line-20-at-0-name-b-total.expect &&
  generate_operate_run +n -cwlant 100 0 b ${file_source} > ${test_current}content-whole-line-100-at-0-name-b-total.expect &&
  generate_operate_run +n -cwlant 0 1 b   ${file_source} > ${test_current}content-whole-line-0-at-1-name-b-total.expect &&
  generate_operate_run +n -cwlant 1 1 b   ${file_source} > ${test_current}content-whole-line-1-at-1-name-b-total.expect &&
  generate_operate_run +n -cwlant 2 1 b   ${file_source} > ${test_current}content-whole-line-2-at-1-name-b-total.expect &&
  generate_operate_run +n -cwlant 3 1 b   ${file_source} > ${test_current}content-whole-line-3-at-1-name-b-total.expect &&
  generate_operate_run +n -cwlant 4 1 b   ${file_source} > ${test_current}content-whole-line-4-at-1-name-b-total.expect &&
  generate_operate_run +n -cwlant 5 1 b   ${file_source} > ${test_current}content-whole-line-5-at-1-name-b-total.expect &&
  generate_operate_run +n -cwlant 6 1 b   ${file_source} > ${test_current}content-whole-line-6-at-1-name-b-total.expect &&
  generate_operate_run +n -cwlant 7 1 b   ${file_source} > ${test_current}content-whole-line-7-at-1-name-b-total.expect &&
  generate_operate_run +n -cwlant 8 1 b   ${file_source} > ${test_current}content-whole-line-8-at-1-name-b-total.expect &&
  generate_operate_run +n -cwlant 9 1 b   ${file_source} > ${test_current}content-whole-line-9-at-1-name-b-total.expect &&
  generate_operate_run +n -cwlant 10 1 b  ${file_source} > ${test_current}content-whole-line-10-at-1-name-b-total.expect &&
  generate_operate_run +n -cwlant 20 1 b  ${file_source} > ${test_current}content-whole-line-20-at-1-name-b-total.expect &&
  generate_operate_run +n -cwlant 100 1 b ${file_source} > ${test_current}content-whole-line-100-at-1-name-b-total.expect &&

  generate_operate_run +n -cwnt is         ${file_source} > ${test_current}content-whole-name-is-total.expect &&
  generate_operate_run +n -cwlnt 0 is      ${file_source} > ${test_current}content-whole-line-0-name-is-total.expect &&
  generate_operate_run +n -cwlnt 1 is      ${file_source} > ${test_current}content-whole-line-1-name-is-total.expect &&
  generate_operate_run +n -cwlnt 2 is      ${file_source} > ${test_current}content-whole-line-2-name-is-total.expect &&
  generate_operate_run +n -cwlnt 3 is      ${file_source} > ${test_current}content-whole-line-3-name-is-total.expect &&
  generate_operate_run +n -cwlnt 4 is      ${file_source} > ${test_current}content-whole-line-4-name-is-total.expect &&
  generate_operate_run +n -cwlnt 5 is      ${file_source} > ${test_current}content-whole-line-5-name-is-total.expect &&
  generate_operate_run +n -cwlnt 6 is      ${file_source} > ${test_current}content-whole-line-6-name-is-total.expect &&
  generate_operate_run +n -cwlnt 7 is      ${file_source} > ${test_current}content-whole-line-7-name-is-total.expect &&
  generate_operate_run +n -cwlnt 8 is      ${file_source} > ${test_current}content-whole-line-8-name-is-total.expect &&
  generate_operate_run +n -cwlnt 9 is      ${file_source} > ${test_current}content-whole-line-9-name-is-total.expect &&
  generate_operate_run +n -cwlnt 10 is     ${file_source} > ${test_current}content-whole-line-10-name-is-total.expect &&
  generate_operate_run +n -cwlnt 20 is     ${file_source} > ${test_current}content-whole-line-20-name-is-total.expect &&
  generate_operate_run +n -cwlnt 100 is    ${file_source} > ${test_current}content-whole-line-100-name-is-total.expect &&
  generate_operate_run +n -cwlant 0 0 is   ${file_source} > ${test_current}content-whole-line-0-at-0-name-is-total.expect &&
  generate_operate_run +n -cwlant 1 0 is   ${file_source} > ${test_current}content-whole-line-1-at-0-name-is-total.expect &&
  generate_operate_run +n -cwlant 2 0 is   ${file_source} > ${test_current}content-whole-line-2-at-0-name-is-total.expect &&
  generate_operate_run +n -cwlant 3 0 is   ${file_source} > ${test_current}content-whole-line-3-at-0-name-is-total.expect &&
  generate_operate_run +n -cwlant 4 0 is   ${file_source} > ${test_current}content-whole-line-4-at-0-name-is-total.expect &&
  generate_operate_run +n -cwlant 5 0 is   ${file_source} > ${test_current}content-whole-line-5-at-0-name-is-total.expect &&
  generate_operate_run +n -cwlant 6 0 is   ${file_source} > ${test_current}content-whole-line-6-at-0-name-is-total.expect &&
  generate_operate_run +n -cwlant 7 0 is   ${file_source} > ${test_current}content-whole-line-7-at-0-name-is-total.expect &&
  generate_operate_run +n -cwlant 8 0 is   ${file_source} > ${test_current}content-whole-line-8-at-0-name-is-total.expect &&
  generate_operate_run +n -cwlant 9 0 is   ${file_source} > ${test_current}content-whole-line-9-at-0-name-is-total.expect &&
  generate_operate_run +n -cwlant 10 0 is  ${file_source} > ${test_current}content-whole-line-10-at-0-name-is-total.expect &&
  generate_operate_run +n -cwlant 20 0 is  ${file_source} > ${test_current}content-whole-line-20-at-0-name-is-total.expect &&
  generate_operate_run +n -cwlant 100 0 is ${file_source} > ${test_current}content-whole-line-100-at-0-name-is-total.expect &&
  generate_operate_run +n -cwlant 0 1 is   ${file_source} > ${test_current}content-whole-line-0-at-1-name-is-total.expect &&
  generate_operate_run +n -cwlant 1 1 is   ${file_source} > ${test_current}content-whole-line-1-at-1-name-is-total.expect &&
  generate_operate_run +n -cwlant 2 1 is   ${file_source} > ${test_current}content-whole-line-2-at-1-name-is-total.expect &&
  generate_operate_run +n -cwlant 3 1 is   ${file_source} > ${test_current}content-whole-line-3-at-1-name-is-total.expect &&
  generate_operate_run +n -cwlant 4 1 is   ${file_source} > ${test_current}content-whole-line-4-at-1-name-is-total.expect &&
  generate_operate_run +n -cwlant 5 1 is   ${file_source} > ${test_current}content-whole-line-5-at-1-name-is-total.expect &&
  generate_operate_run +n -cwlant 6 1 is   ${file_source} > ${test_current}content-whole-line-6-at-1-name-is-total.expect &&
  generate_operate_run +n -cwlant 7 1 is   ${file_source} > ${test_current}content-whole-line-7-at-1-name-is-total.expect &&
  generate_operate_run +n -cwlant 8 1 is   ${file_source} > ${test_current}content-whole-line-8-at-1-name-is-total.expect &&
  generate_operate_run +n -cwlant 9 1 is   ${file_source} > ${test_current}content-whole-line-9-at-1-name-is-total.expect &&
  generate_operate_run +n -cwlant 10 1 is  ${file_source} > ${test_current}content-whole-line-10-at-1-name-is-total.expect &&
  generate_operate_run +n -cwlant 20 1 is  ${file_source} > ${test_current}content-whole-line-20-at-1-name-is-total.expect &&
  generate_operate_run +n -cwlant 100 1 is ${file_source} > ${test_current}content-whole-line-100-at-1-name-is-total.expect &&

  generate_operate_run +n -cwnt yup         ${file_source} > ${test_current}content-whole-name-yup-total.expect &&
  generate_operate_run +n -cwlnt 0 yup      ${file_source} > ${test_current}content-whole-line-0-name-yup-total.expect &&
  generate_operate_run +n -cwlnt 1 yup      ${file_source} > ${test_current}content-whole-line-1-name-yup-total.expect &&
  generate_operate_run +n -cwlnt 2 yup      ${file_source} > ${test_current}content-whole-line-2-name-yup-total.expect &&
  generate_operate_run +n -cwlnt 3 yup      ${file_source} > ${test_current}content-whole-line-3-name-yup-total.expect &&
  generate_operate_run +n -cwlnt 4 yup      ${file_source} > ${test_current}content-whole-line-4-name-yup-total.expect &&
  generate_operate_run +n -cwlnt 5 yup      ${file_source} > ${test_current}content-whole-line-5-name-yup-total.expect &&
  generate_operate_run +n -cwlnt 6 yup      ${file_source} > ${test_current}content-whole-line-6-name-yup-total.expect &&
  generate_operate_run +n -cwlnt 7 yup      ${file_source} > ${test_current}content-whole-line-7-name-yup-total.expect &&
  generate_operate_run +n -cwlnt 8 yup      ${file_source} > ${test_current}content-whole-line-8-name-yup-total.expect &&
  generate_operate_run +n -cwlnt 9 yup      ${file_source} > ${test_current}content-whole-line-9-name-yup-total.expect &&
  generate_operate_run +n -cwlnt 10 yup     ${file_source} > ${test_current}content-whole-line-10-name-yup-total.expect &&
  generate_operate_run +n -cwlnt 20 yup     ${file_source} > ${test_current}content-whole-line-20-name-yup-total.expect &&
  generate_operate_run +n -cwlnt 100 yup    ${file_source} > ${test_current}content-whole-line-100-name-yup-total.expect &&
  generate_operate_run +n -cwlant 0 0 yup   ${file_source} > ${test_current}content-whole-line-0-at-0-name-yup-total.expect &&
  generate_operate_run +n -cwlant 1 0 yup   ${file_source} > ${test_current}content-whole-line-1-at-0-name-yup-total.expect &&
  generate_operate_run +n -cwlant 2 0 yup   ${file_source} > ${test_current}content-whole-line-2-at-0-name-yup-total.expect &&
  generate_operate_run +n -cwlant 3 0 yup   ${file_source} > ${test_current}content-whole-line-3-at-0-name-yup-total.expect &&
  generate_operate_run +n -cwlant 4 0 yup   ${file_source} > ${test_current}content-whole-line-4-at-0-name-yup-total.expect &&
  generate_operate_run +n -cwlant 5 0 yup   ${file_source} > ${test_current}content-whole-line-5-at-0-name-yup-total.expect &&
  generate_operate_run +n -cwlant 6 0 yup   ${file_source} > ${test_current}content-whole-line-6-at-0-name-yup-total.expect &&
  generate_operate_run +n -cwlant 7 0 yup   ${file_source} > ${test_current}content-whole-line-7-at-0-name-yup-total.expect &&
  generate_operate_run +n -cwlant 8 0 yup   ${file_source} > ${test_current}content-whole-line-8-at-0-name-yup-total.expect &&
  generate_operate_run +n -cwlant 9 0 yup   ${file_source} > ${test_current}content-whole-line-9-at-0-name-yup-total.expect &&
  generate_operate_run +n -cwlant 10 0 yup  ${file_source} > ${test_current}content-whole-line-10-at-0-name-yup-total.expect &&
  generate_operate_run +n -cwlant 20 0 yup  ${file_source} > ${test_current}content-whole-line-20-at-0-name-yup-total.expect &&
  generate_operate_run +n -cwlant 100 0 yup ${file_source} > ${test_current}content-whole-line-100-at-0-name-yup-total.expect &&
  generate_operate_run +n -cwlant 0 1 yup   ${file_source} > ${test_current}content-whole-line-0-at-1-name-yup-total.expect &&
  generate_operate_run +n -cwlant 1 1 yup   ${file_source} > ${test_current}content-whole-line-1-at-1-name-yup-total.expect &&
  generate_operate_run +n -cwlant 2 1 yup   ${file_source} > ${test_current}content-whole-line-2-at-1-name-yup-total.expect &&
  generate_operate_run +n -cwlant 3 1 yup   ${file_source} > ${test_current}content-whole-line-3-at-1-name-yup-total.expect &&
  generate_operate_run +n -cwlant 4 1 yup   ${file_source} > ${test_current}content-whole-line-4-at-1-name-yup-total.expect &&
  generate_operate_run +n -cwlant 5 1 yup   ${file_source} > ${test_current}content-whole-line-5-at-1-name-yup-total.expect &&
  generate_operate_run +n -cwlant 6 1 yup   ${file_source} > ${test_current}content-whole-line-6-at-1-name-yup-total.expect &&
  generate_operate_run +n -cwlant 7 1 yup   ${file_source} > ${test_current}content-whole-line-7-at-1-name-yup-total.expect &&
  generate_operate_run +n -cwlant 8 1 yup   ${file_source} > ${test_current}content-whole-line-8-at-1-name-yup-total.expect &&
  generate_operate_run +n -cwlant 9 1 yup   ${file_source} > ${test_current}content-whole-line-9-at-1-name-yup-total.expect &&
  generate_operate_run +n -cwlant 10 1 yup  ${file_source} > ${test_current}content-whole-line-10-at-1-name-yup-total.expect &&
  generate_operate_run +n -cwlant 20 1 yup  ${file_source} > ${test_current}content-whole-line-20-at-1-name-yup-total.expect &&
  generate_operate_run +n -cwlant 100 1 yup ${file_source} > ${test_current}content-whole-line-100-at-1-name-yup-total.expect &&
  generate_operate_run +n -cwlant 100 1 yup ${file_source} > ${test_current}content-whole-line-100-at-1-name-yup-total.expect &&

  #### OBJECT ####
  generate_operate_run +n -o         ${file_source} > ${test_current}content.expect &&
  generate_operate_run +n -ol 0      ${file_source} > ${test_current}object-line-0.expect &&
  generate_operate_run +n -ol 1      ${file_source} > ${test_current}object-line-1.expect &&
  generate_operate_run +n -ol 2      ${file_source} > ${test_current}object-line-2.expect &&
  generate_operate_run +n -ol 3      ${file_source} > ${test_current}object-line-3.expect &&
  generate_operate_run +n -ol 4      ${file_source} > ${test_current}object-line-4.expect &&
  generate_operate_run +n -ol 5      ${file_source} > ${test_current}object-line-5.expect &&
  generate_operate_run +n -ol 6      ${file_source} > ${test_current}object-line-6.expect &&
  generate_operate_run +n -ol 7      ${file_source} > ${test_current}object-line-7.expect &&
  generate_operate_run +n -ol 8      ${file_source} > ${test_current}object-line-8.expect &&
  generate_operate_run +n -ol 9      ${file_source} > ${test_current}object-line-9.expect &&
  generate_operate_run +n -ol 10     ${file_source} > ${test_current}object-line-10.expect &&
  generate_operate_run +n -ol 20     ${file_source} > ${test_current}object-line-20.expect &&
  generate_operate_run +n -ol 100    ${file_source} > ${test_current}object-line-100.expect &&
  generate_operate_run +n -ola 0 0   ${file_source} > ${test_current}object-line-0-at-0.expect &&
  generate_operate_run +n -ola 1 0   ${file_source} > ${test_current}object-line-1-at-0.expect &&
  generate_operate_run +n -ola 2 0   ${file_source} > ${test_current}object-line-2-at-0.expect &&
  generate_operate_run +n -ola 3 0   ${file_source} > ${test_current}object-line-3-at-0.expect &&
  generate_operate_run +n -ola 4 0   ${file_source} > ${test_current}object-line-4-at-0.expect &&
  generate_operate_run +n -ola 5 0   ${file_source} > ${test_current}object-line-5-at-0.expect &&
  generate_operate_run +n -ola 6 0   ${file_source} > ${test_current}object-line-6-at-0.expect &&
  generate_operate_run +n -ola 7 0   ${file_source} > ${test_current}object-line-7-at-0.expect &&
  generate_operate_run +n -ola 8 0   ${file_source} > ${test_current}object-line-8-at-0.expect &&
  generate_operate_run +n -ola 9 0   ${file_source} > ${test_current}object-line-9-at-0.expect &&
  generate_operate_run +n -ola 10 0  ${file_source} > ${test_current}object-line-10-at-0.expect &&
  generate_operate_run +n -ola 20 0  ${file_source} > ${test_current}object-line-20-at-0.expect &&
  generate_operate_run +n -ola 100 0 ${file_source} > ${test_current}object-line-100-at-0.expect &&
  generate_operate_run +n -ola 0 1   ${file_source} > ${test_current}object-line-0-at-1.expect &&
  generate_operate_run +n -ola 1 1   ${file_source} > ${test_current}object-line-1-at-1.expect &&
  generate_operate_run +n -ola 2 1   ${file_source} > ${test_current}object-line-2-at-1.expect &&
  generate_operate_run +n -ola 3 1   ${file_source} > ${test_current}object-line-3-at-1.expect &&
  generate_operate_run +n -ola 4 1   ${file_source} > ${test_current}object-line-4-at-1.expect &&
  generate_operate_run +n -ola 5 1   ${file_source} > ${test_current}object-line-5-at-1.expect &&
  generate_operate_run +n -ola 6 1   ${file_source} > ${test_current}object-line-6-at-1.expect &&
  generate_operate_run +n -ola 7 1   ${file_source} > ${test_current}object-line-7-at-1.expect &&
  generate_operate_run +n -ola 8 1   ${file_source} > ${test_current}object-line-8-at-1.expect &&
  generate_operate_run +n -ola 9 1   ${file_source} > ${test_current}object-line-9-at-1.expect &&
  generate_operate_run +n -ola 10 1  ${file_source} > ${test_current}object-line-10-at-1.expect &&
  generate_operate_run +n -ola 20 1  ${file_source} > ${test_current}object-line-20-at-1.expect &&
  generate_operate_run +n -ola 100 1 ${file_source} > ${test_current}object-line-100-at-1.expect &&

  generate_operate_run +n -on a         ${file_source} > ${test_current}object-name-a.expect &&
  generate_operate_run +n -oln 0 a      ${file_source} > ${test_current}object-line-0-name-a.expect &&
  generate_operate_run +n -oln 1 a      ${file_source} > ${test_current}object-line-1-name-a.expect &&
  generate_operate_run +n -oln 2 a      ${file_source} > ${test_current}object-line-2-name-a.expect &&
  generate_operate_run +n -oln 3 a      ${file_source} > ${test_current}object-line-3-name-a.expect &&
  generate_operate_run +n -oln 4 a      ${file_source} > ${test_current}object-line-4-name-a.expect &&
  generate_operate_run +n -oln 5 a      ${file_source} > ${test_current}object-line-5-name-a.expect &&
  generate_operate_run +n -oln 6 a      ${file_source} > ${test_current}object-line-6-name-a.expect &&
  generate_operate_run +n -oln 7 a      ${file_source} > ${test_current}object-line-7-name-a.expect &&
  generate_operate_run +n -oln 8 a      ${file_source} > ${test_current}object-line-8-name-a.expect &&
  generate_operate_run +n -oln 9 a      ${file_source} > ${test_current}object-line-9-name-a.expect &&
  generate_operate_run +n -oln 10 a     ${file_source} > ${test_current}object-line-10-name-a.expect &&
  generate_operate_run +n -oln 20 a     ${file_source} > ${test_current}object-line-20-name-a.expect &&
  generate_operate_run +n -oln 100 a    ${file_source} > ${test_current}object-line-100-name-a.expect &&
  generate_operate_run +n -olan 0 0 a   ${file_source} > ${test_current}object-line-0-at-0-name-a.expect &&
  generate_operate_run +n -olan 1 0 a   ${file_source} > ${test_current}object-line-1-at-0-name-a.expect &&
  generate_operate_run +n -olan 2 0 a   ${file_source} > ${test_current}object-line-2-at-0-name-a.expect &&
  generate_operate_run +n -olan 3 0 a   ${file_source} > ${test_current}object-line-3-at-0-name-a.expect &&
  generate_operate_run +n -olan 4 0 a   ${file_source} > ${test_current}object-line-4-at-0-name-a.expect &&
  generate_operate_run +n -olan 5 0 a   ${file_source} > ${test_current}object-line-5-at-0-name-a.expect &&
  generate_operate_run +n -olan 6 0 a   ${file_source} > ${test_current}object-line-6-at-0-name-a.expect &&
  generate_operate_run +n -olan 7 0 a   ${file_source} > ${test_current}object-line-7-at-0-name-a.expect &&
  generate_operate_run +n -olan 8 0 a   ${file_source} > ${test_current}object-line-8-at-0-name-a.expect &&
  generate_operate_run +n -olan 9 0 a   ${file_source} > ${test_current}object-line-9-at-0-name-a.expect &&
  generate_operate_run +n -olan 10 0 a  ${file_source} > ${test_current}object-line-10-at-0-name-a.expect &&
  generate_operate_run +n -olan 20 0 a  ${file_source} > ${test_current}object-line-20-at-0-name-a.expect &&
  generate_operate_run +n -olan 100 0 a ${file_source} > ${test_current}object-line-100-at-0-name-a.expect &&
  generate_operate_run +n -olan 0 1 a   ${file_source} > ${test_current}object-line-0-at-1-name-a.expect &&
  generate_operate_run +n -olan 1 1 a   ${file_source} > ${test_current}object-line-1-at-1-name-a.expect &&
  generate_operate_run +n -olan 2 1 a   ${file_source} > ${test_current}object-line-2-at-1-name-a.expect &&
  generate_operate_run +n -olan 3 1 a   ${file_source} > ${test_current}object-line-3-at-1-name-a.expect &&
  generate_operate_run +n -olan 4 1 a   ${file_source} > ${test_current}object-line-4-at-1-name-a.expect &&
  generate_operate_run +n -olan 5 1 a   ${file_source} > ${test_current}object-line-5-at-1-name-a.expect &&
  generate_operate_run +n -olan 6 1 a   ${file_source} > ${test_current}object-line-6-at-1-name-a.expect &&
  generate_operate_run +n -olan 7 1 a   ${file_source} > ${test_current}object-line-7-at-1-name-a.expect &&
  generate_operate_run +n -olan 8 1 a   ${file_source} > ${test_current}object-line-8-at-1-name-a.expect &&
  generate_operate_run +n -olan 9 1 a   ${file_source} > ${test_current}object-line-9-at-1-name-a.expect &&
  generate_operate_run +n -olan 10 1 a  ${file_source} > ${test_current}object-line-10-at-1-name-a.expect &&
  generate_operate_run +n -olan 20 1 a  ${file_source} > ${test_current}object-line-20-at-1-name-a.expect &&
  generate_operate_run +n -olan 100 1 a ${file_source} > ${test_current}object-line-100-at-1-name-a.expect &&

  generate_operate_run +n -on b         ${file_source} > ${test_current}object-name-b.expect &&
  generate_operate_run +n -oln 0 b      ${file_source} > ${test_current}object-line-0-name-b.expect &&
  generate_operate_run +n -oln 1 b      ${file_source} > ${test_current}object-line-1-name-b.expect &&
  generate_operate_run +n -oln 2 b      ${file_source} > ${test_current}object-line-2-name-b.expect &&
  generate_operate_run +n -oln 3 b      ${file_source} > ${test_current}object-line-3-name-b.expect &&
  generate_operate_run +n -oln 4 b      ${file_source} > ${test_current}object-line-4-name-b.expect &&
  generate_operate_run +n -oln 5 b      ${file_source} > ${test_current}object-line-5-name-b.expect &&
  generate_operate_run +n -oln 6 b      ${file_source} > ${test_current}object-line-6-name-b.expect &&
  generate_operate_run +n -oln 7 b      ${file_source} > ${test_current}object-line-7-name-b.expect &&
  generate_operate_run +n -oln 8 b      ${file_source} > ${test_current}object-line-8-name-b.expect &&
  generate_operate_run +n -oln 9 b      ${file_source} > ${test_current}object-line-9-name-b.expect &&
  generate_operate_run +n -oln 10 b     ${file_source} > ${test_current}object-line-10-name-b.expect &&
  generate_operate_run +n -oln 20 b     ${file_source} > ${test_current}object-line-20-name-b.expect &&
  generate_operate_run +n -oln 100 b    ${file_source} > ${test_current}object-line-100-name-b.expect &&
  generate_operate_run +n -olan 0 0 b   ${file_source} > ${test_current}object-line-0-at-0-name-b.expect &&
  generate_operate_run +n -olan 1 0 b   ${file_source} > ${test_current}object-line-1-at-0-name-b.expect &&
  generate_operate_run +n -olan 2 0 b   ${file_source} > ${test_current}object-line-2-at-0-name-b.expect &&
  generate_operate_run +n -olan 3 0 b   ${file_source} > ${test_current}object-line-3-at-0-name-b.expect &&
  generate_operate_run +n -olan 4 0 b   ${file_source} > ${test_current}object-line-4-at-0-name-b.expect &&
  generate_operate_run +n -olan 5 0 b   ${file_source} > ${test_current}object-line-5-at-0-name-b.expect &&
  generate_operate_run +n -olan 6 0 b   ${file_source} > ${test_current}object-line-6-at-0-name-b.expect &&
  generate_operate_run +n -olan 7 0 b   ${file_source} > ${test_current}object-line-7-at-0-name-b.expect &&
  generate_operate_run +n -olan 8 0 b   ${file_source} > ${test_current}object-line-8-at-0-name-b.expect &&
  generate_operate_run +n -olan 9 0 b   ${file_source} > ${test_current}object-line-9-at-0-name-b.expect &&
  generate_operate_run +n -olan 10 0 b  ${file_source} > ${test_current}object-line-10-at-0-name-b.expect &&
  generate_operate_run +n -olan 20 0 b  ${file_source} > ${test_current}object-line-20-at-0-name-b.expect &&
  generate_operate_run +n -olan 100 0 b ${file_source} > ${test_current}object-line-100-at-0-name-b.expect &&
  generate_operate_run +n -olan 0 1 b   ${file_source} > ${test_current}object-line-0-at-1-name-b.expect &&
  generate_operate_run +n -olan 1 1 b   ${file_source} > ${test_current}object-line-1-at-1-name-b.expect &&
  generate_operate_run +n -olan 2 1 b   ${file_source} > ${test_current}object-line-2-at-1-name-b.expect &&
  generate_operate_run +n -olan 3 1 b   ${file_source} > ${test_current}object-line-3-at-1-name-b.expect &&
  generate_operate_run +n -olan 4 1 b   ${file_source} > ${test_current}object-line-4-at-1-name-b.expect &&
  generate_operate_run +n -olan 5 1 b   ${file_source} > ${test_current}object-line-5-at-1-name-b.expect &&
  generate_operate_run +n -olan 6 1 b   ${file_source} > ${test_current}object-line-6-at-1-name-b.expect &&
  generate_operate_run +n -olan 7 1 b   ${file_source} > ${test_current}object-line-7-at-1-name-b.expect &&
  generate_operate_run +n -olan 8 1 b   ${file_source} > ${test_current}object-line-8-at-1-name-b.expect &&
  generate_operate_run +n -olan 9 1 b   ${file_source} > ${test_current}object-line-9-at-1-name-b.expect &&
  generate_operate_run +n -olan 10 1 b  ${file_source} > ${test_current}object-line-10-at-1-name-b.expect &&
  generate_operate_run +n -olan 20 1 b  ${file_source} > ${test_current}object-line-20-at-1-name-b.expect &&
  generate_operate_run +n -olan 100 1 b ${file_source} > ${test_current}object-line-100-at-1-name-b.expect &&

  generate_operate_run +n -on is         ${file_source} > ${test_current}object-name-is.expect &&
  generate_operate_run +n -oln 0 is      ${file_source} > ${test_current}object-line-0-name-is.expect &&
  generate_operate_run +n -oln 1 is      ${file_source} > ${test_current}object-line-1-name-is.expect &&
  generate_operate_run +n -oln 2 is      ${file_source} > ${test_current}object-line-2-name-is.expect &&
  generate_operate_run +n -oln 3 is      ${file_source} > ${test_current}object-line-3-name-is.expect &&
  generate_operate_run +n -oln 4 is      ${file_source} > ${test_current}object-line-4-name-is.expect &&
  generate_operate_run +n -oln 5 is      ${file_source} > ${test_current}object-line-5-name-is.expect &&
  generate_operate_run +n -oln 6 is      ${file_source} > ${test_current}object-line-6-name-is.expect &&
  generate_operate_run +n -oln 7 is      ${file_source} > ${test_current}object-line-7-name-is.expect &&
  generate_operate_run +n -oln 8 is      ${file_source} > ${test_current}object-line-8-name-is.expect &&
  generate_operate_run +n -oln 9 is      ${file_source} > ${test_current}object-line-9-name-is.expect &&
  generate_operate_run +n -oln 10 is     ${file_source} > ${test_current}object-line-10-name-is.expect &&
  generate_operate_run +n -oln 20 is     ${file_source} > ${test_current}object-line-20-name-is.expect &&
  generate_operate_run +n -oln 100 is    ${file_source} > ${test_current}object-line-100-name-is.expect &&
  generate_operate_run +n -olan 0 0 is   ${file_source} > ${test_current}object-line-0-at-0-name-is.expect &&
  generate_operate_run +n -olan 1 0 is   ${file_source} > ${test_current}object-line-1-at-0-name-is.expect &&
  generate_operate_run +n -olan 2 0 is   ${file_source} > ${test_current}object-line-2-at-0-name-is.expect &&
  generate_operate_run +n -olan 3 0 is   ${file_source} > ${test_current}object-line-3-at-0-name-is.expect &&
  generate_operate_run +n -olan 4 0 is   ${file_source} > ${test_current}object-line-4-at-0-name-is.expect &&
  generate_operate_run +n -olan 5 0 is   ${file_source} > ${test_current}object-line-5-at-0-name-is.expect &&
  generate_operate_run +n -olan 6 0 is   ${file_source} > ${test_current}object-line-6-at-0-name-is.expect &&
  generate_operate_run +n -olan 7 0 is   ${file_source} > ${test_current}object-line-7-at-0-name-is.expect &&
  generate_operate_run +n -olan 8 0 is   ${file_source} > ${test_current}object-line-8-at-0-name-is.expect &&
  generate_operate_run +n -olan 9 0 is   ${file_source} > ${test_current}object-line-9-at-0-name-is.expect &&
  generate_operate_run +n -olan 10 0 is  ${file_source} > ${test_current}object-line-10-at-0-name-is.expect &&
  generate_operate_run +n -olan 20 0 is  ${file_source} > ${test_current}object-line-20-at-0-name-is.expect &&
  generate_operate_run +n -olan 100 0 is ${file_source} > ${test_current}object-line-100-at-0-name-is.expect &&
  generate_operate_run +n -olan 0 1 is   ${file_source} > ${test_current}object-line-0-at-1-name-is.expect &&
  generate_operate_run +n -olan 1 1 is   ${file_source} > ${test_current}object-line-1-at-1-name-is.expect &&
  generate_operate_run +n -olan 2 1 is   ${file_source} > ${test_current}object-line-2-at-1-name-is.expect &&
  generate_operate_run +n -olan 3 1 is   ${file_source} > ${test_current}object-line-3-at-1-name-is.expect &&
  generate_operate_run +n -olan 4 1 is   ${file_source} > ${test_current}object-line-4-at-1-name-is.expect &&
  generate_operate_run +n -olan 5 1 is   ${file_source} > ${test_current}object-line-5-at-1-name-is.expect &&
  generate_operate_run +n -olan 6 1 is   ${file_source} > ${test_current}object-line-6-at-1-name-is.expect &&
  generate_operate_run +n -olan 7 1 is   ${file_source} > ${test_current}object-line-7-at-1-name-is.expect &&
  generate_operate_run +n -olan 8 1 is   ${file_source} > ${test_current}object-line-8-at-1-name-is.expect &&
  generate_operate_run +n -olan 9 1 is   ${file_source} > ${test_current}object-line-9-at-1-name-is.expect &&
  generate_operate_run +n -olan 10 1 is  ${file_source} > ${test_current}object-line-10-at-1-name-is.expect &&
  generate_operate_run +n -olan 20 1 is  ${file_source} > ${test_current}object-line-20-at-1-name-is.expect &&
  generate_operate_run +n -olan 100 1 is ${file_source} > ${test_current}object-line-100-at-1-name-is.expect &&

  generate_operate_run +n -on yup         ${file_source} > ${test_current}object-name-yup.expect &&
  generate_operate_run +n -oln 0 yup      ${file_source} > ${test_current}object-line-0-name-yup.expect &&
  generate_operate_run +n -oln 1 yup      ${file_source} > ${test_current}object-line-1-name-yup.expect &&
  generate_operate_run +n -oln 2 yup      ${file_source} > ${test_current}object-line-2-name-yup.expect &&
  generate_operate_run +n -oln 3 yup      ${file_source} > ${test_current}object-line-3-name-yup.expect &&
  generate_operate_run +n -oln 4 yup      ${file_source} > ${test_current}object-line-4-name-yup.expect &&
  generate_operate_run +n -oln 5 yup      ${file_source} > ${test_current}object-line-5-name-yup.expect &&
  generate_operate_run +n -oln 6 yup      ${file_source} > ${test_current}object-line-6-name-yup.expect &&
  generate_operate_run +n -oln 7 yup      ${file_source} > ${test_current}object-line-7-name-yup.expect &&
  generate_operate_run +n -oln 8 yup      ${file_source} > ${test_current}object-line-8-name-yup.expect &&
  generate_operate_run +n -oln 9 yup      ${file_source} > ${test_current}object-line-9-name-yup.expect &&
  generate_operate_run +n -oln 10 yup     ${file_source} > ${test_current}object-line-10-name-yup.expect &&
  generate_operate_run +n -oln 20 yup     ${file_source} > ${test_current}object-line-20-name-yup.expect &&
  generate_operate_run +n -oln 100 yup    ${file_source} > ${test_current}object-line-100-name-yup.expect &&
  generate_operate_run +n -olan 0 0 yup   ${file_source} > ${test_current}object-line-0-at-0-name-yup.expect &&
  generate_operate_run +n -olan 1 0 yup   ${file_source} > ${test_current}object-line-1-at-0-name-yup.expect &&
  generate_operate_run +n -olan 2 0 yup   ${file_source} > ${test_current}object-line-2-at-0-name-yup.expect &&
  generate_operate_run +n -olan 3 0 yup   ${file_source} > ${test_current}object-line-3-at-0-name-yup.expect &&
  generate_operate_run +n -olan 4 0 yup   ${file_source} > ${test_current}object-line-4-at-0-name-yup.expect &&
  generate_operate_run +n -olan 5 0 yup   ${file_source} > ${test_current}object-line-5-at-0-name-yup.expect &&
  generate_operate_run +n -olan 6 0 yup   ${file_source} > ${test_current}object-line-6-at-0-name-yup.expect &&
  generate_operate_run +n -olan 7 0 yup   ${file_source} > ${test_current}object-line-7-at-0-name-yup.expect &&
  generate_operate_run +n -olan 8 0 yup   ${file_source} > ${test_current}object-line-8-at-0-name-yup.expect &&
  generate_operate_run +n -olan 9 0 yup   ${file_source} > ${test_current}object-line-9-at-0-name-yup.expect &&
  generate_operate_run +n -olan 10 0 yup  ${file_source} > ${test_current}object-line-10-at-0-name-yup.expect &&
  generate_operate_run +n -olan 20 0 yup  ${file_source} > ${test_current}object-line-20-at-0-name-yup.expect &&
  generate_operate_run +n -olan 100 0 yup ${file_source} > ${test_current}object-line-100-at-0-name-yup.expect &&
  generate_operate_run +n -olan 0 1 yup   ${file_source} > ${test_current}object-line-0-at-1-name-yup.expect &&
  generate_operate_run +n -olan 1 1 yup   ${file_source} > ${test_current}object-line-1-at-1-name-yup.expect &&
  generate_operate_run +n -olan 2 1 yup   ${file_source} > ${test_current}object-line-2-at-1-name-yup.expect &&
  generate_operate_run +n -olan 3 1 yup   ${file_source} > ${test_current}object-line-3-at-1-name-yup.expect &&
  generate_operate_run +n -olan 4 1 yup   ${file_source} > ${test_current}object-line-4-at-1-name-yup.expect &&
  generate_operate_run +n -olan 5 1 yup   ${file_source} > ${test_current}object-line-5-at-1-name-yup.expect &&
  generate_operate_run +n -olan 6 1 yup   ${file_source} > ${test_current}object-line-6-at-1-name-yup.expect &&
  generate_operate_run +n -olan 7 1 yup   ${file_source} > ${test_current}object-line-7-at-1-name-yup.expect &&
  generate_operate_run +n -olan 8 1 yup   ${file_source} > ${test_current}object-line-8-at-1-name-yup.expect &&
  generate_operate_run +n -olan 9 1 yup   ${file_source} > ${test_current}object-line-9-at-1-name-yup.expect &&
  generate_operate_run +n -olan 10 1 yup  ${file_source} > ${test_current}object-line-10-at-1-name-yup.expect &&
  generate_operate_run +n -olan 20 1 yup  ${file_source} > ${test_current}object-line-20-at-1-name-yup.expect &&

  #### OBJECT - TOTAL ####
  generate_operate_run +n -ot         ${file_source} > ${test_current}object-total.expect &&
  generate_operate_run +n -olt 0      ${file_source} > ${test_current}object-line-0-total.expect &&
  generate_operate_run +n -olt 1      ${file_source} > ${test_current}object-line-1-total.expect &&
  generate_operate_run +n -olt 2      ${file_source} > ${test_current}object-line-2-total.expect &&
  generate_operate_run +n -olt 3      ${file_source} > ${test_current}object-line-3-total.expect &&
  generate_operate_run +n -olt 4      ${file_source} > ${test_current}object-line-4-total.expect &&
  generate_operate_run +n -olt 5      ${file_source} > ${test_current}object-line-5-total.expect &&
  generate_operate_run +n -olt 6      ${file_source} > ${test_current}object-line-6-total.expect &&
  generate_operate_run +n -olt 7      ${file_source} > ${test_current}object-line-7-total.expect &&
  generate_operate_run +n -olt 8      ${file_source} > ${test_current}object-line-8-total.expect &&
  generate_operate_run +n -olt 9      ${file_source} > ${test_current}object-line-9-total.expect &&
  generate_operate_run +n -olt 10     ${file_source} > ${test_current}object-line-10-total.expect &&
  generate_operate_run +n -olt 20     ${file_source} > ${test_current}object-line-20-total.expect &&
  generate_operate_run +n -olt 100    ${file_source} > ${test_current}object-line-100-total.expect &&
  generate_operate_run +n -olat 0 0   ${file_source} > ${test_current}object-line-0-at-0-total.expect &&
  generate_operate_run +n -olat 1 0   ${file_source} > ${test_current}object-line-1-at-0-total.expect &&
  generate_operate_run +n -olat 2 0   ${file_source} > ${test_current}object-line-2-at-0-total.expect &&
  generate_operate_run +n -olat 3 0   ${file_source} > ${test_current}object-line-3-at-0-total.expect &&
  generate_operate_run +n -olat 4 0   ${file_source} > ${test_current}object-line-4-at-0-total.expect &&
  generate_operate_run +n -olat 5 0   ${file_source} > ${test_current}object-line-5-at-0-total.expect &&
  generate_operate_run +n -olat 6 0   ${file_source} > ${test_current}object-line-6-at-0-total.expect &&
  generate_operate_run +n -olat 7 0   ${file_source} > ${test_current}object-line-7-at-0-total.expect &&
  generate_operate_run +n -olat 8 0   ${file_source} > ${test_current}object-line-8-at-0-total.expect &&
  generate_operate_run +n -olat 9 0   ${file_source} > ${test_current}object-line-9-at-0-total.expect &&
  generate_operate_run +n -olat 10 0  ${file_source} > ${test_current}object-line-10-at-0-total.expect &&
  generate_operate_run +n -olat 20 0  ${file_source} > ${test_current}object-line-20-at-0-total.expect &&
  generate_operate_run +n -olat 100 0 ${file_source} > ${test_current}object-line-100-at-0-total.expect &&
  generate_operate_run +n -olat 0 1   ${file_source} > ${test_current}object-line-0-at-1-total.expect &&
  generate_operate_run +n -olat 1 1   ${file_source} > ${test_current}object-line-1-at-1-total.expect &&
  generate_operate_run +n -olat 2 1   ${file_source} > ${test_current}object-line-2-at-1-total.expect &&
  generate_operate_run +n -olat 3 1   ${file_source} > ${test_current}object-line-3-at-1-total.expect &&
  generate_operate_run +n -olat 4 1   ${file_source} > ${test_current}object-line-4-at-1-total.expect &&
  generate_operate_run +n -olat 5 1   ${file_source} > ${test_current}object-line-5-at-1-total.expect &&
  generate_operate_run +n -olat 6 1   ${file_source} > ${test_current}object-line-6-at-1-total.expect &&
  generate_operate_run +n -olat 7 1   ${file_source} > ${test_current}object-line-7-at-1-total.expect &&
  generate_operate_run +n -olat 8 1   ${file_source} > ${test_current}object-line-8-at-1-total.expect &&
  generate_operate_run +n -olat 9 1   ${file_source} > ${test_current}object-line-9-at-1-total.expect &&
  generate_operate_run +n -olat 10 1  ${file_source} > ${test_current}object-line-10-at-1-total.expect &&
  generate_operate_run +n -olat 20 1  ${file_source} > ${test_current}object-line-20-at-1-total.expect &&
  generate_operate_run +n -olat 100 1 ${file_source} > ${test_current}object-line-100-at-1-total.expect &&

  generate_operate_run +n -ont a         ${file_source} > ${test_current}object-name-a-total.expect &&
  generate_operate_run +n -olnt 0 a      ${file_source} > ${test_current}object-line-0-name-a-total.expect &&
  generate_operate_run +n -olnt 1 a      ${file_source} > ${test_current}object-line-1-name-a-total.expect &&
  generate_operate_run +n -olnt 2 a      ${file_source} > ${test_current}object-line-2-name-a-total.expect &&
  generate_operate_run +n -olnt 3 a      ${file_source} > ${test_current}object-line-3-name-a-total.expect &&
  generate_operate_run +n -olnt 4 a      ${file_source} > ${test_current}object-line-4-name-a-total.expect &&
  generate_operate_run +n -olnt 5 a      ${file_source} > ${test_current}object-line-5-name-a-total.expect &&
  generate_operate_run +n -olnt 6 a      ${file_source} > ${test_current}object-line-6-name-a-total.expect &&
  generate_operate_run +n -olnt 7 a      ${file_source} > ${test_current}object-line-7-name-a-total.expect &&
  generate_operate_run +n -olnt 8 a      ${file_source} > ${test_current}object-line-8-name-a-total.expect &&
  generate_operate_run +n -olnt 9 a      ${file_source} > ${test_current}object-line-9-name-a-total.expect &&
  generate_operate_run +n -olnt 10 a     ${file_source} > ${test_current}object-line-10-name-a-total.expect &&
  generate_operate_run +n -olnt 20 a     ${file_source} > ${test_current}object-line-20-name-a-total.expect &&
  generate_operate_run +n -olnt 100 a    ${file_source} > ${test_current}object-line-100-name-a-total.expect &&
  generate_operate_run +n -olant 0 0 a   ${file_source} > ${test_current}object-line-0-at-0-name-a-total.expect &&
  generate_operate_run +n -olant 1 0 a   ${file_source} > ${test_current}object-line-1-at-0-name-a-total.expect &&
  generate_operate_run +n -olant 2 0 a   ${file_source} > ${test_current}object-line-2-at-0-name-a-total.expect &&
  generate_operate_run +n -olant 3 0 a   ${file_source} > ${test_current}object-line-3-at-0-name-a-total.expect &&
  generate_operate_run +n -olant 4 0 a   ${file_source} > ${test_current}object-line-4-at-0-name-a-total.expect &&
  generate_operate_run +n -olant 5 0 a   ${file_source} > ${test_current}object-line-5-at-0-name-a-total.expect &&
  generate_operate_run +n -olant 6 0 a   ${file_source} > ${test_current}object-line-6-at-0-name-a-total.expect &&
  generate_operate_run +n -olant 7 0 a   ${file_source} > ${test_current}object-line-7-at-0-name-a-total.expect &&
  generate_operate_run +n -olant 8 0 a   ${file_source} > ${test_current}object-line-8-at-0-name-a-total.expect &&
  generate_operate_run +n -olant 9 0 a   ${file_source} > ${test_current}object-line-9-at-0-name-a-total.expect &&
  generate_operate_run +n -olant 10 0 a  ${file_source} > ${test_current}object-line-10-at-0-name-a-total.expect &&
  generate_operate_run +n -olant 20 0 a  ${file_source} > ${test_current}object-line-20-at-0-name-a-total.expect &&
  generate_operate_run +n -olant 100 0 a ${file_source} > ${test_current}object-line-100-at-0-name-a-total.expect &&
  generate_operate_run +n -olant 0 1 a   ${file_source} > ${test_current}object-line-0-at-1-name-a-total.expect &&
  generate_operate_run +n -olant 1 1 a   ${file_source} > ${test_current}object-line-1-at-1-name-a-total.expect &&
  generate_operate_run +n -olant 2 1 a   ${file_source} > ${test_current}object-line-2-at-1-name-a-total.expect &&
  generate_operate_run +n -olant 3 1 a   ${file_source} > ${test_current}object-line-3-at-1-name-a-total.expect &&
  generate_operate_run +n -olant 4 1 a   ${file_source} > ${test_current}object-line-4-at-1-name-a-total.expect &&
  generate_operate_run +n -olant 5 1 a   ${file_source} > ${test_current}object-line-5-at-1-name-a-total.expect &&
  generate_operate_run +n -olant 6 1 a   ${file_source} > ${test_current}object-line-6-at-1-name-a-total.expect &&
  generate_operate_run +n -olant 7 1 a   ${file_source} > ${test_current}object-line-7-at-1-name-a-total.expect &&
  generate_operate_run +n -olant 8 1 a   ${file_source} > ${test_current}object-line-8-at-1-name-a-total.expect &&
  generate_operate_run +n -olant 9 1 a   ${file_source} > ${test_current}object-line-9-at-1-name-a-total.expect &&
  generate_operate_run +n -olant 10 1 a  ${file_source} > ${test_current}object-line-10-at-1-name-a-total.expect &&
  generate_operate_run +n -olant 20 1 a  ${file_source} > ${test_current}object-line-20-at-1-name-a-total.expect &&
  generate_operate_run +n -olant 100 1 a ${file_source} > ${test_current}object-line-100-at-1-name-a-total.expect &&

  generate_operate_run +n -ont b         ${file_source} > ${test_current}object-name-b-total.expect &&
  generate_operate_run +n -olnt 0 b      ${file_source} > ${test_current}object-line-0-name-b-total.expect &&
  generate_operate_run +n -olnt 1 b      ${file_source} > ${test_current}object-line-1-name-b-total.expect &&
  generate_operate_run +n -olnt 2 b      ${file_source} > ${test_current}object-line-2-name-b-total.expect &&
  generate_operate_run +n -olnt 3 b      ${file_source} > ${test_current}object-line-3-name-b-total.expect &&
  generate_operate_run +n -olnt 4 b      ${file_source} > ${test_current}object-line-4-name-b-total.expect &&
  generate_operate_run +n -olnt 5 b      ${file_source} > ${test_current}object-line-5-name-b-total.expect &&
  generate_operate_run +n -olnt 6 b      ${file_source} > ${test_current}object-line-6-name-b-total.expect &&
  generate_operate_run +n -olnt 7 b      ${file_source} > ${test_current}object-line-7-name-b-total.expect &&
  generate_operate_run +n -olnt 8 b      ${file_source} > ${test_current}object-line-8-name-b-total.expect &&
  generate_operate_run +n -olnt 9 b      ${file_source} > ${test_current}object-line-9-name-b-total.expect &&
  generate_operate_run +n -olnt 10 b     ${file_source} > ${test_current}object-line-10-name-b-total.expect &&
  generate_operate_run +n -olnt 20 b     ${file_source} > ${test_current}object-line-20-name-b-total.expect &&
  generate_operate_run +n -olnt 100 b    ${file_source} > ${test_current}object-line-100-name-b-total.expect &&
  generate_operate_run +n -olant 0 0 b   ${file_source} > ${test_current}object-line-0-at-0-name-b-total.expect &&
  generate_operate_run +n -olant 1 0 b   ${file_source} > ${test_current}object-line-1-at-0-name-b-total.expect &&
  generate_operate_run +n -olant 2 0 b   ${file_source} > ${test_current}object-line-2-at-0-name-b-total.expect &&
  generate_operate_run +n -olant 3 0 b   ${file_source} > ${test_current}object-line-3-at-0-name-b-total.expect &&
  generate_operate_run +n -olant 4 0 b   ${file_source} > ${test_current}object-line-4-at-0-name-b-total.expect &&
  generate_operate_run +n -olant 5 0 b   ${file_source} > ${test_current}object-line-5-at-0-name-b-total.expect &&
  generate_operate_run +n -olant 6 0 b   ${file_source} > ${test_current}object-line-6-at-0-name-b-total.expect &&
  generate_operate_run +n -olant 7 0 b   ${file_source} > ${test_current}object-line-7-at-0-name-b-total.expect &&
  generate_operate_run +n -olant 8 0 b   ${file_source} > ${test_current}object-line-8-at-0-name-b-total.expect &&
  generate_operate_run +n -olant 9 0 b   ${file_source} > ${test_current}object-line-9-at-0-name-b-total.expect &&
  generate_operate_run +n -olant 10 0 b  ${file_source} > ${test_current}object-line-10-at-0-name-b-total.expect &&
  generate_operate_run +n -olant 20 0 b  ${file_source} > ${test_current}object-line-20-at-0-name-b-total.expect &&
  generate_operate_run +n -olant 100 0 b ${file_source} > ${test_current}object-line-100-at-0-name-b-total.expect &&
  generate_operate_run +n -olant 0 1 b   ${file_source} > ${test_current}object-line-0-at-1-name-b-total.expect &&
  generate_operate_run +n -olant 1 1 b   ${file_source} > ${test_current}object-line-1-at-1-name-b-total.expect &&
  generate_operate_run +n -olant 2 1 b   ${file_source} > ${test_current}object-line-2-at-1-name-b-total.expect &&
  generate_operate_run +n -olant 3 1 b   ${file_source} > ${test_current}object-line-3-at-1-name-b-total.expect &&
  generate_operate_run +n -olant 4 1 b   ${file_source} > ${test_current}object-line-4-at-1-name-b-total.expect &&
  generate_operate_run +n -olant 5 1 b   ${file_source} > ${test_current}object-line-5-at-1-name-b-total.expect &&
  generate_operate_run +n -olant 6 1 b   ${file_source} > ${test_current}object-line-6-at-1-name-b-total.expect &&
  generate_operate_run +n -olant 7 1 b   ${file_source} > ${test_current}object-line-7-at-1-name-b-total.expect &&
  generate_operate_run +n -olant 8 1 b   ${file_source} > ${test_current}object-line-8-at-1-name-b-total.expect &&
  generate_operate_run +n -olant 9 1 b   ${file_source} > ${test_current}object-line-9-at-1-name-b-total.expect &&
  generate_operate_run +n -olant 10 1 b  ${file_source} > ${test_current}object-line-10-at-1-name-b-total.expect &&
  generate_operate_run +n -olant 20 1 b  ${file_source} > ${test_current}object-line-20-at-1-name-b-total.expect &&
  generate_operate_run +n -olant 100 1 b ${file_source} > ${test_current}object-line-100-at-1-name-b-total.expect &&

  generate_operate_run +n -ont is         ${file_source} > ${test_current}object-name-is-total.expect &&
  generate_operate_run +n -olnt 0 is      ${file_source} > ${test_current}object-line-0-name-is-total.expect &&
  generate_operate_run +n -olnt 1 is      ${file_source} > ${test_current}object-line-1-name-is-total.expect &&
  generate_operate_run +n -olnt 2 is      ${file_source} > ${test_current}object-line-2-name-is-total.expect &&
  generate_operate_run +n -olnt 3 is      ${file_source} > ${test_current}object-line-3-name-is-total.expect &&
  generate_operate_run +n -olnt 4 is      ${file_source} > ${test_current}object-line-4-name-is-total.expect &&
  generate_operate_run +n -olnt 5 is      ${file_source} > ${test_current}object-line-5-name-is-total.expect &&
  generate_operate_run +n -olnt 6 is      ${file_source} > ${test_current}object-line-6-name-is-total.expect &&
  generate_operate_run +n -olnt 7 is      ${file_source} > ${test_current}object-line-7-name-is-total.expect &&
  generate_operate_run +n -olnt 8 is      ${file_source} > ${test_current}object-line-8-name-is-total.expect &&
  generate_operate_run +n -olnt 9 is      ${file_source} > ${test_current}object-line-9-name-is-total.expect &&
  generate_operate_run +n -olnt 10 is     ${file_source} > ${test_current}object-line-10-name-is-total.expect &&
  generate_operate_run +n -olnt 20 is     ${file_source} > ${test_current}object-line-20-name-is-total.expect &&
  generate_operate_run +n -olnt 100 is    ${file_source} > ${test_current}object-line-100-name-is-total.expect &&
  generate_operate_run +n -olant 0 0 is   ${file_source} > ${test_current}object-line-0-at-0-name-is-total.expect &&
  generate_operate_run +n -olant 1 0 is   ${file_source} > ${test_current}object-line-1-at-0-name-is-total.expect &&
  generate_operate_run +n -olant 2 0 is   ${file_source} > ${test_current}object-line-2-at-0-name-is-total.expect &&
  generate_operate_run +n -olant 3 0 is   ${file_source} > ${test_current}object-line-3-at-0-name-is-total.expect &&
  generate_operate_run +n -olant 4 0 is   ${file_source} > ${test_current}object-line-4-at-0-name-is-total.expect &&
  generate_operate_run +n -olant 5 0 is   ${file_source} > ${test_current}object-line-5-at-0-name-is-total.expect &&
  generate_operate_run +n -olant 6 0 is   ${file_source} > ${test_current}object-line-6-at-0-name-is-total.expect &&
  generate_operate_run +n -olant 7 0 is   ${file_source} > ${test_current}object-line-7-at-0-name-is-total.expect &&
  generate_operate_run +n -olant 8 0 is   ${file_source} > ${test_current}object-line-8-at-0-name-is-total.expect &&
  generate_operate_run +n -olant 9 0 is   ${file_source} > ${test_current}object-line-9-at-0-name-is-total.expect &&
  generate_operate_run +n -olant 10 0 is  ${file_source} > ${test_current}object-line-10-at-0-name-is-total.expect &&
  generate_operate_run +n -olant 20 0 is  ${file_source} > ${test_current}object-line-20-at-0-name-is-total.expect &&
  generate_operate_run +n -olant 100 0 is ${file_source} > ${test_current}object-line-100-at-0-name-is-total.expect &&
  generate_operate_run +n -olant 0 1 is   ${file_source} > ${test_current}object-line-0-at-1-name-is-total.expect &&
  generate_operate_run +n -olant 1 1 is   ${file_source} > ${test_current}object-line-1-at-1-name-is-total.expect &&
  generate_operate_run +n -olant 2 1 is   ${file_source} > ${test_current}object-line-2-at-1-name-is-total.expect &&
  generate_operate_run +n -olant 3 1 is   ${file_source} > ${test_current}object-line-3-at-1-name-is-total.expect &&
  generate_operate_run +n -olant 4 1 is   ${file_source} > ${test_current}object-line-4-at-1-name-is-total.expect &&
  generate_operate_run +n -olant 5 1 is   ${file_source} > ${test_current}object-line-5-at-1-name-is-total.expect &&
  generate_operate_run +n -olant 6 1 is   ${file_source} > ${test_current}object-line-6-at-1-name-is-total.expect &&
  generate_operate_run +n -olant 7 1 is   ${file_source} > ${test_current}object-line-7-at-1-name-is-total.expect &&
  generate_operate_run +n -olant 8 1 is   ${file_source} > ${test_current}object-line-8-at-1-name-is-total.expect &&
  generate_operate_run +n -olant 9 1 is   ${file_source} > ${test_current}object-line-9-at-1-name-is-total.expect &&
  generate_operate_run +n -olant 10 1 is  ${file_source} > ${test_current}object-line-10-at-1-name-is-total.expect &&
  generate_operate_run +n -olant 20 1 is  ${file_source} > ${test_current}object-line-20-at-1-name-is-total.expect &&
  generate_operate_run +n -olant 100 1 is ${file_source} > ${test_current}object-line-100-at-1-name-is-total.expect &&

  generate_operate_run +n -ont yup         ${file_source} > ${test_current}object-name-yup-total.expect &&
  generate_operate_run +n -olnt 0 yup      ${file_source} > ${test_current}object-line-0-name-yup-total.expect &&
  generate_operate_run +n -olnt 1 yup      ${file_source} > ${test_current}object-line-1-name-yup-total.expect &&
  generate_operate_run +n -olnt 2 yup      ${file_source} > ${test_current}object-line-2-name-yup-total.expect &&
  generate_operate_run +n -olnt 3 yup      ${file_source} > ${test_current}object-line-3-name-yup-total.expect &&
  generate_operate_run +n -olnt 4 yup      ${file_source} > ${test_current}object-line-4-name-yup-total.expect &&
  generate_operate_run +n -olnt 5 yup      ${file_source} > ${test_current}object-line-5-name-yup-total.expect &&
  generate_operate_run +n -olnt 6 yup      ${file_source} > ${test_current}object-line-6-name-yup-total.expect &&
  generate_operate_run +n -olnt 7 yup      ${file_source} > ${test_current}object-line-7-name-yup-total.expect &&
  generate_operate_run +n -olnt 8 yup      ${file_source} > ${test_current}object-line-8-name-yup-total.expect &&
  generate_operate_run +n -olnt 9 yup      ${file_source} > ${test_current}object-line-9-name-yup-total.expect &&
  generate_operate_run +n -olnt 10 yup     ${file_source} > ${test_current}object-line-10-name-yup-total.expect &&
  generate_operate_run +n -olnt 20 yup     ${file_source} > ${test_current}object-line-20-name-yup-total.expect &&
  generate_operate_run +n -olnt 100 yup    ${file_source} > ${test_current}object-line-100-name-yup-total.expect &&
  generate_operate_run +n -olant 0 0 yup   ${file_source} > ${test_current}object-line-0-at-0-name-yup-total.expect &&
  generate_operate_run +n -olant 1 0 yup   ${file_source} > ${test_current}object-line-1-at-0-name-yup-total.expect &&
  generate_operate_run +n -olant 2 0 yup   ${file_source} > ${test_current}object-line-2-at-0-name-yup-total.expect &&
  generate_operate_run +n -olant 3 0 yup   ${file_source} > ${test_current}object-line-3-at-0-name-yup-total.expect &&
  generate_operate_run +n -olant 4 0 yup   ${file_source} > ${test_current}object-line-4-at-0-name-yup-total.expect &&
  generate_operate_run +n -olant 5 0 yup   ${file_source} > ${test_current}object-line-5-at-0-name-yup-total.expect &&
  generate_operate_run +n -olant 6 0 yup   ${file_source} > ${test_current}object-line-6-at-0-name-yup-total.expect &&
  generate_operate_run +n -olant 7 0 yup   ${file_source} > ${test_current}object-line-7-at-0-name-yup-total.expect &&
  generate_operate_run +n -olant 8 0 yup   ${file_source} > ${test_current}object-line-8-at-0-name-yup-total.expect &&
  generate_operate_run +n -olant 9 0 yup   ${file_source} > ${test_current}object-line-9-at-0-name-yup-total.expect &&
  generate_operate_run +n -olant 10 0 yup  ${file_source} > ${test_current}object-line-10-at-0-name-yup-total.expect &&
  generate_operate_run +n -olant 20 0 yup  ${file_source} > ${test_current}object-line-20-at-0-name-yup-total.expect &&
  generate_operate_run +n -olant 100 0 yup ${file_source} > ${test_current}object-line-100-at-0-name-yup-total.expect &&
  generate_operate_run +n -olant 0 1 yup   ${file_source} > ${test_current}object-line-0-at-1-name-yup-total.expect &&
  generate_operate_run +n -olant 1 1 yup   ${file_source} > ${test_current}object-line-1-at-1-name-yup-total.expect &&
  generate_operate_run +n -olant 2 1 yup   ${file_source} > ${test_current}object-line-2-at-1-name-yup-total.expect &&
  generate_operate_run +n -olant 3 1 yup   ${file_source} > ${test_current}object-line-3-at-1-name-yup-total.expect &&
  generate_operate_run +n -olant 4 1 yup   ${file_source} > ${test_current}object-line-4-at-1-name-yup-total.expect &&
  generate_operate_run +n -olant 5 1 yup   ${file_source} > ${test_current}object-line-5-at-1-name-yup-total.expect &&
  generate_operate_run +n -olant 6 1 yup   ${file_source} > ${test_current}object-line-6-at-1-name-yup-total.expect &&
  generate_operate_run +n -olant 7 1 yup   ${file_source} > ${test_current}object-line-7-at-1-name-yup-total.expect &&
  generate_operate_run +n -olant 8 1 yup   ${file_source} > ${test_current}object-line-8-at-1-name-yup-total.expect &&
  generate_operate_run +n -olant 9 1 yup   ${file_source} > ${test_current}object-line-9-at-1-name-yup-total.expect &&
  generate_operate_run +n -olant 10 1 yup  ${file_source} > ${test_current}object-line-10-at-1-name-yup-total.expect &&
  generate_operate_run +n -olant 20 1 yup  ${file_source} > ${test_current}object-line-20-at-1-name-yup-total.expect &&
  generate_operate_run +n -olant 100 1 yup ${file_source} > ${test_current}object-line-100-at-1-name-yup-total.expect &&
  generate_operate_run +n -olant 100 1 yup ${file_source} > ${test_current}object-line-100-at-1-name-yup-total.expect &&

  #### OBJECT - WHOLE ####
  generate_operate_run +n -ow         ${file_source} > ${test_current}content.expect &&
  generate_operate_run +n -owl 0      ${file_source} > ${test_current}object-whole-line-0.expect &&
  generate_operate_run +n -owl 1      ${file_source} > ${test_current}object-whole-line-1.expect &&
  generate_operate_run +n -owl 2      ${file_source} > ${test_current}object-whole-line-2.expect &&
  generate_operate_run +n -owl 3      ${file_source} > ${test_current}object-whole-line-3.expect &&
  generate_operate_run +n -owl 4      ${file_source} > ${test_current}object-whole-line-4.expect &&
  generate_operate_run +n -owl 5      ${file_source} > ${test_current}object-whole-line-5.expect &&
  generate_operate_run +n -owl 6      ${file_source} > ${test_current}object-whole-line-6.expect &&
  generate_operate_run +n -owl 7      ${file_source} > ${test_current}object-whole-line-7.expect &&
  generate_operate_run +n -owl 8      ${file_source} > ${test_current}object-whole-line-8.expect &&
  generate_operate_run +n -owl 9      ${file_source} > ${test_current}object-whole-line-9.expect &&
  generate_operate_run +n -owl 10     ${file_source} > ${test_current}object-whole-line-10.expect &&
  generate_operate_run +n -owl 20     ${file_source} > ${test_current}object-whole-line-20.expect &&
  generate_operate_run +n -owl 100    ${file_source} > ${test_current}object-whole-line-100.expect &&
  generate_operate_run +n -owla 0 0   ${file_source} > ${test_current}object-whole-line-0-at-0.expect &&
  generate_operate_run +n -owla 1 0   ${file_source} > ${test_current}object-whole-line-1-at-0.expect &&
  generate_operate_run +n -owla 2 0   ${file_source} > ${test_current}object-whole-line-2-at-0.expect &&
  generate_operate_run +n -owla 3 0   ${file_source} > ${test_current}object-whole-line-3-at-0.expect &&
  generate_operate_run +n -owla 4 0   ${file_source} > ${test_current}object-whole-line-4-at-0.expect &&
  generate_operate_run +n -owla 5 0   ${file_source} > ${test_current}object-whole-line-5-at-0.expect &&
  generate_operate_run +n -owla 6 0   ${file_source} > ${test_current}object-whole-line-6-at-0.expect &&
  generate_operate_run +n -owla 7 0   ${file_source} > ${test_current}object-whole-line-7-at-0.expect &&
  generate_operate_run +n -owla 8 0   ${file_source} > ${test_current}object-whole-line-8-at-0.expect &&
  generate_operate_run +n -owla 9 0   ${file_source} > ${test_current}object-whole-line-9-at-0.expect &&
  generate_operate_run +n -owla 10 0  ${file_source} > ${test_current}object-whole-line-10-at-0.expect &&
  generate_operate_run +n -owla 20 0  ${file_source} > ${test_current}object-whole-line-20-at-0.expect &&
  generate_operate_run +n -owla 100 0 ${file_source} > ${test_current}object-whole-line-100-at-0.expect &&
  generate_operate_run +n -owla 0 1   ${file_source} > ${test_current}object-whole-line-0-at-1.expect &&
  generate_operate_run +n -owla 1 1   ${file_source} > ${test_current}object-whole-line-1-at-1.expect &&
  generate_operate_run +n -owla 2 1   ${file_source} > ${test_current}object-whole-line-2-at-1.expect &&
  generate_operate_run +n -owla 3 1   ${file_source} > ${test_current}object-whole-line-3-at-1.expect &&
  generate_operate_run +n -owla 4 1   ${file_source} > ${test_current}object-whole-line-4-at-1.expect &&
  generate_operate_run +n -owla 5 1   ${file_source} > ${test_current}object-whole-line-5-at-1.expect &&
  generate_operate_run +n -owla 6 1   ${file_source} > ${test_current}object-whole-line-6-at-1.expect &&
  generate_operate_run +n -owla 7 1   ${file_source} > ${test_current}object-whole-line-7-at-1.expect &&
  generate_operate_run +n -owla 8 1   ${file_source} > ${test_current}object-whole-line-8-at-1.expect &&
  generate_operate_run +n -owla 9 1   ${file_source} > ${test_current}object-whole-line-9-at-1.expect &&
  generate_operate_run +n -owla 10 1  ${file_source} > ${test_current}object-whole-line-10-at-1.expect &&
  generate_operate_run +n -owla 20 1  ${file_source} > ${test_current}object-whole-line-20-at-1.expect &&
  generate_operate_run +n -owla 100 1 ${file_source} > ${test_current}object-whole-line-100-at-1.expect &&

  generate_operate_run +n -own a         ${file_source} > ${test_current}object-whole-name-a.expect &&
  generate_operate_run +n -owln 0 a      ${file_source} > ${test_current}object-whole-line-0-name-a.expect &&
  generate_operate_run +n -owln 1 a      ${file_source} > ${test_current}object-whole-line-1-name-a.expect &&
  generate_operate_run +n -owln 2 a      ${file_source} > ${test_current}object-whole-line-2-name-a.expect &&
  generate_operate_run +n -owln 3 a      ${file_source} > ${test_current}object-whole-line-3-name-a.expect &&
  generate_operate_run +n -owln 4 a      ${file_source} > ${test_current}object-whole-line-4-name-a.expect &&
  generate_operate_run +n -owln 5 a      ${file_source} > ${test_current}object-whole-line-5-name-a.expect &&
  generate_operate_run +n -owln 6 a      ${file_source} > ${test_current}object-whole-line-6-name-a.expect &&
  generate_operate_run +n -owln 7 a      ${file_source} > ${test_current}object-whole-line-7-name-a.expect &&
  generate_operate_run +n -owln 8 a      ${file_source} > ${test_current}object-whole-line-8-name-a.expect &&
  generate_operate_run +n -owln 9 a      ${file_source} > ${test_current}object-whole-line-9-name-a.expect &&
  generate_operate_run +n -owln 10 a     ${file_source} > ${test_current}object-whole-line-10-name-a.expect &&
  generate_operate_run +n -owln 20 a     ${file_source} > ${test_current}object-whole-line-20-name-a.expect &&
  generate_operate_run +n -owln 100 a    ${file_source} > ${test_current}object-whole-line-100-name-a.expect &&
  generate_operate_run +n -owlan 0 0 a   ${file_source} > ${test_current}object-whole-line-0-at-0-name-a.expect &&
  generate_operate_run +n -owlan 1 0 a   ${file_source} > ${test_current}object-whole-line-1-at-0-name-a.expect &&
  generate_operate_run +n -owlan 2 0 a   ${file_source} > ${test_current}object-whole-line-2-at-0-name-a.expect &&
  generate_operate_run +n -owlan 3 0 a   ${file_source} > ${test_current}object-whole-line-3-at-0-name-a.expect &&
  generate_operate_run +n -owlan 4 0 a   ${file_source} > ${test_current}object-whole-line-4-at-0-name-a.expect &&
  generate_operate_run +n -owlan 5 0 a   ${file_source} > ${test_current}object-whole-line-5-at-0-name-a.expect &&
  generate_operate_run +n -owlan 6 0 a   ${file_source} > ${test_current}object-whole-line-6-at-0-name-a.expect &&
  generate_operate_run +n -owlan 7 0 a   ${file_source} > ${test_current}object-whole-line-7-at-0-name-a.expect &&
  generate_operate_run +n -owlan 8 0 a   ${file_source} > ${test_current}object-whole-line-8-at-0-name-a.expect &&
  generate_operate_run +n -owlan 9 0 a   ${file_source} > ${test_current}object-whole-line-9-at-0-name-a.expect &&
  generate_operate_run +n -owlan 10 0 a  ${file_source} > ${test_current}object-whole-line-10-at-0-name-a.expect &&
  generate_operate_run +n -owlan 20 0 a  ${file_source} > ${test_current}object-whole-line-20-at-0-name-a.expect &&
  generate_operate_run +n -owlan 100 0 a ${file_source} > ${test_current}object-whole-line-100-at-0-name-a.expect &&
  generate_operate_run +n -owlan 0 1 a   ${file_source} > ${test_current}object-whole-line-0-at-1-name-a.expect &&
  generate_operate_run +n -owlan 1 1 a   ${file_source} > ${test_current}object-whole-line-1-at-1-name-a.expect &&
  generate_operate_run +n -owlan 2 1 a   ${file_source} > ${test_current}object-whole-line-2-at-1-name-a.expect &&
  generate_operate_run +n -owlan 3 1 a   ${file_source} > ${test_current}object-whole-line-3-at-1-name-a.expect &&
  generate_operate_run +n -owlan 4 1 a   ${file_source} > ${test_current}object-whole-line-4-at-1-name-a.expect &&
  generate_operate_run +n -owlan 5 1 a   ${file_source} > ${test_current}object-whole-line-5-at-1-name-a.expect &&
  generate_operate_run +n -owlan 6 1 a   ${file_source} > ${test_current}object-whole-line-6-at-1-name-a.expect &&
  generate_operate_run +n -owlan 7 1 a   ${file_source} > ${test_current}object-whole-line-7-at-1-name-a.expect &&
  generate_operate_run +n -owlan 8 1 a   ${file_source} > ${test_current}object-whole-line-8-at-1-name-a.expect &&
  generate_operate_run +n -owlan 9 1 a   ${file_source} > ${test_current}object-whole-line-9-at-1-name-a.expect &&
  generate_operate_run +n -owlan 10 1 a  ${file_source} > ${test_current}object-whole-line-10-at-1-name-a.expect &&
  generate_operate_run +n -owlan 20 1 a  ${file_source} > ${test_current}object-whole-line-20-at-1-name-a.expect &&
  generate_operate_run +n -owlan 100 1 a ${file_source} > ${test_current}object-whole-line-100-at-1-name-a.expect &&

  generate_operate_run +n -own b         ${file_source} > ${test_current}object-whole-name-b.expect &&
  generate_operate_run +n -owln 0 b      ${file_source} > ${test_current}object-whole-line-0-name-b.expect &&
  generate_operate_run +n -owln 1 b      ${file_source} > ${test_current}object-whole-line-1-name-b.expect &&
  generate_operate_run +n -owln 2 b      ${file_source} > ${test_current}object-whole-line-2-name-b.expect &&
  generate_operate_run +n -owln 3 b      ${file_source} > ${test_current}object-whole-line-3-name-b.expect &&
  generate_operate_run +n -owln 4 b      ${file_source} > ${test_current}object-whole-line-4-name-b.expect &&
  generate_operate_run +n -owln 5 b      ${file_source} > ${test_current}object-whole-line-5-name-b.expect &&
  generate_operate_run +n -owln 6 b      ${file_source} > ${test_current}object-whole-line-6-name-b.expect &&
  generate_operate_run +n -owln 7 b      ${file_source} > ${test_current}object-whole-line-7-name-b.expect &&
  generate_operate_run +n -owln 8 b      ${file_source} > ${test_current}object-whole-line-8-name-b.expect &&
  generate_operate_run +n -owln 9 b      ${file_source} > ${test_current}object-whole-line-9-name-b.expect &&
  generate_operate_run +n -owln 10 b     ${file_source} > ${test_current}object-whole-line-10-name-b.expect &&
  generate_operate_run +n -owln 20 b     ${file_source} > ${test_current}object-whole-line-20-name-b.expect &&
  generate_operate_run +n -owln 100 b    ${file_source} > ${test_current}object-whole-line-100-name-b.expect &&
  generate_operate_run +n -owlan 0 0 b   ${file_source} > ${test_current}object-whole-line-0-at-0-name-b.expect &&
  generate_operate_run +n -owlan 1 0 b   ${file_source} > ${test_current}object-whole-line-1-at-0-name-b.expect &&
  generate_operate_run +n -owlan 2 0 b   ${file_source} > ${test_current}object-whole-line-2-at-0-name-b.expect &&
  generate_operate_run +n -owlan 3 0 b   ${file_source} > ${test_current}object-whole-line-3-at-0-name-b.expect &&
  generate_operate_run +n -owlan 4 0 b   ${file_source} > ${test_current}object-whole-line-4-at-0-name-b.expect &&
  generate_operate_run +n -owlan 5 0 b   ${file_source} > ${test_current}object-whole-line-5-at-0-name-b.expect &&
  generate_operate_run +n -owlan 6 0 b   ${file_source} > ${test_current}object-whole-line-6-at-0-name-b.expect &&
  generate_operate_run +n -owlan 7 0 b   ${file_source} > ${test_current}object-whole-line-7-at-0-name-b.expect &&
  generate_operate_run +n -owlan 8 0 b   ${file_source} > ${test_current}object-whole-line-8-at-0-name-b.expect &&
  generate_operate_run +n -owlan 9 0 b   ${file_source} > ${test_current}object-whole-line-9-at-0-name-b.expect &&
  generate_operate_run +n -owlan 10 0 b  ${file_source} > ${test_current}object-whole-line-10-at-0-name-b.expect &&
  generate_operate_run +n -owlan 20 0 b  ${file_source} > ${test_current}object-whole-line-20-at-0-name-b.expect &&
  generate_operate_run +n -owlan 100 0 b ${file_source} > ${test_current}object-whole-line-100-at-0-name-b.expect &&
  generate_operate_run +n -owlan 0 1 b   ${file_source} > ${test_current}object-whole-line-0-at-1-name-b.expect &&
  generate_operate_run +n -owlan 1 1 b   ${file_source} > ${test_current}object-whole-line-1-at-1-name-b.expect &&
  generate_operate_run +n -owlan 2 1 b   ${file_source} > ${test_current}object-whole-line-2-at-1-name-b.expect &&
  generate_operate_run +n -owlan 3 1 b   ${file_source} > ${test_current}object-whole-line-3-at-1-name-b.expect &&
  generate_operate_run +n -owlan 4 1 b   ${file_source} > ${test_current}object-whole-line-4-at-1-name-b.expect &&
  generate_operate_run +n -owlan 5 1 b   ${file_source} > ${test_current}object-whole-line-5-at-1-name-b.expect &&
  generate_operate_run +n -owlan 6 1 b   ${file_source} > ${test_current}object-whole-line-6-at-1-name-b.expect &&
  generate_operate_run +n -owlan 7 1 b   ${file_source} > ${test_current}object-whole-line-7-at-1-name-b.expect &&
  generate_operate_run +n -owlan 8 1 b   ${file_source} > ${test_current}object-whole-line-8-at-1-name-b.expect &&
  generate_operate_run +n -owlan 9 1 b   ${file_source} > ${test_current}object-whole-line-9-at-1-name-b.expect &&
  generate_operate_run +n -owlan 10 1 b  ${file_source} > ${test_current}object-whole-line-10-at-1-name-b.expect &&
  generate_operate_run +n -owlan 20 1 b  ${file_source} > ${test_current}object-whole-line-20-at-1-name-b.expect &&
  generate_operate_run +n -owlan 100 1 b ${file_source} > ${test_current}object-whole-line-100-at-1-name-b.expect &&

  generate_operate_run +n -own is         ${file_source} > ${test_current}object-whole-name-is.expect &&
  generate_operate_run +n -owln 0 is      ${file_source} > ${test_current}object-whole-line-0-name-is.expect &&
  generate_operate_run +n -owln 1 is      ${file_source} > ${test_current}object-whole-line-1-name-is.expect &&
  generate_operate_run +n -owln 2 is      ${file_source} > ${test_current}object-whole-line-2-name-is.expect &&
  generate_operate_run +n -owln 3 is      ${file_source} > ${test_current}object-whole-line-3-name-is.expect &&
  generate_operate_run +n -owln 4 is      ${file_source} > ${test_current}object-whole-line-4-name-is.expect &&
  generate_operate_run +n -owln 5 is      ${file_source} > ${test_current}object-whole-line-5-name-is.expect &&
  generate_operate_run +n -owln 6 is      ${file_source} > ${test_current}object-whole-line-6-name-is.expect &&
  generate_operate_run +n -owln 7 is      ${file_source} > ${test_current}object-whole-line-7-name-is.expect &&
  generate_operate_run +n -owln 8 is      ${file_source} > ${test_current}object-whole-line-8-name-is.expect &&
  generate_operate_run +n -owln 9 is      ${file_source} > ${test_current}object-whole-line-9-name-is.expect &&
  generate_operate_run +n -owln 10 is     ${file_source} > ${test_current}object-whole-line-10-name-is.expect &&
  generate_operate_run +n -owln 20 is     ${file_source} > ${test_current}object-whole-line-20-name-is.expect &&
  generate_operate_run +n -owln 100 is    ${file_source} > ${test_current}object-whole-line-100-name-is.expect &&
  generate_operate_run +n -owlan 0 0 is   ${file_source} > ${test_current}object-whole-line-0-at-0-name-is.expect &&
  generate_operate_run +n -owlan 1 0 is   ${file_source} > ${test_current}object-whole-line-1-at-0-name-is.expect &&
  generate_operate_run +n -owlan 2 0 is   ${file_source} > ${test_current}object-whole-line-2-at-0-name-is.expect &&
  generate_operate_run +n -owlan 3 0 is   ${file_source} > ${test_current}object-whole-line-3-at-0-name-is.expect &&
  generate_operate_run +n -owlan 4 0 is   ${file_source} > ${test_current}object-whole-line-4-at-0-name-is.expect &&
  generate_operate_run +n -owlan 5 0 is   ${file_source} > ${test_current}object-whole-line-5-at-0-name-is.expect &&
  generate_operate_run +n -owlan 6 0 is   ${file_source} > ${test_current}object-whole-line-6-at-0-name-is.expect &&
  generate_operate_run +n -owlan 7 0 is   ${file_source} > ${test_current}object-whole-line-7-at-0-name-is.expect &&
  generate_operate_run +n -owlan 8 0 is   ${file_source} > ${test_current}object-whole-line-8-at-0-name-is.expect &&
  generate_operate_run +n -owlan 9 0 is   ${file_source} > ${test_current}object-whole-line-9-at-0-name-is.expect &&
  generate_operate_run +n -owlan 10 0 is  ${file_source} > ${test_current}object-whole-line-10-at-0-name-is.expect &&
  generate_operate_run +n -owlan 20 0 is  ${file_source} > ${test_current}object-whole-line-20-at-0-name-is.expect &&
  generate_operate_run +n -owlan 100 0 is ${file_source} > ${test_current}object-whole-line-100-at-0-name-is.expect &&
  generate_operate_run +n -owlan 0 1 is   ${file_source} > ${test_current}object-whole-line-0-at-1-name-is.expect &&
  generate_operate_run +n -owlan 1 1 is   ${file_source} > ${test_current}object-whole-line-1-at-1-name-is.expect &&
  generate_operate_run +n -owlan 2 1 is   ${file_source} > ${test_current}object-whole-line-2-at-1-name-is.expect &&
  generate_operate_run +n -owlan 3 1 is   ${file_source} > ${test_current}object-whole-line-3-at-1-name-is.expect &&
  generate_operate_run +n -owlan 4 1 is   ${file_source} > ${test_current}object-whole-line-4-at-1-name-is.expect &&
  generate_operate_run +n -owlan 5 1 is   ${file_source} > ${test_current}object-whole-line-5-at-1-name-is.expect &&
  generate_operate_run +n -owlan 6 1 is   ${file_source} > ${test_current}object-whole-line-6-at-1-name-is.expect &&
  generate_operate_run +n -owlan 7 1 is   ${file_source} > ${test_current}object-whole-line-7-at-1-name-is.expect &&
  generate_operate_run +n -owlan 8 1 is   ${file_source} > ${test_current}object-whole-line-8-at-1-name-is.expect &&
  generate_operate_run +n -owlan 9 1 is   ${file_source} > ${test_current}object-whole-line-9-at-1-name-is.expect &&
  generate_operate_run +n -owlan 10 1 is  ${file_source} > ${test_current}object-whole-line-10-at-1-name-is.expect &&
  generate_operate_run +n -owlan 20 1 is  ${file_source} > ${test_current}object-whole-line-20-at-1-name-is.expect &&
  generate_operate_run +n -owlan 100 1 is ${file_source} > ${test_current}object-whole-line-100-at-1-name-is.expect &&

  generate_operate_run +n -own yup         ${file_source} > ${test_current}object-whole-name-yup.expect &&
  generate_operate_run +n -owln 0 yup      ${file_source} > ${test_current}object-whole-line-0-name-yup.expect &&
  generate_operate_run +n -owln 1 yup      ${file_source} > ${test_current}object-whole-line-1-name-yup.expect &&
  generate_operate_run +n -owln 2 yup      ${file_source} > ${test_current}object-whole-line-2-name-yup.expect &&
  generate_operate_run +n -owln 3 yup      ${file_source} > ${test_current}object-whole-line-3-name-yup.expect &&
  generate_operate_run +n -owln 4 yup      ${file_source} > ${test_current}object-whole-line-4-name-yup.expect &&
  generate_operate_run +n -owln 5 yup      ${file_source} > ${test_current}object-whole-line-5-name-yup.expect &&
  generate_operate_run +n -owln 6 yup      ${file_source} > ${test_current}object-whole-line-6-name-yup.expect &&
  generate_operate_run +n -owln 7 yup      ${file_source} > ${test_current}object-whole-line-7-name-yup.expect &&
  generate_operate_run +n -owln 8 yup      ${file_source} > ${test_current}object-whole-line-8-name-yup.expect &&
  generate_operate_run +n -owln 9 yup      ${file_source} > ${test_current}object-whole-line-9-name-yup.expect &&
  generate_operate_run +n -owln 10 yup     ${file_source} > ${test_current}object-whole-line-10-name-yup.expect &&
  generate_operate_run +n -owln 20 yup     ${file_source} > ${test_current}object-whole-line-20-name-yup.expect &&
  generate_operate_run +n -owln 100 yup    ${file_source} > ${test_current}object-whole-line-100-name-yup.expect &&
  generate_operate_run +n -owlan 0 0 yup   ${file_source} > ${test_current}object-whole-line-0-at-0-name-yup.expect &&
  generate_operate_run +n -owlan 1 0 yup   ${file_source} > ${test_current}object-whole-line-1-at-0-name-yup.expect &&
  generate_operate_run +n -owlan 2 0 yup   ${file_source} > ${test_current}object-whole-line-2-at-0-name-yup.expect &&
  generate_operate_run +n -owlan 3 0 yup   ${file_source} > ${test_current}object-whole-line-3-at-0-name-yup.expect &&
  generate_operate_run +n -owlan 4 0 yup   ${file_source} > ${test_current}object-whole-line-4-at-0-name-yup.expect &&
  generate_operate_run +n -owlan 5 0 yup   ${file_source} > ${test_current}object-whole-line-5-at-0-name-yup.expect &&
  generate_operate_run +n -owlan 6 0 yup   ${file_source} > ${test_current}object-whole-line-6-at-0-name-yup.expect &&
  generate_operate_run +n -owlan 7 0 yup   ${file_source} > ${test_current}object-whole-line-7-at-0-name-yup.expect &&
  generate_operate_run +n -owlan 8 0 yup   ${file_source} > ${test_current}object-whole-line-8-at-0-name-yup.expect &&
  generate_operate_run +n -owlan 9 0 yup   ${file_source} > ${test_current}object-whole-line-9-at-0-name-yup.expect &&
  generate_operate_run +n -owlan 10 0 yup  ${file_source} > ${test_current}object-whole-line-10-at-0-name-yup.expect &&
  generate_operate_run +n -owlan 20 0 yup  ${file_source} > ${test_current}object-whole-line-20-at-0-name-yup.expect &&
  generate_operate_run +n -owlan 100 0 yup ${file_source} > ${test_current}object-whole-line-100-at-0-name-yup.expect &&
  generate_operate_run +n -owlan 0 1 yup   ${file_source} > ${test_current}object-whole-line-0-at-1-name-yup.expect &&
  generate_operate_run +n -owlan 1 1 yup   ${file_source} > ${test_current}object-whole-line-1-at-1-name-yup.expect &&
  generate_operate_run +n -owlan 2 1 yup   ${file_source} > ${test_current}object-whole-line-2-at-1-name-yup.expect &&
  generate_operate_run +n -owlan 3 1 yup   ${file_source} > ${test_current}object-whole-line-3-at-1-name-yup.expect &&
  generate_operate_run +n -owlan 4 1 yup   ${file_source} > ${test_current}object-whole-line-4-at-1-name-yup.expect &&
  generate_operate_run +n -owlan 5 1 yup   ${file_source} > ${test_current}object-whole-line-5-at-1-name-yup.expect &&
  generate_operate_run +n -owlan 6 1 yup   ${file_source} > ${test_current}object-whole-line-6-at-1-name-yup.expect &&
  generate_operate_run +n -owlan 7 1 yup   ${file_source} > ${test_current}object-whole-line-7-at-1-name-yup.expect &&
  generate_operate_run +n -owlan 8 1 yup   ${file_source} > ${test_current}object-whole-line-8-at-1-name-yup.expect &&
  generate_operate_run +n -owlan 9 1 yup   ${file_source} > ${test_current}object-whole-line-9-at-1-name-yup.expect &&
  generate_operate_run +n -owlan 10 1 yup  ${file_source} > ${test_current}object-whole-line-10-at-1-name-yup.expect &&
  generate_operate_run +n -owlan 20 1 yup  ${file_source} > ${test_current}object-whole-line-20-at-1-name-yup.expect &&

  #### OBJECT - WHOLE - TOTAL ####
  generate_operate_run +n -owt         ${file_source} > ${test_current}object-whole-total.expect &&
  generate_operate_run +n -owlt 0      ${file_source} > ${test_current}object-whole-line-0-total.expect &&
  generate_operate_run +n -owlt 1      ${file_source} > ${test_current}object-whole-line-1-total.expect &&
  generate_operate_run +n -owlt 2      ${file_source} > ${test_current}object-whole-line-2-total.expect &&
  generate_operate_run +n -owlt 3      ${file_source} > ${test_current}object-whole-line-3-total.expect &&
  generate_operate_run +n -owlt 4      ${file_source} > ${test_current}object-whole-line-4-total.expect &&
  generate_operate_run +n -owlt 5      ${file_source} > ${test_current}object-whole-line-5-total.expect &&
  generate_operate_run +n -owlt 6      ${file_source} > ${test_current}object-whole-line-6-total.expect &&
  generate_operate_run +n -owlt 7      ${file_source} > ${test_current}object-whole-line-7-total.expect &&
  generate_operate_run +n -owlt 8      ${file_source} > ${test_current}object-whole-line-8-total.expect &&
  generate_operate_run +n -owlt 9      ${file_source} > ${test_current}object-whole-line-9-total.expect &&
  generate_operate_run +n -owlt 10     ${file_source} > ${test_current}object-whole-line-10-total.expect &&
  generate_operate_run +n -owlt 20     ${file_source} > ${test_current}object-whole-line-20-total.expect &&
  generate_operate_run +n -owlt 100    ${file_source} > ${test_current}object-whole-line-100-total.expect &&
  generate_operate_run +n -owlat 0 0   ${file_source} > ${test_current}object-whole-line-0-at-0-total.expect &&
  generate_operate_run +n -owlat 1 0   ${file_source} > ${test_current}object-whole-line-1-at-0-total.expect &&
  generate_operate_run +n -owlat 2 0   ${file_source} > ${test_current}object-whole-line-2-at-0-total.expect &&
  generate_operate_run +n -owlat 3 0   ${file_source} > ${test_current}object-whole-line-3-at-0-total.expect &&
  generate_operate_run +n -owlat 4 0   ${file_source} > ${test_current}object-whole-line-4-at-0-total.expect &&
  generate_operate_run +n -owlat 5 0   ${file_source} > ${test_current}object-whole-line-5-at-0-total.expect &&
  generate_operate_run +n -owlat 6 0   ${file_source} > ${test_current}object-whole-line-6-at-0-total.expect &&
  generate_operate_run +n -owlat 7 0   ${file_source} > ${test_current}object-whole-line-7-at-0-total.expect &&
  generate_operate_run +n -owlat 8 0   ${file_source} > ${test_current}object-whole-line-8-at-0-total.expect &&
  generate_operate_run +n -owlat 9 0   ${file_source} > ${test_current}object-whole-line-9-at-0-total.expect &&
  generate_operate_run +n -owlat 10 0  ${file_source} > ${test_current}object-whole-line-10-at-0-total.expect &&
  generate_operate_run +n -owlat 20 0  ${file_source} > ${test_current}object-whole-line-20-at-0-total.expect &&
  generate_operate_run +n -owlat 100 0 ${file_source} > ${test_current}object-whole-line-100-at-0-total.expect &&
  generate_operate_run +n -owlat 0 1   ${file_source} > ${test_current}object-whole-line-0-at-1-total.expect &&
  generate_operate_run +n -owlat 1 1   ${file_source} > ${test_current}object-whole-line-1-at-1-total.expect &&
  generate_operate_run +n -owlat 2 1   ${file_source} > ${test_current}object-whole-line-2-at-1-total.expect &&
  generate_operate_run +n -owlat 3 1   ${file_source} > ${test_current}object-whole-line-3-at-1-total.expect &&
  generate_operate_run +n -owlat 4 1   ${file_source} > ${test_current}object-whole-line-4-at-1-total.expect &&
  generate_operate_run +n -owlat 5 1   ${file_source} > ${test_current}object-whole-line-5-at-1-total.expect &&
  generate_operate_run +n -owlat 6 1   ${file_source} > ${test_current}object-whole-line-6-at-1-total.expect &&
  generate_operate_run +n -owlat 7 1   ${file_source} > ${test_current}object-whole-line-7-at-1-total.expect &&
  generate_operate_run +n -owlat 8 1   ${file_source} > ${test_current}object-whole-line-8-at-1-total.expect &&
  generate_operate_run +n -owlat 9 1   ${file_source} > ${test_current}object-whole-line-9-at-1-total.expect &&
  generate_operate_run +n -owlat 10 1  ${file_source} > ${test_current}object-whole-line-10-at-1-total.expect &&
  generate_operate_run +n -owlat 20 1  ${file_source} > ${test_current}object-whole-line-20-at-1-total.expect &&
  generate_operate_run +n -owlat 100 1 ${file_source} > ${test_current}object-whole-line-100-at-1-total.expect &&

  generate_operate_run +n -ownt a         ${file_source} > ${test_current}object-whole-name-a-total.expect &&
  generate_operate_run +n -owlnt 0 a      ${file_source} > ${test_current}object-whole-line-0-name-a-total.expect &&
  generate_operate_run +n -owlnt 1 a      ${file_source} > ${test_current}object-whole-line-1-name-a-total.expect &&
  generate_operate_run +n -owlnt 2 a      ${file_source} > ${test_current}object-whole-line-2-name-a-total.expect &&
  generate_operate_run +n -owlnt 3 a      ${file_source} > ${test_current}object-whole-line-3-name-a-total.expect &&
  generate_operate_run +n -owlnt 4 a      ${file_source} > ${test_current}object-whole-line-4-name-a-total.expect &&
  generate_operate_run +n -owlnt 5 a      ${file_source} > ${test_current}object-whole-line-5-name-a-total.expect &&
  generate_operate_run +n -owlnt 6 a      ${file_source} > ${test_current}object-whole-line-6-name-a-total.expect &&
  generate_operate_run +n -owlnt 7 a      ${file_source} > ${test_current}object-whole-line-7-name-a-total.expect &&
  generate_operate_run +n -owlnt 8 a      ${file_source} > ${test_current}object-whole-line-8-name-a-total.expect &&
  generate_operate_run +n -owlnt 9 a      ${file_source} > ${test_current}object-whole-line-9-name-a-total.expect &&
  generate_operate_run +n -owlnt 10 a     ${file_source} > ${test_current}object-whole-line-10-name-a-total.expect &&
  generate_operate_run +n -owlnt 20 a     ${file_source} > ${test_current}object-whole-line-20-name-a-total.expect &&
  generate_operate_run +n -owlnt 100 a    ${file_source} > ${test_current}object-whole-line-100-name-a-total.expect &&
  generate_operate_run +n -owlant 0 0 a   ${file_source} > ${test_current}object-whole-line-0-at-0-name-a-total.expect &&
  generate_operate_run +n -owlant 1 0 a   ${file_source} > ${test_current}object-whole-line-1-at-0-name-a-total.expect &&
  generate_operate_run +n -owlant 2 0 a   ${file_source} > ${test_current}object-whole-line-2-at-0-name-a-total.expect &&
  generate_operate_run +n -owlant 3 0 a   ${file_source} > ${test_current}object-whole-line-3-at-0-name-a-total.expect &&
  generate_operate_run +n -owlant 4 0 a   ${file_source} > ${test_current}object-whole-line-4-at-0-name-a-total.expect &&
  generate_operate_run +n -owlant 5 0 a   ${file_source} > ${test_current}object-whole-line-5-at-0-name-a-total.expect &&
  generate_operate_run +n -owlant 6 0 a   ${file_source} > ${test_current}object-whole-line-6-at-0-name-a-total.expect &&
  generate_operate_run +n -owlant 7 0 a   ${file_source} > ${test_current}object-whole-line-7-at-0-name-a-total.expect &&
  generate_operate_run +n -owlant 8 0 a   ${file_source} > ${test_current}object-whole-line-8-at-0-name-a-total.expect &&
  generate_operate_run +n -owlant 9 0 a   ${file_source} > ${test_current}object-whole-line-9-at-0-name-a-total.expect &&
  generate_operate_run +n -owlant 10 0 a  ${file_source} > ${test_current}object-whole-line-10-at-0-name-a-total.expect &&
  generate_operate_run +n -owlant 20 0 a  ${file_source} > ${test_current}object-whole-line-20-at-0-name-a-total.expect &&
  generate_operate_run +n -owlant 100 0 a ${file_source} > ${test_current}object-whole-line-100-at-0-name-a-total.expect &&
  generate_operate_run +n -owlant 0 1 a   ${file_source} > ${test_current}object-whole-line-0-at-1-name-a-total.expect &&
  generate_operate_run +n -owlant 1 1 a   ${file_source} > ${test_current}object-whole-line-1-at-1-name-a-total.expect &&
  generate_operate_run +n -owlant 2 1 a   ${file_source} > ${test_current}object-whole-line-2-at-1-name-a-total.expect &&
  generate_operate_run +n -owlant 3 1 a   ${file_source} > ${test_current}object-whole-line-3-at-1-name-a-total.expect &&
  generate_operate_run +n -owlant 4 1 a   ${file_source} > ${test_current}object-whole-line-4-at-1-name-a-total.expect &&
  generate_operate_run +n -owlant 5 1 a   ${file_source} > ${test_current}object-whole-line-5-at-1-name-a-total.expect &&
  generate_operate_run +n -owlant 6 1 a   ${file_source} > ${test_current}object-whole-line-6-at-1-name-a-total.expect &&
  generate_operate_run +n -owlant 7 1 a   ${file_source} > ${test_current}object-whole-line-7-at-1-name-a-total.expect &&
  generate_operate_run +n -owlant 8 1 a   ${file_source} > ${test_current}object-whole-line-8-at-1-name-a-total.expect &&
  generate_operate_run +n -owlant 9 1 a   ${file_source} > ${test_current}object-whole-line-9-at-1-name-a-total.expect &&
  generate_operate_run +n -owlant 10 1 a  ${file_source} > ${test_current}object-whole-line-10-at-1-name-a-total.expect &&
  generate_operate_run +n -owlant 20 1 a  ${file_source} > ${test_current}object-whole-line-20-at-1-name-a-total.expect &&
  generate_operate_run +n -owlant 100 1 a ${file_source} > ${test_current}object-whole-line-100-at-1-name-a-total.expect &&

  generate_operate_run +n -ownt b         ${file_source} > ${test_current}object-whole-name-b-total.expect &&
  generate_operate_run +n -owlnt 0 b      ${file_source} > ${test_current}object-whole-line-0-name-b-total.expect &&
  generate_operate_run +n -owlnt 1 b      ${file_source} > ${test_current}object-whole-line-1-name-b-total.expect &&
  generate_operate_run +n -owlnt 2 b      ${file_source} > ${test_current}object-whole-line-2-name-b-total.expect &&
  generate_operate_run +n -owlnt 3 b      ${file_source} > ${test_current}object-whole-line-3-name-b-total.expect &&
  generate_operate_run +n -owlnt 4 b      ${file_source} > ${test_current}object-whole-line-4-name-b-total.expect &&
  generate_operate_run +n -owlnt 5 b      ${file_source} > ${test_current}object-whole-line-5-name-b-total.expect &&
  generate_operate_run +n -owlnt 6 b      ${file_source} > ${test_current}object-whole-line-6-name-b-total.expect &&
  generate_operate_run +n -owlnt 7 b      ${file_source} > ${test_current}object-whole-line-7-name-b-total.expect &&
  generate_operate_run +n -owlnt 8 b      ${file_source} > ${test_current}object-whole-line-8-name-b-total.expect &&
  generate_operate_run +n -owlnt 9 b      ${file_source} > ${test_current}object-whole-line-9-name-b-total.expect &&
  generate_operate_run +n -owlnt 10 b     ${file_source} > ${test_current}object-whole-line-10-name-b-total.expect &&
  generate_operate_run +n -owlnt 20 b     ${file_source} > ${test_current}object-whole-line-20-name-b-total.expect &&
  generate_operate_run +n -owlnt 100 b    ${file_source} > ${test_current}object-whole-line-100-name-b-total.expect &&
  generate_operate_run +n -owlant 0 0 b   ${file_source} > ${test_current}object-whole-line-0-at-0-name-b-total.expect &&
  generate_operate_run +n -owlant 1 0 b   ${file_source} > ${test_current}object-whole-line-1-at-0-name-b-total.expect &&
  generate_operate_run +n -owlant 2 0 b   ${file_source} > ${test_current}object-whole-line-2-at-0-name-b-total.expect &&
  generate_operate_run +n -owlant 3 0 b   ${file_source} > ${test_current}object-whole-line-3-at-0-name-b-total.expect &&
  generate_operate_run +n -owlant 4 0 b   ${file_source} > ${test_current}object-whole-line-4-at-0-name-b-total.expect &&
  generate_operate_run +n -owlant 5 0 b   ${file_source} > ${test_current}object-whole-line-5-at-0-name-b-total.expect &&
  generate_operate_run +n -owlant 6 0 b   ${file_source} > ${test_current}object-whole-line-6-at-0-name-b-total.expect &&
  generate_operate_run +n -owlant 7 0 b   ${file_source} > ${test_current}object-whole-line-7-at-0-name-b-total.expect &&
  generate_operate_run +n -owlant 8 0 b   ${file_source} > ${test_current}object-whole-line-8-at-0-name-b-total.expect &&
  generate_operate_run +n -owlant 9 0 b   ${file_source} > ${test_current}object-whole-line-9-at-0-name-b-total.expect &&
  generate_operate_run +n -owlant 10 0 b  ${file_source} > ${test_current}object-whole-line-10-at-0-name-b-total.expect &&
  generate_operate_run +n -owlant 20 0 b  ${file_source} > ${test_current}object-whole-line-20-at-0-name-b-total.expect &&
  generate_operate_run +n -owlant 100 0 b ${file_source} > ${test_current}object-whole-line-100-at-0-name-b-total.expect &&
  generate_operate_run +n -owlant 0 1 b   ${file_source} > ${test_current}object-whole-line-0-at-1-name-b-total.expect &&
  generate_operate_run +n -owlant 1 1 b   ${file_source} > ${test_current}object-whole-line-1-at-1-name-b-total.expect &&
  generate_operate_run +n -owlant 2 1 b   ${file_source} > ${test_current}object-whole-line-2-at-1-name-b-total.expect &&
  generate_operate_run +n -owlant 3 1 b   ${file_source} > ${test_current}object-whole-line-3-at-1-name-b-total.expect &&
  generate_operate_run +n -owlant 4 1 b   ${file_source} > ${test_current}object-whole-line-4-at-1-name-b-total.expect &&
  generate_operate_run +n -owlant 5 1 b   ${file_source} > ${test_current}object-whole-line-5-at-1-name-b-total.expect &&
  generate_operate_run +n -owlant 6 1 b   ${file_source} > ${test_current}object-whole-line-6-at-1-name-b-total.expect &&
  generate_operate_run +n -owlant 7 1 b   ${file_source} > ${test_current}object-whole-line-7-at-1-name-b-total.expect &&
  generate_operate_run +n -owlant 8 1 b   ${file_source} > ${test_current}object-whole-line-8-at-1-name-b-total.expect &&
  generate_operate_run +n -owlant 9 1 b   ${file_source} > ${test_current}object-whole-line-9-at-1-name-b-total.expect &&
  generate_operate_run +n -owlant 10 1 b  ${file_source} > ${test_current}object-whole-line-10-at-1-name-b-total.expect &&
  generate_operate_run +n -owlant 20 1 b  ${file_source} > ${test_current}object-whole-line-20-at-1-name-b-total.expect &&
  generate_operate_run +n -owlant 100 1 b ${file_source} > ${test_current}object-whole-line-100-at-1-name-b-total.expect &&

  generate_operate_run +n -ownt is         ${file_source} > ${test_current}object-whole-name-is-total.expect &&
  generate_operate_run +n -owlnt 0 is      ${file_source} > ${test_current}object-whole-line-0-name-is-total.expect &&
  generate_operate_run +n -owlnt 1 is      ${file_source} > ${test_current}object-whole-line-1-name-is-total.expect &&
  generate_operate_run +n -owlnt 2 is      ${file_source} > ${test_current}object-whole-line-2-name-is-total.expect &&
  generate_operate_run +n -owlnt 3 is      ${file_source} > ${test_current}object-whole-line-3-name-is-total.expect &&
  generate_operate_run +n -owlnt 4 is      ${file_source} > ${test_current}object-whole-line-4-name-is-total.expect &&
  generate_operate_run +n -owlnt 5 is      ${file_source} > ${test_current}object-whole-line-5-name-is-total.expect &&
  generate_operate_run +n -owlnt 6 is      ${file_source} > ${test_current}object-whole-line-6-name-is-total.expect &&
  generate_operate_run +n -owlnt 7 is      ${file_source} > ${test_current}object-whole-line-7-name-is-total.expect &&
  generate_operate_run +n -owlnt 8 is      ${file_source} > ${test_current}object-whole-line-8-name-is-total.expect &&
  generate_operate_run +n -owlnt 9 is      ${file_source} > ${test_current}object-whole-line-9-name-is-total.expect &&
  generate_operate_run +n -owlnt 10 is     ${file_source} > ${test_current}object-whole-line-10-name-is-total.expect &&
  generate_operate_run +n -owlnt 20 is     ${file_source} > ${test_current}object-whole-line-20-name-is-total.expect &&
  generate_operate_run +n -owlnt 100 is    ${file_source} > ${test_current}object-whole-line-100-name-is-total.expect &&
  generate_operate_run +n -owlant 0 0 is   ${file_source} > ${test_current}object-whole-line-0-at-0-name-is-total.expect &&
  generate_operate_run +n -owlant 1 0 is   ${file_source} > ${test_current}object-whole-line-1-at-0-name-is-total.expect &&
  generate_operate_run +n -owlant 2 0 is   ${file_source} > ${test_current}object-whole-line-2-at-0-name-is-total.expect &&
  generate_operate_run +n -owlant 3 0 is   ${file_source} > ${test_current}object-whole-line-3-at-0-name-is-total.expect &&
  generate_operate_run +n -owlant 4 0 is   ${file_source} > ${test_current}object-whole-line-4-at-0-name-is-total.expect &&
  generate_operate_run +n -owlant 5 0 is   ${file_source} > ${test_current}object-whole-line-5-at-0-name-is-total.expect &&
  generate_operate_run +n -owlant 6 0 is   ${file_source} > ${test_current}object-whole-line-6-at-0-name-is-total.expect &&
  generate_operate_run +n -owlant 7 0 is   ${file_source} > ${test_current}object-whole-line-7-at-0-name-is-total.expect &&
  generate_operate_run +n -owlant 8 0 is   ${file_source} > ${test_current}object-whole-line-8-at-0-name-is-total.expect &&
  generate_operate_run +n -owlant 9 0 is   ${file_source} > ${test_current}object-whole-line-9-at-0-name-is-total.expect &&
  generate_operate_run +n -owlant 10 0 is  ${file_source} > ${test_current}object-whole-line-10-at-0-name-is-total.expect &&
  generate_operate_run +n -owlant 20 0 is  ${file_source} > ${test_current}object-whole-line-20-at-0-name-is-total.expect &&
  generate_operate_run +n -owlant 100 0 is ${file_source} > ${test_current}object-whole-line-100-at-0-name-is-total.expect &&
  generate_operate_run +n -owlant 0 1 is   ${file_source} > ${test_current}object-whole-line-0-at-1-name-is-total.expect &&
  generate_operate_run +n -owlant 1 1 is   ${file_source} > ${test_current}object-whole-line-1-at-1-name-is-total.expect &&
  generate_operate_run +n -owlant 2 1 is   ${file_source} > ${test_current}object-whole-line-2-at-1-name-is-total.expect &&
  generate_operate_run +n -owlant 3 1 is   ${file_source} > ${test_current}object-whole-line-3-at-1-name-is-total.expect &&
  generate_operate_run +n -owlant 4 1 is   ${file_source} > ${test_current}object-whole-line-4-at-1-name-is-total.expect &&
  generate_operate_run +n -owlant 5 1 is   ${file_source} > ${test_current}object-whole-line-5-at-1-name-is-total.expect &&
  generate_operate_run +n -owlant 6 1 is   ${file_source} > ${test_current}object-whole-line-6-at-1-name-is-total.expect &&
  generate_operate_run +n -owlant 7 1 is   ${file_source} > ${test_current}object-whole-line-7-at-1-name-is-total.expect &&
  generate_operate_run +n -owlant 8 1 is   ${file_source} > ${test_current}object-whole-line-8-at-1-name-is-total.expect &&
  generate_operate_run +n -owlant 9 1 is   ${file_source} > ${test_current}object-whole-line-9-at-1-name-is-total.expect &&
  generate_operate_run +n -owlant 10 1 is  ${file_source} > ${test_current}object-whole-line-10-at-1-name-is-total.expect &&
  generate_operate_run +n -owlant 20 1 is  ${file_source} > ${test_current}object-whole-line-20-at-1-name-is-total.expect &&
  generate_operate_run +n -owlant 100 1 is ${file_source} > ${test_current}object-whole-line-100-at-1-name-is-total.expect &&

  generate_operate_run +n -ownt yup         ${file_source} > ${test_current}object-whole-name-yup-total.expect &&
  generate_operate_run +n -owlnt 0 yup      ${file_source} > ${test_current}object-whole-line-0-name-yup-total.expect &&
  generate_operate_run +n -owlnt 1 yup      ${file_source} > ${test_current}object-whole-line-1-name-yup-total.expect &&
  generate_operate_run +n -owlnt 2 yup      ${file_source} > ${test_current}object-whole-line-2-name-yup-total.expect &&
  generate_operate_run +n -owlnt 3 yup      ${file_source} > ${test_current}object-whole-line-3-name-yup-total.expect &&
  generate_operate_run +n -owlnt 4 yup      ${file_source} > ${test_current}object-whole-line-4-name-yup-total.expect &&
  generate_operate_run +n -owlnt 5 yup      ${file_source} > ${test_current}object-whole-line-5-name-yup-total.expect &&
  generate_operate_run +n -owlnt 6 yup      ${file_source} > ${test_current}object-whole-line-6-name-yup-total.expect &&
  generate_operate_run +n -owlnt 7 yup      ${file_source} > ${test_current}object-whole-line-7-name-yup-total.expect &&
  generate_operate_run +n -owlnt 8 yup      ${file_source} > ${test_current}object-whole-line-8-name-yup-total.expect &&
  generate_operate_run +n -owlnt 9 yup      ${file_source} > ${test_current}object-whole-line-9-name-yup-total.expect &&
  generate_operate_run +n -owlnt 10 yup     ${file_source} > ${test_current}object-whole-line-10-name-yup-total.expect &&
  generate_operate_run +n -owlnt 20 yup     ${file_source} > ${test_current}object-whole-line-20-name-yup-total.expect &&
  generate_operate_run +n -owlnt 100 yup    ${file_source} > ${test_current}object-whole-line-100-name-yup-total.expect &&
  generate_operate_run +n -owlant 0 0 yup   ${file_source} > ${test_current}object-whole-line-0-at-0-name-yup-total.expect &&
  generate_operate_run +n -owlant 1 0 yup   ${file_source} > ${test_current}object-whole-line-1-at-0-name-yup-total.expect &&
  generate_operate_run +n -owlant 2 0 yup   ${file_source} > ${test_current}object-whole-line-2-at-0-name-yup-total.expect &&
  generate_operate_run +n -owlant 3 0 yup   ${file_source} > ${test_current}object-whole-line-3-at-0-name-yup-total.expect &&
  generate_operate_run +n -owlant 4 0 yup   ${file_source} > ${test_current}object-whole-line-4-at-0-name-yup-total.expect &&
  generate_operate_run +n -owlant 5 0 yup   ${file_source} > ${test_current}object-whole-line-5-at-0-name-yup-total.expect &&
  generate_operate_run +n -owlant 6 0 yup   ${file_source} > ${test_current}object-whole-line-6-at-0-name-yup-total.expect &&
  generate_operate_run +n -owlant 7 0 yup   ${file_source} > ${test_current}object-whole-line-7-at-0-name-yup-total.expect &&
  generate_operate_run +n -owlant 8 0 yup   ${file_source} > ${test_current}object-whole-line-8-at-0-name-yup-total.expect &&
  generate_operate_run +n -owlant 9 0 yup   ${file_source} > ${test_current}object-whole-line-9-at-0-name-yup-total.expect &&
  generate_operate_run +n -owlant 10 0 yup  ${file_source} > ${test_current}object-whole-line-10-at-0-name-yup-total.expect &&
  generate_operate_run +n -owlant 20 0 yup  ${file_source} > ${test_current}object-whole-line-20-at-0-name-yup-total.expect &&
  generate_operate_run +n -owlant 100 0 yup ${file_source} > ${test_current}object-whole-line-100-at-0-name-yup-total.expect &&
  generate_operate_run +n -owlant 0 1 yup   ${file_source} > ${test_current}object-whole-line-0-at-1-name-yup-total.expect &&
  generate_operate_run +n -owlant 1 1 yup   ${file_source} > ${test_current}object-whole-line-1-at-1-name-yup-total.expect &&
  generate_operate_run +n -owlant 2 1 yup   ${file_source} > ${test_current}object-whole-line-2-at-1-name-yup-total.expect &&
  generate_operate_run +n -owlant 3 1 yup   ${file_source} > ${test_current}object-whole-line-3-at-1-name-yup-total.expect &&
  generate_operate_run +n -owlant 4 1 yup   ${file_source} > ${test_current}object-whole-line-4-at-1-name-yup-total.expect &&
  generate_operate_run +n -owlant 5 1 yup   ${file_source} > ${test_current}object-whole-line-5-at-1-name-yup-total.expect &&
  generate_operate_run +n -owlant 6 1 yup   ${file_source} > ${test_current}object-whole-line-6-at-1-name-yup-total.expect &&
  generate_operate_run +n -owlant 7 1 yup   ${file_source} > ${test_current}object-whole-line-7-at-1-name-yup-total.expect &&
  generate_operate_run +n -owlant 8 1 yup   ${file_source} > ${test_current}object-whole-line-8-at-1-name-yup-total.expect &&
  generate_operate_run +n -owlant 9 1 yup   ${file_source} > ${test_current}object-whole-line-9-at-1-name-yup-total.expect &&
  generate_operate_run +n -owlant 10 1 yup  ${file_source} > ${test_current}object-whole-line-10-at-1-name-yup-total.expect &&
  generate_operate_run +n -owlant 20 1 yup  ${file_source} > ${test_current}object-whole-line-20-at-1-name-yup-total.expect &&
  generate_operate_run +n -owlant 100 1 yup ${file_source} > ${test_current}object-whole-line-100-at-1-name-yup-total.expect &&
  generate_operate_run +n -owlant 100 1 yup ${file_source} > ${test_current}object-whole-line-100-at-1-name-yup-total.expect &&

  #### LITERAL ####
  generate_operate_run +n -L         ${file_source} > ${test_current}content.expect &&
  generate_operate_run +n -Ll 0      ${file_source} > ${test_current}literal-line-0.expect &&
  generate_operate_run +n -Ll 1      ${file_source} > ${test_current}literal-line-1.expect &&
  generate_operate_run +n -Ll 2      ${file_source} > ${test_current}literal-line-2.expect &&
  generate_operate_run +n -Ll 3      ${file_source} > ${test_current}literal-line-3.expect &&
  generate_operate_run +n -Ll 4      ${file_source} > ${test_current}literal-line-4.expect &&
  generate_operate_run +n -Ll 5      ${file_source} > ${test_current}literal-line-5.expect &&
  generate_operate_run +n -Ll 6      ${file_source} > ${test_current}literal-line-6.expect &&
  generate_operate_run +n -Ll 7      ${file_source} > ${test_current}literal-line-7.expect &&
  generate_operate_run +n -Ll 8      ${file_source} > ${test_current}literal-line-8.expect &&
  generate_operate_run +n -Ll 9      ${file_source} > ${test_current}literal-line-9.expect &&
  generate_operate_run +n -Ll 10     ${file_source} > ${test_current}literal-line-10.expect &&
  generate_operate_run +n -Ll 20     ${file_source} > ${test_current}literal-line-20.expect &&
  generate_operate_run +n -Ll 100    ${file_source} > ${test_current}literal-line-100.expect &&
  generate_operate_run +n -Lla 0 0   ${file_source} > ${test_current}literal-line-0-at-0.expect &&
  generate_operate_run +n -Lla 1 0   ${file_source} > ${test_current}literal-line-1-at-0.expect &&
  generate_operate_run +n -Lla 2 0   ${file_source} > ${test_current}literal-line-2-at-0.expect &&
  generate_operate_run +n -Lla 3 0   ${file_source} > ${test_current}literal-line-3-at-0.expect &&
  generate_operate_run +n -Lla 4 0   ${file_source} > ${test_current}literal-line-4-at-0.expect &&
  generate_operate_run +n -Lla 5 0   ${file_source} > ${test_current}literal-line-5-at-0.expect &&
  generate_operate_run +n -Lla 6 0   ${file_source} > ${test_current}literal-line-6-at-0.expect &&
  generate_operate_run +n -Lla 7 0   ${file_source} > ${test_current}literal-line-7-at-0.expect &&
  generate_operate_run +n -Lla 8 0   ${file_source} > ${test_current}literal-line-8-at-0.expect &&
  generate_operate_run +n -Lla 9 0   ${file_source} > ${test_current}literal-line-9-at-0.expect &&
  generate_operate_run +n -Lla 10 0  ${file_source} > ${test_current}literal-line-10-at-0.expect &&
  generate_operate_run +n -Lla 20 0  ${file_source} > ${test_current}literal-line-20-at-0.expect &&
  generate_operate_run +n -Lla 100 0 ${file_source} > ${test_current}literal-line-100-at-0.expect &&
  generate_operate_run +n -Lla 0 1   ${file_source} > ${test_current}literal-line-0-at-1.expect &&
  generate_operate_run +n -Lla 1 1   ${file_source} > ${test_current}literal-line-1-at-1.expect &&
  generate_operate_run +n -Lla 2 1   ${file_source} > ${test_current}literal-line-2-at-1.expect &&
  generate_operate_run +n -Lla 3 1   ${file_source} > ${test_current}literal-line-3-at-1.expect &&
  generate_operate_run +n -Lla 4 1   ${file_source} > ${test_current}literal-line-4-at-1.expect &&
  generate_operate_run +n -Lla 5 1   ${file_source} > ${test_current}literal-line-5-at-1.expect &&
  generate_operate_run +n -Lla 6 1   ${file_source} > ${test_current}literal-line-6-at-1.expect &&
  generate_operate_run +n -Lla 7 1   ${file_source} > ${test_current}literal-line-7-at-1.expect &&
  generate_operate_run +n -Lla 8 1   ${file_source} > ${test_current}literal-line-8-at-1.expect &&
  generate_operate_run +n -Lla 9 1   ${file_source} > ${test_current}literal-line-9-at-1.expect &&
  generate_operate_run +n -Lla 10 1  ${file_source} > ${test_current}literal-line-10-at-1.expect &&
  generate_operate_run +n -Lla 20 1  ${file_source} > ${test_current}literal-line-20-at-1.expect &&
  generate_operate_run +n -Lla 100 1 ${file_source} > ${test_current}literal-line-100-at-1.expect &&

  generate_operate_run +n -Ln a         ${file_source} > ${test_current}literal-name-a.expect &&
  generate_operate_run +n -Lln 0 a      ${file_source} > ${test_current}literal-line-0-name-a.expect &&
  generate_operate_run +n -Lln 1 a      ${file_source} > ${test_current}literal-line-1-name-a.expect &&
  generate_operate_run +n -Lln 2 a      ${file_source} > ${test_current}literal-line-2-name-a.expect &&
  generate_operate_run +n -Lln 3 a      ${file_source} > ${test_current}literal-line-3-name-a.expect &&
  generate_operate_run +n -Lln 4 a      ${file_source} > ${test_current}literal-line-4-name-a.expect &&
  generate_operate_run +n -Lln 5 a      ${file_source} > ${test_current}literal-line-5-name-a.expect &&
  generate_operate_run +n -Lln 6 a      ${file_source} > ${test_current}literal-line-6-name-a.expect &&
  generate_operate_run +n -Lln 7 a      ${file_source} > ${test_current}literal-line-7-name-a.expect &&
  generate_operate_run +n -Lln 8 a      ${file_source} > ${test_current}literal-line-8-name-a.expect &&
  generate_operate_run +n -Lln 9 a      ${file_source} > ${test_current}literal-line-9-name-a.expect &&
  generate_operate_run +n -Lln 10 a     ${file_source} > ${test_current}literal-line-10-name-a.expect &&
  generate_operate_run +n -Lln 20 a     ${file_source} > ${test_current}literal-line-20-name-a.expect &&
  generate_operate_run +n -Lln 100 a    ${file_source} > ${test_current}literal-line-100-name-a.expect &&
  generate_operate_run +n -Llan 0 0 a   ${file_source} > ${test_current}literal-line-0-at-0-name-a.expect &&
  generate_operate_run +n -Llan 1 0 a   ${file_source} > ${test_current}literal-line-1-at-0-name-a.expect &&
  generate_operate_run +n -Llan 2 0 a   ${file_source} > ${test_current}literal-line-2-at-0-name-a.expect &&
  generate_operate_run +n -Llan 3 0 a   ${file_source} > ${test_current}literal-line-3-at-0-name-a.expect &&
  generate_operate_run +n -Llan 4 0 a   ${file_source} > ${test_current}literal-line-4-at-0-name-a.expect &&
  generate_operate_run +n -Llan 5 0 a   ${file_source} > ${test_current}literal-line-5-at-0-name-a.expect &&
  generate_operate_run +n -Llan 6 0 a   ${file_source} > ${test_current}literal-line-6-at-0-name-a.expect &&
  generate_operate_run +n -Llan 7 0 a   ${file_source} > ${test_current}literal-line-7-at-0-name-a.expect &&
  generate_operate_run +n -Llan 8 0 a   ${file_source} > ${test_current}literal-line-8-at-0-name-a.expect &&
  generate_operate_run +n -Llan 9 0 a   ${file_source} > ${test_current}literal-line-9-at-0-name-a.expect &&
  generate_operate_run +n -Llan 10 0 a  ${file_source} > ${test_current}literal-line-10-at-0-name-a.expect &&
  generate_operate_run +n -Llan 20 0 a  ${file_source} > ${test_current}literal-line-20-at-0-name-a.expect &&
  generate_operate_run +n -Llan 100 0 a ${file_source} > ${test_current}literal-line-100-at-0-name-a.expect &&
  generate_operate_run +n -Llan 0 1 a   ${file_source} > ${test_current}literal-line-0-at-1-name-a.expect &&
  generate_operate_run +n -Llan 1 1 a   ${file_source} > ${test_current}literal-line-1-at-1-name-a.expect &&
  generate_operate_run +n -Llan 2 1 a   ${file_source} > ${test_current}literal-line-2-at-1-name-a.expect &&
  generate_operate_run +n -Llan 3 1 a   ${file_source} > ${test_current}literal-line-3-at-1-name-a.expect &&
  generate_operate_run +n -Llan 4 1 a   ${file_source} > ${test_current}literal-line-4-at-1-name-a.expect &&
  generate_operate_run +n -Llan 5 1 a   ${file_source} > ${test_current}literal-line-5-at-1-name-a.expect &&
  generate_operate_run +n -Llan 6 1 a   ${file_source} > ${test_current}literal-line-6-at-1-name-a.expect &&
  generate_operate_run +n -Llan 7 1 a   ${file_source} > ${test_current}literal-line-7-at-1-name-a.expect &&
  generate_operate_run +n -Llan 8 1 a   ${file_source} > ${test_current}literal-line-8-at-1-name-a.expect &&
  generate_operate_run +n -Llan 9 1 a   ${file_source} > ${test_current}literal-line-9-at-1-name-a.expect &&
  generate_operate_run +n -Llan 10 1 a  ${file_source} > ${test_current}literal-line-10-at-1-name-a.expect &&
  generate_operate_run +n -Llan 20 1 a  ${file_source} > ${test_current}literal-line-20-at-1-name-a.expect &&
  generate_operate_run +n -Llan 100 1 a ${file_source} > ${test_current}literal-line-100-at-1-name-a.expect &&

  generate_operate_run +n -Ln b         ${file_source} > ${test_current}literal-name-b.expect &&
  generate_operate_run +n -Lln 0 b      ${file_source} > ${test_current}literal-line-0-name-b.expect &&
  generate_operate_run +n -Lln 1 b      ${file_source} > ${test_current}literal-line-1-name-b.expect &&
  generate_operate_run +n -Lln 2 b      ${file_source} > ${test_current}literal-line-2-name-b.expect &&
  generate_operate_run +n -Lln 3 b      ${file_source} > ${test_current}literal-line-3-name-b.expect &&
  generate_operate_run +n -Lln 4 b      ${file_source} > ${test_current}literal-line-4-name-b.expect &&
  generate_operate_run +n -Lln 5 b      ${file_source} > ${test_current}literal-line-5-name-b.expect &&
  generate_operate_run +n -Lln 6 b      ${file_source} > ${test_current}literal-line-6-name-b.expect &&
  generate_operate_run +n -Lln 7 b      ${file_source} > ${test_current}literal-line-7-name-b.expect &&
  generate_operate_run +n -Lln 8 b      ${file_source} > ${test_current}literal-line-8-name-b.expect &&
  generate_operate_run +n -Lln 9 b      ${file_source} > ${test_current}literal-line-9-name-b.expect &&
  generate_operate_run +n -Lln 10 b     ${file_source} > ${test_current}literal-line-10-name-b.expect &&
  generate_operate_run +n -Lln 20 b     ${file_source} > ${test_current}literal-line-20-name-b.expect &&
  generate_operate_run +n -Lln 100 b    ${file_source} > ${test_current}literal-line-100-name-b.expect &&
  generate_operate_run +n -Llan 0 0 b   ${file_source} > ${test_current}literal-line-0-at-0-name-b.expect &&
  generate_operate_run +n -Llan 1 0 b   ${file_source} > ${test_current}literal-line-1-at-0-name-b.expect &&
  generate_operate_run +n -Llan 2 0 b   ${file_source} > ${test_current}literal-line-2-at-0-name-b.expect &&
  generate_operate_run +n -Llan 3 0 b   ${file_source} > ${test_current}literal-line-3-at-0-name-b.expect &&
  generate_operate_run +n -Llan 4 0 b   ${file_source} > ${test_current}literal-line-4-at-0-name-b.expect &&
  generate_operate_run +n -Llan 5 0 b   ${file_source} > ${test_current}literal-line-5-at-0-name-b.expect &&
  generate_operate_run +n -Llan 6 0 b   ${file_source} > ${test_current}literal-line-6-at-0-name-b.expect &&
  generate_operate_run +n -Llan 7 0 b   ${file_source} > ${test_current}literal-line-7-at-0-name-b.expect &&
  generate_operate_run +n -Llan 8 0 b   ${file_source} > ${test_current}literal-line-8-at-0-name-b.expect &&
  generate_operate_run +n -Llan 9 0 b   ${file_source} > ${test_current}literal-line-9-at-0-name-b.expect &&
  generate_operate_run +n -Llan 10 0 b  ${file_source} > ${test_current}literal-line-10-at-0-name-b.expect &&
  generate_operate_run +n -Llan 20 0 b  ${file_source} > ${test_current}literal-line-20-at-0-name-b.expect &&
  generate_operate_run +n -Llan 100 0 b ${file_source} > ${test_current}literal-line-100-at-0-name-b.expect &&
  generate_operate_run +n -Llan 0 1 b   ${file_source} > ${test_current}literal-line-0-at-1-name-b.expect &&
  generate_operate_run +n -Llan 1 1 b   ${file_source} > ${test_current}literal-line-1-at-1-name-b.expect &&
  generate_operate_run +n -Llan 2 1 b   ${file_source} > ${test_current}literal-line-2-at-1-name-b.expect &&
  generate_operate_run +n -Llan 3 1 b   ${file_source} > ${test_current}literal-line-3-at-1-name-b.expect &&
  generate_operate_run +n -Llan 4 1 b   ${file_source} > ${test_current}literal-line-4-at-1-name-b.expect &&
  generate_operate_run +n -Llan 5 1 b   ${file_source} > ${test_current}literal-line-5-at-1-name-b.expect &&
  generate_operate_run +n -Llan 6 1 b   ${file_source} > ${test_current}literal-line-6-at-1-name-b.expect &&
  generate_operate_run +n -Llan 7 1 b   ${file_source} > ${test_current}literal-line-7-at-1-name-b.expect &&
  generate_operate_run +n -Llan 8 1 b   ${file_source} > ${test_current}literal-line-8-at-1-name-b.expect &&
  generate_operate_run +n -Llan 9 1 b   ${file_source} > ${test_current}literal-line-9-at-1-name-b.expect &&
  generate_operate_run +n -Llan 10 1 b  ${file_source} > ${test_current}literal-line-10-at-1-name-b.expect &&
  generate_operate_run +n -Llan 20 1 b  ${file_source} > ${test_current}literal-line-20-at-1-name-b.expect &&
  generate_operate_run +n -Llan 100 1 b ${file_source} > ${test_current}literal-line-100-at-1-name-b.expect &&

  generate_operate_run +n -Ln is         ${file_source} > ${test_current}literal-name-is.expect &&
  generate_operate_run +n -Lln 0 is      ${file_source} > ${test_current}literal-line-0-name-is.expect &&
  generate_operate_run +n -Lln 1 is      ${file_source} > ${test_current}literal-line-1-name-is.expect &&
  generate_operate_run +n -Lln 2 is      ${file_source} > ${test_current}literal-line-2-name-is.expect &&
  generate_operate_run +n -Lln 3 is      ${file_source} > ${test_current}literal-line-3-name-is.expect &&
  generate_operate_run +n -Lln 4 is      ${file_source} > ${test_current}literal-line-4-name-is.expect &&
  generate_operate_run +n -Lln 5 is      ${file_source} > ${test_current}literal-line-5-name-is.expect &&
  generate_operate_run +n -Lln 6 is      ${file_source} > ${test_current}literal-line-6-name-is.expect &&
  generate_operate_run +n -Lln 7 is      ${file_source} > ${test_current}literal-line-7-name-is.expect &&
  generate_operate_run +n -Lln 8 is      ${file_source} > ${test_current}literal-line-8-name-is.expect &&
  generate_operate_run +n -Lln 9 is      ${file_source} > ${test_current}literal-line-9-name-is.expect &&
  generate_operate_run +n -Lln 10 is     ${file_source} > ${test_current}literal-line-10-name-is.expect &&
  generate_operate_run +n -Lln 20 is     ${file_source} > ${test_current}literal-line-20-name-is.expect &&
  generate_operate_run +n -Lln 100 is    ${file_source} > ${test_current}literal-line-100-name-is.expect &&
  generate_operate_run +n -Llan 0 0 is   ${file_source} > ${test_current}literal-line-0-at-0-name-is.expect &&
  generate_operate_run +n -Llan 1 0 is   ${file_source} > ${test_current}literal-line-1-at-0-name-is.expect &&
  generate_operate_run +n -Llan 2 0 is   ${file_source} > ${test_current}literal-line-2-at-0-name-is.expect &&
  generate_operate_run +n -Llan 3 0 is   ${file_source} > ${test_current}literal-line-3-at-0-name-is.expect &&
  generate_operate_run +n -Llan 4 0 is   ${file_source} > ${test_current}literal-line-4-at-0-name-is.expect &&
  generate_operate_run +n -Llan 5 0 is   ${file_source} > ${test_current}literal-line-5-at-0-name-is.expect &&
  generate_operate_run +n -Llan 6 0 is   ${file_source} > ${test_current}literal-line-6-at-0-name-is.expect &&
  generate_operate_run +n -Llan 7 0 is   ${file_source} > ${test_current}literal-line-7-at-0-name-is.expect &&
  generate_operate_run +n -Llan 8 0 is   ${file_source} > ${test_current}literal-line-8-at-0-name-is.expect &&
  generate_operate_run +n -Llan 9 0 is   ${file_source} > ${test_current}literal-line-9-at-0-name-is.expect &&
  generate_operate_run +n -Llan 10 0 is  ${file_source} > ${test_current}literal-line-10-at-0-name-is.expect &&
  generate_operate_run +n -Llan 20 0 is  ${file_source} > ${test_current}literal-line-20-at-0-name-is.expect &&
  generate_operate_run +n -Llan 100 0 is ${file_source} > ${test_current}literal-line-100-at-0-name-is.expect &&
  generate_operate_run +n -Llan 0 1 is   ${file_source} > ${test_current}literal-line-0-at-1-name-is.expect &&
  generate_operate_run +n -Llan 1 1 is   ${file_source} > ${test_current}literal-line-1-at-1-name-is.expect &&
  generate_operate_run +n -Llan 2 1 is   ${file_source} > ${test_current}literal-line-2-at-1-name-is.expect &&
  generate_operate_run +n -Llan 3 1 is   ${file_source} > ${test_current}literal-line-3-at-1-name-is.expect &&
  generate_operate_run +n -Llan 4 1 is   ${file_source} > ${test_current}literal-line-4-at-1-name-is.expect &&
  generate_operate_run +n -Llan 5 1 is   ${file_source} > ${test_current}literal-line-5-at-1-name-is.expect &&
  generate_operate_run +n -Llan 6 1 is   ${file_source} > ${test_current}literal-line-6-at-1-name-is.expect &&
  generate_operate_run +n -Llan 7 1 is   ${file_source} > ${test_current}literal-line-7-at-1-name-is.expect &&
  generate_operate_run +n -Llan 8 1 is   ${file_source} > ${test_current}literal-line-8-at-1-name-is.expect &&
  generate_operate_run +n -Llan 9 1 is   ${file_source} > ${test_current}literal-line-9-at-1-name-is.expect &&
  generate_operate_run +n -Llan 10 1 is  ${file_source} > ${test_current}literal-line-10-at-1-name-is.expect &&
  generate_operate_run +n -Llan 20 1 is  ${file_source} > ${test_current}literal-line-20-at-1-name-is.expect &&
  generate_operate_run +n -Llan 100 1 is ${file_source} > ${test_current}literal-line-100-at-1-name-is.expect &&

  generate_operate_run +n -Ln yup         ${file_source} > ${test_current}literal-name-yup.expect &&
  generate_operate_run +n -Lln 0 yup      ${file_source} > ${test_current}literal-line-0-name-yup.expect &&
  generate_operate_run +n -Lln 1 yup      ${file_source} > ${test_current}literal-line-1-name-yup.expect &&
  generate_operate_run +n -Lln 2 yup      ${file_source} > ${test_current}literal-line-2-name-yup.expect &&
  generate_operate_run +n -Lln 3 yup      ${file_source} > ${test_current}literal-line-3-name-yup.expect &&
  generate_operate_run +n -Lln 4 yup      ${file_source} > ${test_current}literal-line-4-name-yup.expect &&
  generate_operate_run +n -Lln 5 yup      ${file_source} > ${test_current}literal-line-5-name-yup.expect &&
  generate_operate_run +n -Lln 6 yup      ${file_source} > ${test_current}literal-line-6-name-yup.expect &&
  generate_operate_run +n -Lln 7 yup      ${file_source} > ${test_current}literal-line-7-name-yup.expect &&
  generate_operate_run +n -Lln 8 yup      ${file_source} > ${test_current}literal-line-8-name-yup.expect &&
  generate_operate_run +n -Lln 9 yup      ${file_source} > ${test_current}literal-line-9-name-yup.expect &&
  generate_operate_run +n -Lln 10 yup     ${file_source} > ${test_current}literal-line-10-name-yup.expect &&
  generate_operate_run +n -Lln 20 yup     ${file_source} > ${test_current}literal-line-20-name-yup.expect &&
  generate_operate_run +n -Lln 100 yup    ${file_source} > ${test_current}literal-line-100-name-yup.expect &&
  generate_operate_run +n -Llan 0 0 yup   ${file_source} > ${test_current}literal-line-0-at-0-name-yup.expect &&
  generate_operate_run +n -Llan 1 0 yup   ${file_source} > ${test_current}literal-line-1-at-0-name-yup.expect &&
  generate_operate_run +n -Llan 2 0 yup   ${file_source} > ${test_current}literal-line-2-at-0-name-yup.expect &&
  generate_operate_run +n -Llan 3 0 yup   ${file_source} > ${test_current}literal-line-3-at-0-name-yup.expect &&
  generate_operate_run +n -Llan 4 0 yup   ${file_source} > ${test_current}literal-line-4-at-0-name-yup.expect &&
  generate_operate_run +n -Llan 5 0 yup   ${file_source} > ${test_current}literal-line-5-at-0-name-yup.expect &&
  generate_operate_run +n -Llan 6 0 yup   ${file_source} > ${test_current}literal-line-6-at-0-name-yup.expect &&
  generate_operate_run +n -Llan 7 0 yup   ${file_source} > ${test_current}literal-line-7-at-0-name-yup.expect &&
  generate_operate_run +n -Llan 8 0 yup   ${file_source} > ${test_current}literal-line-8-at-0-name-yup.expect &&
  generate_operate_run +n -Llan 9 0 yup   ${file_source} > ${test_current}literal-line-9-at-0-name-yup.expect &&
  generate_operate_run +n -Llan 10 0 yup  ${file_source} > ${test_current}literal-line-10-at-0-name-yup.expect &&
  generate_operate_run +n -Llan 20 0 yup  ${file_source} > ${test_current}literal-line-20-at-0-name-yup.expect &&
  generate_operate_run +n -Llan 100 0 yup ${file_source} > ${test_current}literal-line-100-at-0-name-yup.expect &&
  generate_operate_run +n -Llan 0 1 yup   ${file_source} > ${test_current}literal-line-0-at-1-name-yup.expect &&
  generate_operate_run +n -Llan 1 1 yup   ${file_source} > ${test_current}literal-line-1-at-1-name-yup.expect &&
  generate_operate_run +n -Llan 2 1 yup   ${file_source} > ${test_current}literal-line-2-at-1-name-yup.expect &&
  generate_operate_run +n -Llan 3 1 yup   ${file_source} > ${test_current}literal-line-3-at-1-name-yup.expect &&
  generate_operate_run +n -Llan 4 1 yup   ${file_source} > ${test_current}literal-line-4-at-1-name-yup.expect &&
  generate_operate_run +n -Llan 5 1 yup   ${file_source} > ${test_current}literal-line-5-at-1-name-yup.expect &&
  generate_operate_run +n -Llan 6 1 yup   ${file_source} > ${test_current}literal-line-6-at-1-name-yup.expect &&
  generate_operate_run +n -Llan 7 1 yup   ${file_source} > ${test_current}literal-line-7-at-1-name-yup.expect &&
  generate_operate_run +n -Llan 8 1 yup   ${file_source} > ${test_current}literal-line-8-at-1-name-yup.expect &&
  generate_operate_run +n -Llan 9 1 yup   ${file_source} > ${test_current}literal-line-9-at-1-name-yup.expect &&
  generate_operate_run +n -Llan 10 1 yup  ${file_source} > ${test_current}literal-line-10-at-1-name-yup.expect &&
  generate_operate_run +n -Llan 20 1 yup  ${file_source} > ${test_current}literal-line-20-at-1-name-yup.expect &&

  #### LITERAL - TOTAL ####
  generate_operate_run +n -Lt         ${file_source} > ${test_current}literal-total.expect &&
  generate_operate_run +n -Llt 0      ${file_source} > ${test_current}literal-line-0-total.expect &&
  generate_operate_run +n -Llt 1      ${file_source} > ${test_current}literal-line-1-total.expect &&
  generate_operate_run +n -Llt 2      ${file_source} > ${test_current}literal-line-2-total.expect &&
  generate_operate_run +n -Llt 3      ${file_source} > ${test_current}literal-line-3-total.expect &&
  generate_operate_run +n -Llt 4      ${file_source} > ${test_current}literal-line-4-total.expect &&
  generate_operate_run +n -Llt 5      ${file_source} > ${test_current}literal-line-5-total.expect &&
  generate_operate_run +n -Llt 6      ${file_source} > ${test_current}literal-line-6-total.expect &&
  generate_operate_run +n -Llt 7      ${file_source} > ${test_current}literal-line-7-total.expect &&
  generate_operate_run +n -Llt 8      ${file_source} > ${test_current}literal-line-8-total.expect &&
  generate_operate_run +n -Llt 9      ${file_source} > ${test_current}literal-line-9-total.expect &&
  generate_operate_run +n -Llt 10     ${file_source} > ${test_current}literal-line-10-total.expect &&
  generate_operate_run +n -Llt 20     ${file_source} > ${test_current}literal-line-20-total.expect &&
  generate_operate_run +n -Llt 100    ${file_source} > ${test_current}literal-line-100-total.expect &&
  generate_operate_run +n -Llat 0 0   ${file_source} > ${test_current}literal-line-0-at-0-total.expect &&
  generate_operate_run +n -Llat 1 0   ${file_source} > ${test_current}literal-line-1-at-0-total.expect &&
  generate_operate_run +n -Llat 2 0   ${file_source} > ${test_current}literal-line-2-at-0-total.expect &&
  generate_operate_run +n -Llat 3 0   ${file_source} > ${test_current}literal-line-3-at-0-total.expect &&
  generate_operate_run +n -Llat 4 0   ${file_source} > ${test_current}literal-line-4-at-0-total.expect &&
  generate_operate_run +n -Llat 5 0   ${file_source} > ${test_current}literal-line-5-at-0-total.expect &&
  generate_operate_run +n -Llat 6 0   ${file_source} > ${test_current}literal-line-6-at-0-total.expect &&
  generate_operate_run +n -Llat 7 0   ${file_source} > ${test_current}literal-line-7-at-0-total.expect &&
  generate_operate_run +n -Llat 8 0   ${file_source} > ${test_current}literal-line-8-at-0-total.expect &&
  generate_operate_run +n -Llat 9 0   ${file_source} > ${test_current}literal-line-9-at-0-total.expect &&
  generate_operate_run +n -Llat 10 0  ${file_source} > ${test_current}literal-line-10-at-0-total.expect &&
  generate_operate_run +n -Llat 20 0  ${file_source} > ${test_current}literal-line-20-at-0-total.expect &&
  generate_operate_run +n -Llat 100 0 ${file_source} > ${test_current}literal-line-100-at-0-total.expect &&
  generate_operate_run +n -Llat 0 1   ${file_source} > ${test_current}literal-line-0-at-1-total.expect &&
  generate_operate_run +n -Llat 1 1   ${file_source} > ${test_current}literal-line-1-at-1-total.expect &&
  generate_operate_run +n -Llat 2 1   ${file_source} > ${test_current}literal-line-2-at-1-total.expect &&
  generate_operate_run +n -Llat 3 1   ${file_source} > ${test_current}literal-line-3-at-1-total.expect &&
  generate_operate_run +n -Llat 4 1   ${file_source} > ${test_current}literal-line-4-at-1-total.expect &&
  generate_operate_run +n -Llat 5 1   ${file_source} > ${test_current}literal-line-5-at-1-total.expect &&
  generate_operate_run +n -Llat 6 1   ${file_source} > ${test_current}literal-line-6-at-1-total.expect &&
  generate_operate_run +n -Llat 7 1   ${file_source} > ${test_current}literal-line-7-at-1-total.expect &&
  generate_operate_run +n -Llat 8 1   ${file_source} > ${test_current}literal-line-8-at-1-total.expect &&
  generate_operate_run +n -Llat 9 1   ${file_source} > ${test_current}literal-line-9-at-1-total.expect &&
  generate_operate_run +n -Llat 10 1  ${file_source} > ${test_current}literal-line-10-at-1-total.expect &&
  generate_operate_run +n -Llat 20 1  ${file_source} > ${test_current}literal-line-20-at-1-total.expect &&
  generate_operate_run +n -Llat 100 1 ${file_source} > ${test_current}literal-line-100-at-1-total.expect &&

  generate_operate_run +n -Lnt a         ${file_source} > ${test_current}literal-name-a-total.expect &&
  generate_operate_run +n -Llnt 0 a      ${file_source} > ${test_current}literal-line-0-name-a-total.expect &&
  generate_operate_run +n -Llnt 1 a      ${file_source} > ${test_current}literal-line-1-name-a-total.expect &&
  generate_operate_run +n -Llnt 2 a      ${file_source} > ${test_current}literal-line-2-name-a-total.expect &&
  generate_operate_run +n -Llnt 3 a      ${file_source} > ${test_current}literal-line-3-name-a-total.expect &&
  generate_operate_run +n -Llnt 4 a      ${file_source} > ${test_current}literal-line-4-name-a-total.expect &&
  generate_operate_run +n -Llnt 5 a      ${file_source} > ${test_current}literal-line-5-name-a-total.expect &&
  generate_operate_run +n -Llnt 6 a      ${file_source} > ${test_current}literal-line-6-name-a-total.expect &&
  generate_operate_run +n -Llnt 7 a      ${file_source} > ${test_current}literal-line-7-name-a-total.expect &&
  generate_operate_run +n -Llnt 8 a      ${file_source} > ${test_current}literal-line-8-name-a-total.expect &&
  generate_operate_run +n -Llnt 9 a      ${file_source} > ${test_current}literal-line-9-name-a-total.expect &&
  generate_operate_run +n -Llnt 10 a     ${file_source} > ${test_current}literal-line-10-name-a-total.expect &&
  generate_operate_run +n -Llnt 20 a     ${file_source} > ${test_current}literal-line-20-name-a-total.expect &&
  generate_operate_run +n -Llnt 100 a    ${file_source} > ${test_current}literal-line-100-name-a-total.expect &&
  generate_operate_run +n -Llant 0 0 a   ${file_source} > ${test_current}literal-line-0-at-0-name-a-total.expect &&
  generate_operate_run +n -Llant 1 0 a   ${file_source} > ${test_current}literal-line-1-at-0-name-a-total.expect &&
  generate_operate_run +n -Llant 2 0 a   ${file_source} > ${test_current}literal-line-2-at-0-name-a-total.expect &&
  generate_operate_run +n -Llant 3 0 a   ${file_source} > ${test_current}literal-line-3-at-0-name-a-total.expect &&
  generate_operate_run +n -Llant 4 0 a   ${file_source} > ${test_current}literal-line-4-at-0-name-a-total.expect &&
  generate_operate_run +n -Llant 5 0 a   ${file_source} > ${test_current}literal-line-5-at-0-name-a-total.expect &&
  generate_operate_run +n -Llant 6 0 a   ${file_source} > ${test_current}literal-line-6-at-0-name-a-total.expect &&
  generate_operate_run +n -Llant 7 0 a   ${file_source} > ${test_current}literal-line-7-at-0-name-a-total.expect &&
  generate_operate_run +n -Llant 8 0 a   ${file_source} > ${test_current}literal-line-8-at-0-name-a-total.expect &&
  generate_operate_run +n -Llant 9 0 a   ${file_source} > ${test_current}literal-line-9-at-0-name-a-total.expect &&
  generate_operate_run +n -Llant 10 0 a  ${file_source} > ${test_current}literal-line-10-at-0-name-a-total.expect &&
  generate_operate_run +n -Llant 20 0 a  ${file_source} > ${test_current}literal-line-20-at-0-name-a-total.expect &&
  generate_operate_run +n -Llant 100 0 a ${file_source} > ${test_current}literal-line-100-at-0-name-a-total.expect &&
  generate_operate_run +n -Llant 0 1 a   ${file_source} > ${test_current}literal-line-0-at-1-name-a-total.expect &&
  generate_operate_run +n -Llant 1 1 a   ${file_source} > ${test_current}literal-line-1-at-1-name-a-total.expect &&
  generate_operate_run +n -Llant 2 1 a   ${file_source} > ${test_current}literal-line-2-at-1-name-a-total.expect &&
  generate_operate_run +n -Llant 3 1 a   ${file_source} > ${test_current}literal-line-3-at-1-name-a-total.expect &&
  generate_operate_run +n -Llant 4 1 a   ${file_source} > ${test_current}literal-line-4-at-1-name-a-total.expect &&
  generate_operate_run +n -Llant 5 1 a   ${file_source} > ${test_current}literal-line-5-at-1-name-a-total.expect &&
  generate_operate_run +n -Llant 6 1 a   ${file_source} > ${test_current}literal-line-6-at-1-name-a-total.expect &&
  generate_operate_run +n -Llant 7 1 a   ${file_source} > ${test_current}literal-line-7-at-1-name-a-total.expect &&
  generate_operate_run +n -Llant 8 1 a   ${file_source} > ${test_current}literal-line-8-at-1-name-a-total.expect &&
  generate_operate_run +n -Llant 9 1 a   ${file_source} > ${test_current}literal-line-9-at-1-name-a-total.expect &&
  generate_operate_run +n -Llant 10 1 a  ${file_source} > ${test_current}literal-line-10-at-1-name-a-total.expect &&
  generate_operate_run +n -Llant 20 1 a  ${file_source} > ${test_current}literal-line-20-at-1-name-a-total.expect &&
  generate_operate_run +n -Llant 100 1 a ${file_source} > ${test_current}literal-line-100-at-1-name-a-total.expect &&

  generate_operate_run +n -Lnt b         ${file_source} > ${test_current}literal-name-b-total.expect &&
  generate_operate_run +n -Llnt 0 b      ${file_source} > ${test_current}literal-line-0-name-b-total.expect &&
  generate_operate_run +n -Llnt 1 b      ${file_source} > ${test_current}literal-line-1-name-b-total.expect &&
  generate_operate_run +n -Llnt 2 b      ${file_source} > ${test_current}literal-line-2-name-b-total.expect &&
  generate_operate_run +n -Llnt 3 b      ${file_source} > ${test_current}literal-line-3-name-b-total.expect &&
  generate_operate_run +n -Llnt 4 b      ${file_source} > ${test_current}literal-line-4-name-b-total.expect &&
  generate_operate_run +n -Llnt 5 b      ${file_source} > ${test_current}literal-line-5-name-b-total.expect &&
  generate_operate_run +n -Llnt 6 b      ${file_source} > ${test_current}literal-line-6-name-b-total.expect &&
  generate_operate_run +n -Llnt 7 b      ${file_source} > ${test_current}literal-line-7-name-b-total.expect &&
  generate_operate_run +n -Llnt 8 b      ${file_source} > ${test_current}literal-line-8-name-b-total.expect &&
  generate_operate_run +n -Llnt 9 b      ${file_source} > ${test_current}literal-line-9-name-b-total.expect &&
  generate_operate_run +n -Llnt 10 b     ${file_source} > ${test_current}literal-line-10-name-b-total.expect &&
  generate_operate_run +n -Llnt 20 b     ${file_source} > ${test_current}literal-line-20-name-b-total.expect &&
  generate_operate_run +n -Llnt 100 b    ${file_source} > ${test_current}literal-line-100-name-b-total.expect &&
  generate_operate_run +n -Llant 0 0 b   ${file_source} > ${test_current}literal-line-0-at-0-name-b-total.expect &&
  generate_operate_run +n -Llant 1 0 b   ${file_source} > ${test_current}literal-line-1-at-0-name-b-total.expect &&
  generate_operate_run +n -Llant 2 0 b   ${file_source} > ${test_current}literal-line-2-at-0-name-b-total.expect &&
  generate_operate_run +n -Llant 3 0 b   ${file_source} > ${test_current}literal-line-3-at-0-name-b-total.expect &&
  generate_operate_run +n -Llant 4 0 b   ${file_source} > ${test_current}literal-line-4-at-0-name-b-total.expect &&
  generate_operate_run +n -Llant 5 0 b   ${file_source} > ${test_current}literal-line-5-at-0-name-b-total.expect &&
  generate_operate_run +n -Llant 6 0 b   ${file_source} > ${test_current}literal-line-6-at-0-name-b-total.expect &&
  generate_operate_run +n -Llant 7 0 b   ${file_source} > ${test_current}literal-line-7-at-0-name-b-total.expect &&
  generate_operate_run +n -Llant 8 0 b   ${file_source} > ${test_current}literal-line-8-at-0-name-b-total.expect &&
  generate_operate_run +n -Llant 9 0 b   ${file_source} > ${test_current}literal-line-9-at-0-name-b-total.expect &&
  generate_operate_run +n -Llant 10 0 b  ${file_source} > ${test_current}literal-line-10-at-0-name-b-total.expect &&
  generate_operate_run +n -Llant 20 0 b  ${file_source} > ${test_current}literal-line-20-at-0-name-b-total.expect &&
  generate_operate_run +n -Llant 100 0 b ${file_source} > ${test_current}literal-line-100-at-0-name-b-total.expect &&
  generate_operate_run +n -Llant 0 1 b   ${file_source} > ${test_current}literal-line-0-at-1-name-b-total.expect &&
  generate_operate_run +n -Llant 1 1 b   ${file_source} > ${test_current}literal-line-1-at-1-name-b-total.expect &&
  generate_operate_run +n -Llant 2 1 b   ${file_source} > ${test_current}literal-line-2-at-1-name-b-total.expect &&
  generate_operate_run +n -Llant 3 1 b   ${file_source} > ${test_current}literal-line-3-at-1-name-b-total.expect &&
  generate_operate_run +n -Llant 4 1 b   ${file_source} > ${test_current}literal-line-4-at-1-name-b-total.expect &&
  generate_operate_run +n -Llant 5 1 b   ${file_source} > ${test_current}literal-line-5-at-1-name-b-total.expect &&
  generate_operate_run +n -Llant 6 1 b   ${file_source} > ${test_current}literal-line-6-at-1-name-b-total.expect &&
  generate_operate_run +n -Llant 7 1 b   ${file_source} > ${test_current}literal-line-7-at-1-name-b-total.expect &&
  generate_operate_run +n -Llant 8 1 b   ${file_source} > ${test_current}literal-line-8-at-1-name-b-total.expect &&
  generate_operate_run +n -Llant 9 1 b   ${file_source} > ${test_current}literal-line-9-at-1-name-b-total.expect &&
  generate_operate_run +n -Llant 10 1 b  ${file_source} > ${test_current}literal-line-10-at-1-name-b-total.expect &&
  generate_operate_run +n -Llant 20 1 b  ${file_source} > ${test_current}literal-line-20-at-1-name-b-total.expect &&
  generate_operate_run +n -Llant 100 1 b ${file_source} > ${test_current}literal-line-100-at-1-name-b-total.expect &&

  generate_operate_run +n -Lnt is         ${file_source} > ${test_current}literal-name-is-total.expect &&
  generate_operate_run +n -Llnt 0 is      ${file_source} > ${test_current}literal-line-0-name-is-total.expect &&
  generate_operate_run +n -Llnt 1 is      ${file_source} > ${test_current}literal-line-1-name-is-total.expect &&
  generate_operate_run +n -Llnt 2 is      ${file_source} > ${test_current}literal-line-2-name-is-total.expect &&
  generate_operate_run +n -Llnt 3 is      ${file_source} > ${test_current}literal-line-3-name-is-total.expect &&
  generate_operate_run +n -Llnt 4 is      ${file_source} > ${test_current}literal-line-4-name-is-total.expect &&
  generate_operate_run +n -Llnt 5 is      ${file_source} > ${test_current}literal-line-5-name-is-total.expect &&
  generate_operate_run +n -Llnt 6 is      ${file_source} > ${test_current}literal-line-6-name-is-total.expect &&
  generate_operate_run +n -Llnt 7 is      ${file_source} > ${test_current}literal-line-7-name-is-total.expect &&
  generate_operate_run +n -Llnt 8 is      ${file_source} > ${test_current}literal-line-8-name-is-total.expect &&
  generate_operate_run +n -Llnt 9 is      ${file_source} > ${test_current}literal-line-9-name-is-total.expect &&
  generate_operate_run +n -Llnt 10 is     ${file_source} > ${test_current}literal-line-10-name-is-total.expect &&
  generate_operate_run +n -Llnt 20 is     ${file_source} > ${test_current}literal-line-20-name-is-total.expect &&
  generate_operate_run +n -Llnt 100 is    ${file_source} > ${test_current}literal-line-100-name-is-total.expect &&
  generate_operate_run +n -Llant 0 0 is   ${file_source} > ${test_current}literal-line-0-at-0-name-is-total.expect &&
  generate_operate_run +n -Llant 1 0 is   ${file_source} > ${test_current}literal-line-1-at-0-name-is-total.expect &&
  generate_operate_run +n -Llant 2 0 is   ${file_source} > ${test_current}literal-line-2-at-0-name-is-total.expect &&
  generate_operate_run +n -Llant 3 0 is   ${file_source} > ${test_current}literal-line-3-at-0-name-is-total.expect &&
  generate_operate_run +n -Llant 4 0 is   ${file_source} > ${test_current}literal-line-4-at-0-name-is-total.expect &&
  generate_operate_run +n -Llant 5 0 is   ${file_source} > ${test_current}literal-line-5-at-0-name-is-total.expect &&
  generate_operate_run +n -Llant 6 0 is   ${file_source} > ${test_current}literal-line-6-at-0-name-is-total.expect &&
  generate_operate_run +n -Llant 7 0 is   ${file_source} > ${test_current}literal-line-7-at-0-name-is-total.expect &&
  generate_operate_run +n -Llant 8 0 is   ${file_source} > ${test_current}literal-line-8-at-0-name-is-total.expect &&
  generate_operate_run +n -Llant 9 0 is   ${file_source} > ${test_current}literal-line-9-at-0-name-is-total.expect &&
  generate_operate_run +n -Llant 10 0 is  ${file_source} > ${test_current}literal-line-10-at-0-name-is-total.expect &&
  generate_operate_run +n -Llant 20 0 is  ${file_source} > ${test_current}literal-line-20-at-0-name-is-total.expect &&
  generate_operate_run +n -Llant 100 0 is ${file_source} > ${test_current}literal-line-100-at-0-name-is-total.expect &&
  generate_operate_run +n -Llant 0 1 is   ${file_source} > ${test_current}literal-line-0-at-1-name-is-total.expect &&
  generate_operate_run +n -Llant 1 1 is   ${file_source} > ${test_current}literal-line-1-at-1-name-is-total.expect &&
  generate_operate_run +n -Llant 2 1 is   ${file_source} > ${test_current}literal-line-2-at-1-name-is-total.expect &&
  generate_operate_run +n -Llant 3 1 is   ${file_source} > ${test_current}literal-line-3-at-1-name-is-total.expect &&
  generate_operate_run +n -Llant 4 1 is   ${file_source} > ${test_current}literal-line-4-at-1-name-is-total.expect &&
  generate_operate_run +n -Llant 5 1 is   ${file_source} > ${test_current}literal-line-5-at-1-name-is-total.expect &&
  generate_operate_run +n -Llant 6 1 is   ${file_source} > ${test_current}literal-line-6-at-1-name-is-total.expect &&
  generate_operate_run +n -Llant 7 1 is   ${file_source} > ${test_current}literal-line-7-at-1-name-is-total.expect &&
  generate_operate_run +n -Llant 8 1 is   ${file_source} > ${test_current}literal-line-8-at-1-name-is-total.expect &&
  generate_operate_run +n -Llant 9 1 is   ${file_source} > ${test_current}literal-line-9-at-1-name-is-total.expect &&
  generate_operate_run +n -Llant 10 1 is  ${file_source} > ${test_current}literal-line-10-at-1-name-is-total.expect &&
  generate_operate_run +n -Llant 20 1 is  ${file_source} > ${test_current}literal-line-20-at-1-name-is-total.expect &&
  generate_operate_run +n -Llant 100 1 is ${file_source} > ${test_current}literal-line-100-at-1-name-is-total.expect &&

  generate_operate_run +n -Lnt yup         ${file_source} > ${test_current}literal-name-yup-total.expect &&
  generate_operate_run +n -Llnt 0 yup      ${file_source} > ${test_current}literal-line-0-name-yup-total.expect &&
  generate_operate_run +n -Llnt 1 yup      ${file_source} > ${test_current}literal-line-1-name-yup-total.expect &&
  generate_operate_run +n -Llnt 2 yup      ${file_source} > ${test_current}literal-line-2-name-yup-total.expect &&
  generate_operate_run +n -Llnt 3 yup      ${file_source} > ${test_current}literal-line-3-name-yup-total.expect &&
  generate_operate_run +n -Llnt 4 yup      ${file_source} > ${test_current}literal-line-4-name-yup-total.expect &&
  generate_operate_run +n -Llnt 5 yup      ${file_source} > ${test_current}literal-line-5-name-yup-total.expect &&
  generate_operate_run +n -Llnt 6 yup      ${file_source} > ${test_current}literal-line-6-name-yup-total.expect &&
  generate_operate_run +n -Llnt 7 yup      ${file_source} > ${test_current}literal-line-7-name-yup-total.expect &&
  generate_operate_run +n -Llnt 8 yup      ${file_source} > ${test_current}literal-line-8-name-yup-total.expect &&
  generate_operate_run +n -Llnt 9 yup      ${file_source} > ${test_current}literal-line-9-name-yup-total.expect &&
  generate_operate_run +n -Llnt 10 yup     ${file_source} > ${test_current}literal-line-10-name-yup-total.expect &&
  generate_operate_run +n -Llnt 20 yup     ${file_source} > ${test_current}literal-line-20-name-yup-total.expect &&
  generate_operate_run +n -Llnt 100 yup    ${file_source} > ${test_current}literal-line-100-name-yup-total.expect &&
  generate_operate_run +n -Llant 0 0 yup   ${file_source} > ${test_current}literal-line-0-at-0-name-yup-total.expect &&
  generate_operate_run +n -Llant 1 0 yup   ${file_source} > ${test_current}literal-line-1-at-0-name-yup-total.expect &&
  generate_operate_run +n -Llant 2 0 yup   ${file_source} > ${test_current}literal-line-2-at-0-name-yup-total.expect &&
  generate_operate_run +n -Llant 3 0 yup   ${file_source} > ${test_current}literal-line-3-at-0-name-yup-total.expect &&
  generate_operate_run +n -Llant 4 0 yup   ${file_source} > ${test_current}literal-line-4-at-0-name-yup-total.expect &&
  generate_operate_run +n -Llant 5 0 yup   ${file_source} > ${test_current}literal-line-5-at-0-name-yup-total.expect &&
  generate_operate_run +n -Llant 6 0 yup   ${file_source} > ${test_current}literal-line-6-at-0-name-yup-total.expect &&
  generate_operate_run +n -Llant 7 0 yup   ${file_source} > ${test_current}literal-line-7-at-0-name-yup-total.expect &&
  generate_operate_run +n -Llant 8 0 yup   ${file_source} > ${test_current}literal-line-8-at-0-name-yup-total.expect &&
  generate_operate_run +n -Llant 9 0 yup   ${file_source} > ${test_current}literal-line-9-at-0-name-yup-total.expect &&
  generate_operate_run +n -Llant 10 0 yup  ${file_source} > ${test_current}literal-line-10-at-0-name-yup-total.expect &&
  generate_operate_run +n -Llant 20 0 yup  ${file_source} > ${test_current}literal-line-20-at-0-name-yup-total.expect &&
  generate_operate_run +n -Llant 100 0 yup ${file_source} > ${test_current}literal-line-100-at-0-name-yup-total.expect &&
  generate_operate_run +n -Llant 0 1 yup   ${file_source} > ${test_current}literal-line-0-at-1-name-yup-total.expect &&
  generate_operate_run +n -Llant 1 1 yup   ${file_source} > ${test_current}literal-line-1-at-1-name-yup-total.expect &&
  generate_operate_run +n -Llant 2 1 yup   ${file_source} > ${test_current}literal-line-2-at-1-name-yup-total.expect &&
  generate_operate_run +n -Llant 3 1 yup   ${file_source} > ${test_current}literal-line-3-at-1-name-yup-total.expect &&
  generate_operate_run +n -Llant 4 1 yup   ${file_source} > ${test_current}literal-line-4-at-1-name-yup-total.expect &&
  generate_operate_run +n -Llant 5 1 yup   ${file_source} > ${test_current}literal-line-5-at-1-name-yup-total.expect &&
  generate_operate_run +n -Llant 6 1 yup   ${file_source} > ${test_current}literal-line-6-at-1-name-yup-total.expect &&
  generate_operate_run +n -Llant 7 1 yup   ${file_source} > ${test_current}literal-line-7-at-1-name-yup-total.expect &&
  generate_operate_run +n -Llant 8 1 yup   ${file_source} > ${test_current}literal-line-8-at-1-name-yup-total.expect &&
  generate_operate_run +n -Llant 9 1 yup   ${file_source} > ${test_current}literal-line-9-at-1-name-yup-total.expect &&
  generate_operate_run +n -Llant 10 1 yup  ${file_source} > ${test_current}literal-line-10-at-1-name-yup-total.expect &&
  generate_operate_run +n -Llant 20 1 yup  ${file_source} > ${test_current}literal-line-20-at-1-name-yup-total.expect &&
  generate_operate_run +n -Llant 100 1 yup ${file_source} > ${test_current}literal-line-100-at-1-name-yup-total.expect &&
  generate_operate_run +n -Llant 100 1 yup ${file_source} > ${test_current}literal-line-100-at-1-name-yup-total.expect &&

  #### LITERAL - WHOLE ####
  generate_operate_run +n -Lw         ${file_source} > ${test_current}content.expect &&
  generate_operate_run +n -Lwl 0      ${file_source} > ${test_current}literal-whole-line-0.expect &&
  generate_operate_run +n -Lwl 1      ${file_source} > ${test_current}literal-whole-line-1.expect &&
  generate_operate_run +n -Lwl 2      ${file_source} > ${test_current}literal-whole-line-2.expect &&
  generate_operate_run +n -Lwl 3      ${file_source} > ${test_current}literal-whole-line-3.expect &&
  generate_operate_run +n -Lwl 4      ${file_source} > ${test_current}literal-whole-line-4.expect &&
  generate_operate_run +n -Lwl 5      ${file_source} > ${test_current}literal-whole-line-5.expect &&
  generate_operate_run +n -Lwl 6      ${file_source} > ${test_current}literal-whole-line-6.expect &&
  generate_operate_run +n -Lwl 7      ${file_source} > ${test_current}literal-whole-line-7.expect &&
  generate_operate_run +n -Lwl 8      ${file_source} > ${test_current}literal-whole-line-8.expect &&
  generate_operate_run +n -Lwl 9      ${file_source} > ${test_current}literal-whole-line-9.expect &&
  generate_operate_run +n -Lwl 10     ${file_source} > ${test_current}literal-whole-line-10.expect &&
  generate_operate_run +n -Lwl 20     ${file_source} > ${test_current}literal-whole-line-20.expect &&
  generate_operate_run +n -Lwl 100    ${file_source} > ${test_current}literal-whole-line-100.expect &&
  generate_operate_run +n -Lwla 0 0   ${file_source} > ${test_current}literal-whole-line-0-at-0.expect &&
  generate_operate_run +n -Lwla 1 0   ${file_source} > ${test_current}literal-whole-line-1-at-0.expect &&
  generate_operate_run +n -Lwla 2 0   ${file_source} > ${test_current}literal-whole-line-2-at-0.expect &&
  generate_operate_run +n -Lwla 3 0   ${file_source} > ${test_current}literal-whole-line-3-at-0.expect &&
  generate_operate_run +n -Lwla 4 0   ${file_source} > ${test_current}literal-whole-line-4-at-0.expect &&
  generate_operate_run +n -Lwla 5 0   ${file_source} > ${test_current}literal-whole-line-5-at-0.expect &&
  generate_operate_run +n -Lwla 6 0   ${file_source} > ${test_current}literal-whole-line-6-at-0.expect &&
  generate_operate_run +n -Lwla 7 0   ${file_source} > ${test_current}literal-whole-line-7-at-0.expect &&
  generate_operate_run +n -Lwla 8 0   ${file_source} > ${test_current}literal-whole-line-8-at-0.expect &&
  generate_operate_run +n -Lwla 9 0   ${file_source} > ${test_current}literal-whole-line-9-at-0.expect &&
  generate_operate_run +n -Lwla 10 0  ${file_source} > ${test_current}literal-whole-line-10-at-0.expect &&
  generate_operate_run +n -Lwla 20 0  ${file_source} > ${test_current}literal-whole-line-20-at-0.expect &&
  generate_operate_run +n -Lwla 100 0 ${file_source} > ${test_current}literal-whole-line-100-at-0.expect &&
  generate_operate_run +n -Lwla 0 1   ${file_source} > ${test_current}literal-whole-line-0-at-1.expect &&
  generate_operate_run +n -Lwla 1 1   ${file_source} > ${test_current}literal-whole-line-1-at-1.expect &&
  generate_operate_run +n -Lwla 2 1   ${file_source} > ${test_current}literal-whole-line-2-at-1.expect &&
  generate_operate_run +n -Lwla 3 1   ${file_source} > ${test_current}literal-whole-line-3-at-1.expect &&
  generate_operate_run +n -Lwla 4 1   ${file_source} > ${test_current}literal-whole-line-4-at-1.expect &&
  generate_operate_run +n -Lwla 5 1   ${file_source} > ${test_current}literal-whole-line-5-at-1.expect &&
  generate_operate_run +n -Lwla 6 1   ${file_source} > ${test_current}literal-whole-line-6-at-1.expect &&
  generate_operate_run +n -Lwla 7 1   ${file_source} > ${test_current}literal-whole-line-7-at-1.expect &&
  generate_operate_run +n -Lwla 8 1   ${file_source} > ${test_current}literal-whole-line-8-at-1.expect &&
  generate_operate_run +n -Lwla 9 1   ${file_source} > ${test_current}literal-whole-line-9-at-1.expect &&
  generate_operate_run +n -Lwla 10 1  ${file_source} > ${test_current}literal-whole-line-10-at-1.expect &&
  generate_operate_run +n -Lwla 20 1  ${file_source} > ${test_current}literal-whole-line-20-at-1.expect &&
  generate_operate_run +n -Lwla 100 1 ${file_source} > ${test_current}literal-whole-line-100-at-1.expect &&

  generate_operate_run +n -Lwn a         ${file_source} > ${test_current}literal-whole-name-a.expect &&
  generate_operate_run +n -Lwln 0 a      ${file_source} > ${test_current}literal-whole-line-0-name-a.expect &&
  generate_operate_run +n -Lwln 1 a      ${file_source} > ${test_current}literal-whole-line-1-name-a.expect &&
  generate_operate_run +n -Lwln 2 a      ${file_source} > ${test_current}literal-whole-line-2-name-a.expect &&
  generate_operate_run +n -Lwln 3 a      ${file_source} > ${test_current}literal-whole-line-3-name-a.expect &&
  generate_operate_run +n -Lwln 4 a      ${file_source} > ${test_current}literal-whole-line-4-name-a.expect &&
  generate_operate_run +n -Lwln 5 a      ${file_source} > ${test_current}literal-whole-line-5-name-a.expect &&
  generate_operate_run +n -Lwln 6 a      ${file_source} > ${test_current}literal-whole-line-6-name-a.expect &&
  generate_operate_run +n -Lwln 7 a      ${file_source} > ${test_current}literal-whole-line-7-name-a.expect &&
  generate_operate_run +n -Lwln 8 a      ${file_source} > ${test_current}literal-whole-line-8-name-a.expect &&
  generate_operate_run +n -Lwln 9 a      ${file_source} > ${test_current}literal-whole-line-9-name-a.expect &&
  generate_operate_run +n -Lwln 10 a     ${file_source} > ${test_current}literal-whole-line-10-name-a.expect &&
  generate_operate_run +n -Lwln 20 a     ${file_source} > ${test_current}literal-whole-line-20-name-a.expect &&
  generate_operate_run +n -Lwln 100 a    ${file_source} > ${test_current}literal-whole-line-100-name-a.expect &&
  generate_operate_run +n -Lwlan 0 0 a   ${file_source} > ${test_current}literal-whole-line-0-at-0-name-a.expect &&
  generate_operate_run +n -Lwlan 1 0 a   ${file_source} > ${test_current}literal-whole-line-1-at-0-name-a.expect &&
  generate_operate_run +n -Lwlan 2 0 a   ${file_source} > ${test_current}literal-whole-line-2-at-0-name-a.expect &&
  generate_operate_run +n -Lwlan 3 0 a   ${file_source} > ${test_current}literal-whole-line-3-at-0-name-a.expect &&
  generate_operate_run +n -Lwlan 4 0 a   ${file_source} > ${test_current}literal-whole-line-4-at-0-name-a.expect &&
  generate_operate_run +n -Lwlan 5 0 a   ${file_source} > ${test_current}literal-whole-line-5-at-0-name-a.expect &&
  generate_operate_run +n -Lwlan 6 0 a   ${file_source} > ${test_current}literal-whole-line-6-at-0-name-a.expect &&
  generate_operate_run +n -Lwlan 7 0 a   ${file_source} > ${test_current}literal-whole-line-7-at-0-name-a.expect &&
  generate_operate_run +n -Lwlan 8 0 a   ${file_source} > ${test_current}literal-whole-line-8-at-0-name-a.expect &&
  generate_operate_run +n -Lwlan 9 0 a   ${file_source} > ${test_current}literal-whole-line-9-at-0-name-a.expect &&
  generate_operate_run +n -Lwlan 10 0 a  ${file_source} > ${test_current}literal-whole-line-10-at-0-name-a.expect &&
  generate_operate_run +n -Lwlan 20 0 a  ${file_source} > ${test_current}literal-whole-line-20-at-0-name-a.expect &&
  generate_operate_run +n -Lwlan 100 0 a ${file_source} > ${test_current}literal-whole-line-100-at-0-name-a.expect &&
  generate_operate_run +n -Lwlan 0 1 a   ${file_source} > ${test_current}literal-whole-line-0-at-1-name-a.expect &&
  generate_operate_run +n -Lwlan 1 1 a   ${file_source} > ${test_current}literal-whole-line-1-at-1-name-a.expect &&
  generate_operate_run +n -Lwlan 2 1 a   ${file_source} > ${test_current}literal-whole-line-2-at-1-name-a.expect &&
  generate_operate_run +n -Lwlan 3 1 a   ${file_source} > ${test_current}literal-whole-line-3-at-1-name-a.expect &&
  generate_operate_run +n -Lwlan 4 1 a   ${file_source} > ${test_current}literal-whole-line-4-at-1-name-a.expect &&
  generate_operate_run +n -Lwlan 5 1 a   ${file_source} > ${test_current}literal-whole-line-5-at-1-name-a.expect &&
  generate_operate_run +n -Lwlan 6 1 a   ${file_source} > ${test_current}literal-whole-line-6-at-1-name-a.expect &&
  generate_operate_run +n -Lwlan 7 1 a   ${file_source} > ${test_current}literal-whole-line-7-at-1-name-a.expect &&
  generate_operate_run +n -Lwlan 8 1 a   ${file_source} > ${test_current}literal-whole-line-8-at-1-name-a.expect &&
  generate_operate_run +n -Lwlan 9 1 a   ${file_source} > ${test_current}literal-whole-line-9-at-1-name-a.expect &&
  generate_operate_run +n -Lwlan 10 1 a  ${file_source} > ${test_current}literal-whole-line-10-at-1-name-a.expect &&
  generate_operate_run +n -Lwlan 20 1 a  ${file_source} > ${test_current}literal-whole-line-20-at-1-name-a.expect &&
  generate_operate_run +n -Lwlan 100 1 a ${file_source} > ${test_current}literal-whole-line-100-at-1-name-a.expect &&

  generate_operate_run +n -Lwn b         ${file_source} > ${test_current}literal-whole-name-b.expect &&
  generate_operate_run +n -Lwln 0 b      ${file_source} > ${test_current}literal-whole-line-0-name-b.expect &&
  generate_operate_run +n -Lwln 1 b      ${file_source} > ${test_current}literal-whole-line-1-name-b.expect &&
  generate_operate_run +n -Lwln 2 b      ${file_source} > ${test_current}literal-whole-line-2-name-b.expect &&
  generate_operate_run +n -Lwln 3 b      ${file_source} > ${test_current}literal-whole-line-3-name-b.expect &&
  generate_operate_run +n -Lwln 4 b      ${file_source} > ${test_current}literal-whole-line-4-name-b.expect &&
  generate_operate_run +n -Lwln 5 b      ${file_source} > ${test_current}literal-whole-line-5-name-b.expect &&
  generate_operate_run +n -Lwln 6 b      ${file_source} > ${test_current}literal-whole-line-6-name-b.expect &&
  generate_operate_run +n -Lwln 7 b      ${file_source} > ${test_current}literal-whole-line-7-name-b.expect &&
  generate_operate_run +n -Lwln 8 b      ${file_source} > ${test_current}literal-whole-line-8-name-b.expect &&
  generate_operate_run +n -Lwln 9 b      ${file_source} > ${test_current}literal-whole-line-9-name-b.expect &&
  generate_operate_run +n -Lwln 10 b     ${file_source} > ${test_current}literal-whole-line-10-name-b.expect &&
  generate_operate_run +n -Lwln 20 b     ${file_source} > ${test_current}literal-whole-line-20-name-b.expect &&
  generate_operate_run +n -Lwln 100 b    ${file_source} > ${test_current}literal-whole-line-100-name-b.expect &&
  generate_operate_run +n -Lwlan 0 0 b   ${file_source} > ${test_current}literal-whole-line-0-at-0-name-b.expect &&
  generate_operate_run +n -Lwlan 1 0 b   ${file_source} > ${test_current}literal-whole-line-1-at-0-name-b.expect &&
  generate_operate_run +n -Lwlan 2 0 b   ${file_source} > ${test_current}literal-whole-line-2-at-0-name-b.expect &&
  generate_operate_run +n -Lwlan 3 0 b   ${file_source} > ${test_current}literal-whole-line-3-at-0-name-b.expect &&
  generate_operate_run +n -Lwlan 4 0 b   ${file_source} > ${test_current}literal-whole-line-4-at-0-name-b.expect &&
  generate_operate_run +n -Lwlan 5 0 b   ${file_source} > ${test_current}literal-whole-line-5-at-0-name-b.expect &&
  generate_operate_run +n -Lwlan 6 0 b   ${file_source} > ${test_current}literal-whole-line-6-at-0-name-b.expect &&
  generate_operate_run +n -Lwlan 7 0 b   ${file_source} > ${test_current}literal-whole-line-7-at-0-name-b.expect &&
  generate_operate_run +n -Lwlan 8 0 b   ${file_source} > ${test_current}literal-whole-line-8-at-0-name-b.expect &&
  generate_operate_run +n -Lwlan 9 0 b   ${file_source} > ${test_current}literal-whole-line-9-at-0-name-b.expect &&
  generate_operate_run +n -Lwlan 10 0 b  ${file_source} > ${test_current}literal-whole-line-10-at-0-name-b.expect &&
  generate_operate_run +n -Lwlan 20 0 b  ${file_source} > ${test_current}literal-whole-line-20-at-0-name-b.expect &&
  generate_operate_run +n -Lwlan 100 0 b ${file_source} > ${test_current}literal-whole-line-100-at-0-name-b.expect &&
  generate_operate_run +n -Lwlan 0 1 b   ${file_source} > ${test_current}literal-whole-line-0-at-1-name-b.expect &&
  generate_operate_run +n -Lwlan 1 1 b   ${file_source} > ${test_current}literal-whole-line-1-at-1-name-b.expect &&
  generate_operate_run +n -Lwlan 2 1 b   ${file_source} > ${test_current}literal-whole-line-2-at-1-name-b.expect &&
  generate_operate_run +n -Lwlan 3 1 b   ${file_source} > ${test_current}literal-whole-line-3-at-1-name-b.expect &&
  generate_operate_run +n -Lwlan 4 1 b   ${file_source} > ${test_current}literal-whole-line-4-at-1-name-b.expect &&
  generate_operate_run +n -Lwlan 5 1 b   ${file_source} > ${test_current}literal-whole-line-5-at-1-name-b.expect &&
  generate_operate_run +n -Lwlan 6 1 b   ${file_source} > ${test_current}literal-whole-line-6-at-1-name-b.expect &&
  generate_operate_run +n -Lwlan 7 1 b   ${file_source} > ${test_current}literal-whole-line-7-at-1-name-b.expect &&
  generate_operate_run +n -Lwlan 8 1 b   ${file_source} > ${test_current}literal-whole-line-8-at-1-name-b.expect &&
  generate_operate_run +n -Lwlan 9 1 b   ${file_source} > ${test_current}literal-whole-line-9-at-1-name-b.expect &&
  generate_operate_run +n -Lwlan 10 1 b  ${file_source} > ${test_current}literal-whole-line-10-at-1-name-b.expect &&
  generate_operate_run +n -Lwlan 20 1 b  ${file_source} > ${test_current}literal-whole-line-20-at-1-name-b.expect &&
  generate_operate_run +n -Lwlan 100 1 b ${file_source} > ${test_current}literal-whole-line-100-at-1-name-b.expect &&

  generate_operate_run +n -Lwn is         ${file_source} > ${test_current}literal-whole-name-is.expect &&
  generate_operate_run +n -Lwln 0 is      ${file_source} > ${test_current}literal-whole-line-0-name-is.expect &&
  generate_operate_run +n -Lwln 1 is      ${file_source} > ${test_current}literal-whole-line-1-name-is.expect &&
  generate_operate_run +n -Lwln 2 is      ${file_source} > ${test_current}literal-whole-line-2-name-is.expect &&
  generate_operate_run +n -Lwln 3 is      ${file_source} > ${test_current}literal-whole-line-3-name-is.expect &&
  generate_operate_run +n -Lwln 4 is      ${file_source} > ${test_current}literal-whole-line-4-name-is.expect &&
  generate_operate_run +n -Lwln 5 is      ${file_source} > ${test_current}literal-whole-line-5-name-is.expect &&
  generate_operate_run +n -Lwln 6 is      ${file_source} > ${test_current}literal-whole-line-6-name-is.expect &&
  generate_operate_run +n -Lwln 7 is      ${file_source} > ${test_current}literal-whole-line-7-name-is.expect &&
  generate_operate_run +n -Lwln 8 is      ${file_source} > ${test_current}literal-whole-line-8-name-is.expect &&
  generate_operate_run +n -Lwln 9 is      ${file_source} > ${test_current}literal-whole-line-9-name-is.expect &&
  generate_operate_run +n -Lwln 10 is     ${file_source} > ${test_current}literal-whole-line-10-name-is.expect &&
  generate_operate_run +n -Lwln 20 is     ${file_source} > ${test_current}literal-whole-line-20-name-is.expect &&
  generate_operate_run +n -Lwln 100 is    ${file_source} > ${test_current}literal-whole-line-100-name-is.expect &&
  generate_operate_run +n -Lwlan 0 0 is   ${file_source} > ${test_current}literal-whole-line-0-at-0-name-is.expect &&
  generate_operate_run +n -Lwlan 1 0 is   ${file_source} > ${test_current}literal-whole-line-1-at-0-name-is.expect &&
  generate_operate_run +n -Lwlan 2 0 is   ${file_source} > ${test_current}literal-whole-line-2-at-0-name-is.expect &&
  generate_operate_run +n -Lwlan 3 0 is   ${file_source} > ${test_current}literal-whole-line-3-at-0-name-is.expect &&
  generate_operate_run +n -Lwlan 4 0 is   ${file_source} > ${test_current}literal-whole-line-4-at-0-name-is.expect &&
  generate_operate_run +n -Lwlan 5 0 is   ${file_source} > ${test_current}literal-whole-line-5-at-0-name-is.expect &&
  generate_operate_run +n -Lwlan 6 0 is   ${file_source} > ${test_current}literal-whole-line-6-at-0-name-is.expect &&
  generate_operate_run +n -Lwlan 7 0 is   ${file_source} > ${test_current}literal-whole-line-7-at-0-name-is.expect &&
  generate_operate_run +n -Lwlan 8 0 is   ${file_source} > ${test_current}literal-whole-line-8-at-0-name-is.expect &&
  generate_operate_run +n -Lwlan 9 0 is   ${file_source} > ${test_current}literal-whole-line-9-at-0-name-is.expect &&
  generate_operate_run +n -Lwlan 10 0 is  ${file_source} > ${test_current}literal-whole-line-10-at-0-name-is.expect &&
  generate_operate_run +n -Lwlan 20 0 is  ${file_source} > ${test_current}literal-whole-line-20-at-0-name-is.expect &&
  generate_operate_run +n -Lwlan 100 0 is ${file_source} > ${test_current}literal-whole-line-100-at-0-name-is.expect &&
  generate_operate_run +n -Lwlan 0 1 is   ${file_source} > ${test_current}literal-whole-line-0-at-1-name-is.expect &&
  generate_operate_run +n -Lwlan 1 1 is   ${file_source} > ${test_current}literal-whole-line-1-at-1-name-is.expect &&
  generate_operate_run +n -Lwlan 2 1 is   ${file_source} > ${test_current}literal-whole-line-2-at-1-name-is.expect &&
  generate_operate_run +n -Lwlan 3 1 is   ${file_source} > ${test_current}literal-whole-line-3-at-1-name-is.expect &&
  generate_operate_run +n -Lwlan 4 1 is   ${file_source} > ${test_current}literal-whole-line-4-at-1-name-is.expect &&
  generate_operate_run +n -Lwlan 5 1 is   ${file_source} > ${test_current}literal-whole-line-5-at-1-name-is.expect &&
  generate_operate_run +n -Lwlan 6 1 is   ${file_source} > ${test_current}literal-whole-line-6-at-1-name-is.expect &&
  generate_operate_run +n -Lwlan 7 1 is   ${file_source} > ${test_current}literal-whole-line-7-at-1-name-is.expect &&
  generate_operate_run +n -Lwlan 8 1 is   ${file_source} > ${test_current}literal-whole-line-8-at-1-name-is.expect &&
  generate_operate_run +n -Lwlan 9 1 is   ${file_source} > ${test_current}literal-whole-line-9-at-1-name-is.expect &&
  generate_operate_run +n -Lwlan 10 1 is  ${file_source} > ${test_current}literal-whole-line-10-at-1-name-is.expect &&
  generate_operate_run +n -Lwlan 20 1 is  ${file_source} > ${test_current}literal-whole-line-20-at-1-name-is.expect &&
  generate_operate_run +n -Lwlan 100 1 is ${file_source} > ${test_current}literal-whole-line-100-at-1-name-is.expect &&

  generate_operate_run +n -Lwn yup         ${file_source} > ${test_current}literal-whole-name-yup.expect &&
  generate_operate_run +n -Lwln 0 yup      ${file_source} > ${test_current}literal-whole-line-0-name-yup.expect &&
  generate_operate_run +n -Lwln 1 yup      ${file_source} > ${test_current}literal-whole-line-1-name-yup.expect &&
  generate_operate_run +n -Lwln 2 yup      ${file_source} > ${test_current}literal-whole-line-2-name-yup.expect &&
  generate_operate_run +n -Lwln 3 yup      ${file_source} > ${test_current}literal-whole-line-3-name-yup.expect &&
  generate_operate_run +n -Lwln 4 yup      ${file_source} > ${test_current}literal-whole-line-4-name-yup.expect &&
  generate_operate_run +n -Lwln 5 yup      ${file_source} > ${test_current}literal-whole-line-5-name-yup.expect &&
  generate_operate_run +n -Lwln 6 yup      ${file_source} > ${test_current}literal-whole-line-6-name-yup.expect &&
  generate_operate_run +n -Lwln 7 yup      ${file_source} > ${test_current}literal-whole-line-7-name-yup.expect &&
  generate_operate_run +n -Lwln 8 yup      ${file_source} > ${test_current}literal-whole-line-8-name-yup.expect &&
  generate_operate_run +n -Lwln 9 yup      ${file_source} > ${test_current}literal-whole-line-9-name-yup.expect &&
  generate_operate_run +n -Lwln 10 yup     ${file_source} > ${test_current}literal-whole-line-10-name-yup.expect &&
  generate_operate_run +n -Lwln 20 yup     ${file_source} > ${test_current}literal-whole-line-20-name-yup.expect &&
  generate_operate_run +n -Lwln 100 yup    ${file_source} > ${test_current}literal-whole-line-100-name-yup.expect &&
  generate_operate_run +n -Lwlan 0 0 yup   ${file_source} > ${test_current}literal-whole-line-0-at-0-name-yup.expect &&
  generate_operate_run +n -Lwlan 1 0 yup   ${file_source} > ${test_current}literal-whole-line-1-at-0-name-yup.expect &&
  generate_operate_run +n -Lwlan 2 0 yup   ${file_source} > ${test_current}literal-whole-line-2-at-0-name-yup.expect &&
  generate_operate_run +n -Lwlan 3 0 yup   ${file_source} > ${test_current}literal-whole-line-3-at-0-name-yup.expect &&
  generate_operate_run +n -Lwlan 4 0 yup   ${file_source} > ${test_current}literal-whole-line-4-at-0-name-yup.expect &&
  generate_operate_run +n -Lwlan 5 0 yup   ${file_source} > ${test_current}literal-whole-line-5-at-0-name-yup.expect &&
  generate_operate_run +n -Lwlan 6 0 yup   ${file_source} > ${test_current}literal-whole-line-6-at-0-name-yup.expect &&
  generate_operate_run +n -Lwlan 7 0 yup   ${file_source} > ${test_current}literal-whole-line-7-at-0-name-yup.expect &&
  generate_operate_run +n -Lwlan 8 0 yup   ${file_source} > ${test_current}literal-whole-line-8-at-0-name-yup.expect &&
  generate_operate_run +n -Lwlan 9 0 yup   ${file_source} > ${test_current}literal-whole-line-9-at-0-name-yup.expect &&
  generate_operate_run +n -Lwlan 10 0 yup  ${file_source} > ${test_current}literal-whole-line-10-at-0-name-yup.expect &&
  generate_operate_run +n -Lwlan 20 0 yup  ${file_source} > ${test_current}literal-whole-line-20-at-0-name-yup.expect &&
  generate_operate_run +n -Lwlan 100 0 yup ${file_source} > ${test_current}literal-whole-line-100-at-0-name-yup.expect &&
  generate_operate_run +n -Lwlan 0 1 yup   ${file_source} > ${test_current}literal-whole-line-0-at-1-name-yup.expect &&
  generate_operate_run +n -Lwlan 1 1 yup   ${file_source} > ${test_current}literal-whole-line-1-at-1-name-yup.expect &&
  generate_operate_run +n -Lwlan 2 1 yup   ${file_source} > ${test_current}literal-whole-line-2-at-1-name-yup.expect &&
  generate_operate_run +n -Lwlan 3 1 yup   ${file_source} > ${test_current}literal-whole-line-3-at-1-name-yup.expect &&
  generate_operate_run +n -Lwlan 4 1 yup   ${file_source} > ${test_current}literal-whole-line-4-at-1-name-yup.expect &&
  generate_operate_run +n -Lwlan 5 1 yup   ${file_source} > ${test_current}literal-whole-line-5-at-1-name-yup.expect &&
  generate_operate_run +n -Lwlan 6 1 yup   ${file_source} > ${test_current}literal-whole-line-6-at-1-name-yup.expect &&
  generate_operate_run +n -Lwlan 7 1 yup   ${file_source} > ${test_current}literal-whole-line-7-at-1-name-yup.expect &&
  generate_operate_run +n -Lwlan 8 1 yup   ${file_source} > ${test_current}literal-whole-line-8-at-1-name-yup.expect &&
  generate_operate_run +n -Lwlan 9 1 yup   ${file_source} > ${test_current}literal-whole-line-9-at-1-name-yup.expect &&
  generate_operate_run +n -Lwlan 10 1 yup  ${file_source} > ${test_current}literal-whole-line-10-at-1-name-yup.expect &&
  generate_operate_run +n -Lwlan 20 1 yup  ${file_source} > ${test_current}literal-whole-line-20-at-1-name-yup.expect &&

  #### LITERAL - WHOLE - TOTAL ####
  generate_operate_run +n -Lwt         ${file_source} > ${test_current}literal-whole-total.expect &&
  generate_operate_run +n -Lwlt 0      ${file_source} > ${test_current}literal-whole-line-0-total.expect &&
  generate_operate_run +n -Lwlt 1      ${file_source} > ${test_current}literal-whole-line-1-total.expect &&
  generate_operate_run +n -Lwlt 2      ${file_source} > ${test_current}literal-whole-line-2-total.expect &&
  generate_operate_run +n -Lwlt 3      ${file_source} > ${test_current}literal-whole-line-3-total.expect &&
  generate_operate_run +n -Lwlt 4      ${file_source} > ${test_current}literal-whole-line-4-total.expect &&
  generate_operate_run +n -Lwlt 5      ${file_source} > ${test_current}literal-whole-line-5-total.expect &&
  generate_operate_run +n -Lwlt 6      ${file_source} > ${test_current}literal-whole-line-6-total.expect &&
  generate_operate_run +n -Lwlt 7      ${file_source} > ${test_current}literal-whole-line-7-total.expect &&
  generate_operate_run +n -Lwlt 8      ${file_source} > ${test_current}literal-whole-line-8-total.expect &&
  generate_operate_run +n -Lwlt 9      ${file_source} > ${test_current}literal-whole-line-9-total.expect &&
  generate_operate_run +n -Lwlt 10     ${file_source} > ${test_current}literal-whole-line-10-total.expect &&
  generate_operate_run +n -Lwlt 20     ${file_source} > ${test_current}literal-whole-line-20-total.expect &&
  generate_operate_run +n -Lwlt 100    ${file_source} > ${test_current}literal-whole-line-100-total.expect &&
  generate_operate_run +n -Lwlat 0 0   ${file_source} > ${test_current}literal-whole-line-0-at-0-total.expect &&
  generate_operate_run +n -Lwlat 1 0   ${file_source} > ${test_current}literal-whole-line-1-at-0-total.expect &&
  generate_operate_run +n -Lwlat 2 0   ${file_source} > ${test_current}literal-whole-line-2-at-0-total.expect &&
  generate_operate_run +n -Lwlat 3 0   ${file_source} > ${test_current}literal-whole-line-3-at-0-total.expect &&
  generate_operate_run +n -Lwlat 4 0   ${file_source} > ${test_current}literal-whole-line-4-at-0-total.expect &&
  generate_operate_run +n -Lwlat 5 0   ${file_source} > ${test_current}literal-whole-line-5-at-0-total.expect &&
  generate_operate_run +n -Lwlat 6 0   ${file_source} > ${test_current}literal-whole-line-6-at-0-total.expect &&
  generate_operate_run +n -Lwlat 7 0   ${file_source} > ${test_current}literal-whole-line-7-at-0-total.expect &&
  generate_operate_run +n -Lwlat 8 0   ${file_source} > ${test_current}literal-whole-line-8-at-0-total.expect &&
  generate_operate_run +n -Lwlat 9 0   ${file_source} > ${test_current}literal-whole-line-9-at-0-total.expect &&
  generate_operate_run +n -Lwlat 10 0  ${file_source} > ${test_current}literal-whole-line-10-at-0-total.expect &&
  generate_operate_run +n -Lwlat 20 0  ${file_source} > ${test_current}literal-whole-line-20-at-0-total.expect &&
  generate_operate_run +n -Lwlat 100 0 ${file_source} > ${test_current}literal-whole-line-100-at-0-total.expect &&
  generate_operate_run +n -Lwlat 0 1   ${file_source} > ${test_current}literal-whole-line-0-at-1-total.expect &&
  generate_operate_run +n -Lwlat 1 1   ${file_source} > ${test_current}literal-whole-line-1-at-1-total.expect &&
  generate_operate_run +n -Lwlat 2 1   ${file_source} > ${test_current}literal-whole-line-2-at-1-total.expect &&
  generate_operate_run +n -Lwlat 3 1   ${file_source} > ${test_current}literal-whole-line-3-at-1-total.expect &&
  generate_operate_run +n -Lwlat 4 1   ${file_source} > ${test_current}literal-whole-line-4-at-1-total.expect &&
  generate_operate_run +n -Lwlat 5 1   ${file_source} > ${test_current}literal-whole-line-5-at-1-total.expect &&
  generate_operate_run +n -Lwlat 6 1   ${file_source} > ${test_current}literal-whole-line-6-at-1-total.expect &&
  generate_operate_run +n -Lwlat 7 1   ${file_source} > ${test_current}literal-whole-line-7-at-1-total.expect &&
  generate_operate_run +n -Lwlat 8 1   ${file_source} > ${test_current}literal-whole-line-8-at-1-total.expect &&
  generate_operate_run +n -Lwlat 9 1   ${file_source} > ${test_current}literal-whole-line-9-at-1-total.expect &&
  generate_operate_run +n -Lwlat 10 1  ${file_source} > ${test_current}literal-whole-line-10-at-1-total.expect &&
  generate_operate_run +n -Lwlat 20 1  ${file_source} > ${test_current}literal-whole-line-20-at-1-total.expect &&
  generate_operate_run +n -Lwlat 100 1 ${file_source} > ${test_current}literal-whole-line-100-at-1-total.expect &&

  generate_operate_run +n -Lwnt a         ${file_source} > ${test_current}literal-whole-name-a-total.expect &&
  generate_operate_run +n -Lwlnt 0 a      ${file_source} > ${test_current}literal-whole-line-0-name-a-total.expect &&
  generate_operate_run +n -Lwlnt 1 a      ${file_source} > ${test_current}literal-whole-line-1-name-a-total.expect &&
  generate_operate_run +n -Lwlnt 2 a      ${file_source} > ${test_current}literal-whole-line-2-name-a-total.expect &&
  generate_operate_run +n -Lwlnt 3 a      ${file_source} > ${test_current}literal-whole-line-3-name-a-total.expect &&
  generate_operate_run +n -Lwlnt 4 a      ${file_source} > ${test_current}literal-whole-line-4-name-a-total.expect &&
  generate_operate_run +n -Lwlnt 5 a      ${file_source} > ${test_current}literal-whole-line-5-name-a-total.expect &&
  generate_operate_run +n -Lwlnt 6 a      ${file_source} > ${test_current}literal-whole-line-6-name-a-total.expect &&
  generate_operate_run +n -Lwlnt 7 a      ${file_source} > ${test_current}literal-whole-line-7-name-a-total.expect &&
  generate_operate_run +n -Lwlnt 8 a      ${file_source} > ${test_current}literal-whole-line-8-name-a-total.expect &&
  generate_operate_run +n -Lwlnt 9 a      ${file_source} > ${test_current}literal-whole-line-9-name-a-total.expect &&
  generate_operate_run +n -Lwlnt 10 a     ${file_source} > ${test_current}literal-whole-line-10-name-a-total.expect &&
  generate_operate_run +n -Lwlnt 20 a     ${file_source} > ${test_current}literal-whole-line-20-name-a-total.expect &&
  generate_operate_run +n -Lwlnt 100 a    ${file_source} > ${test_current}literal-whole-line-100-name-a-total.expect &&
  generate_operate_run +n -Lwlant 0 0 a   ${file_source} > ${test_current}literal-whole-line-0-at-0-name-a-total.expect &&
  generate_operate_run +n -Lwlant 1 0 a   ${file_source} > ${test_current}literal-whole-line-1-at-0-name-a-total.expect &&
  generate_operate_run +n -Lwlant 2 0 a   ${file_source} > ${test_current}literal-whole-line-2-at-0-name-a-total.expect &&
  generate_operate_run +n -Lwlant 3 0 a   ${file_source} > ${test_current}literal-whole-line-3-at-0-name-a-total.expect &&
  generate_operate_run +n -Lwlant 4 0 a   ${file_source} > ${test_current}literal-whole-line-4-at-0-name-a-total.expect &&
  generate_operate_run +n -Lwlant 5 0 a   ${file_source} > ${test_current}literal-whole-line-5-at-0-name-a-total.expect &&
  generate_operate_run +n -Lwlant 6 0 a   ${file_source} > ${test_current}literal-whole-line-6-at-0-name-a-total.expect &&
  generate_operate_run +n -Lwlant 7 0 a   ${file_source} > ${test_current}literal-whole-line-7-at-0-name-a-total.expect &&
  generate_operate_run +n -Lwlant 8 0 a   ${file_source} > ${test_current}literal-whole-line-8-at-0-name-a-total.expect &&
  generate_operate_run +n -Lwlant 9 0 a   ${file_source} > ${test_current}literal-whole-line-9-at-0-name-a-total.expect &&
  generate_operate_run +n -Lwlant 10 0 a  ${file_source} > ${test_current}literal-whole-line-10-at-0-name-a-total.expect &&
  generate_operate_run +n -Lwlant 20 0 a  ${file_source} > ${test_current}literal-whole-line-20-at-0-name-a-total.expect &&
  generate_operate_run +n -Lwlant 100 0 a ${file_source} > ${test_current}literal-whole-line-100-at-0-name-a-total.expect &&
  generate_operate_run +n -Lwlant 0 1 a   ${file_source} > ${test_current}literal-whole-line-0-at-1-name-a-total.expect &&
  generate_operate_run +n -Lwlant 1 1 a   ${file_source} > ${test_current}literal-whole-line-1-at-1-name-a-total.expect &&
  generate_operate_run +n -Lwlant 2 1 a   ${file_source} > ${test_current}literal-whole-line-2-at-1-name-a-total.expect &&
  generate_operate_run +n -Lwlant 3 1 a   ${file_source} > ${test_current}literal-whole-line-3-at-1-name-a-total.expect &&
  generate_operate_run +n -Lwlant 4 1 a   ${file_source} > ${test_current}literal-whole-line-4-at-1-name-a-total.expect &&
  generate_operate_run +n -Lwlant 5 1 a   ${file_source} > ${test_current}literal-whole-line-5-at-1-name-a-total.expect &&
  generate_operate_run +n -Lwlant 6 1 a   ${file_source} > ${test_current}literal-whole-line-6-at-1-name-a-total.expect &&
  generate_operate_run +n -Lwlant 7 1 a   ${file_source} > ${test_current}literal-whole-line-7-at-1-name-a-total.expect &&
  generate_operate_run +n -Lwlant 8 1 a   ${file_source} > ${test_current}literal-whole-line-8-at-1-name-a-total.expect &&
  generate_operate_run +n -Lwlant 9 1 a   ${file_source} > ${test_current}literal-whole-line-9-at-1-name-a-total.expect &&
  generate_operate_run +n -Lwlant 10 1 a  ${file_source} > ${test_current}literal-whole-line-10-at-1-name-a-total.expect &&
  generate_operate_run +n -Lwlant 20 1 a  ${file_source} > ${test_current}literal-whole-line-20-at-1-name-a-total.expect &&
  generate_operate_run +n -Lwlant 100 1 a ${file_source} > ${test_current}literal-whole-line-100-at-1-name-a-total.expect &&

  generate_operate_run +n -Lwnt b         ${file_source} > ${test_current}literal-whole-name-b-total.expect &&
  generate_operate_run +n -Lwlnt 0 b      ${file_source} > ${test_current}literal-whole-line-0-name-b-total.expect &&
  generate_operate_run +n -Lwlnt 1 b      ${file_source} > ${test_current}literal-whole-line-1-name-b-total.expect &&
  generate_operate_run +n -Lwlnt 2 b      ${file_source} > ${test_current}literal-whole-line-2-name-b-total.expect &&
  generate_operate_run +n -Lwlnt 3 b      ${file_source} > ${test_current}literal-whole-line-3-name-b-total.expect &&
  generate_operate_run +n -Lwlnt 4 b      ${file_source} > ${test_current}literal-whole-line-4-name-b-total.expect &&
  generate_operate_run +n -Lwlnt 5 b      ${file_source} > ${test_current}literal-whole-line-5-name-b-total.expect &&
  generate_operate_run +n -Lwlnt 6 b      ${file_source} > ${test_current}literal-whole-line-6-name-b-total.expect &&
  generate_operate_run +n -Lwlnt 7 b      ${file_source} > ${test_current}literal-whole-line-7-name-b-total.expect &&
  generate_operate_run +n -Lwlnt 8 b      ${file_source} > ${test_current}literal-whole-line-8-name-b-total.expect &&
  generate_operate_run +n -Lwlnt 9 b      ${file_source} > ${test_current}literal-whole-line-9-name-b-total.expect &&
  generate_operate_run +n -Lwlnt 10 b     ${file_source} > ${test_current}literal-whole-line-10-name-b-total.expect &&
  generate_operate_run +n -Lwlnt 20 b     ${file_source} > ${test_current}literal-whole-line-20-name-b-total.expect &&
  generate_operate_run +n -Lwlnt 100 b    ${file_source} > ${test_current}literal-whole-line-100-name-b-total.expect &&
  generate_operate_run +n -Lwlant 0 0 b   ${file_source} > ${test_current}literal-whole-line-0-at-0-name-b-total.expect &&
  generate_operate_run +n -Lwlant 1 0 b   ${file_source} > ${test_current}literal-whole-line-1-at-0-name-b-total.expect &&
  generate_operate_run +n -Lwlant 2 0 b   ${file_source} > ${test_current}literal-whole-line-2-at-0-name-b-total.expect &&
  generate_operate_run +n -Lwlant 3 0 b   ${file_source} > ${test_current}literal-whole-line-3-at-0-name-b-total.expect &&
  generate_operate_run +n -Lwlant 4 0 b   ${file_source} > ${test_current}literal-whole-line-4-at-0-name-b-total.expect &&
  generate_operate_run +n -Lwlant 5 0 b   ${file_source} > ${test_current}literal-whole-line-5-at-0-name-b-total.expect &&
  generate_operate_run +n -Lwlant 6 0 b   ${file_source} > ${test_current}literal-whole-line-6-at-0-name-b-total.expect &&
  generate_operate_run +n -Lwlant 7 0 b   ${file_source} > ${test_current}literal-whole-line-7-at-0-name-b-total.expect &&
  generate_operate_run +n -Lwlant 8 0 b   ${file_source} > ${test_current}literal-whole-line-8-at-0-name-b-total.expect &&
  generate_operate_run +n -Lwlant 9 0 b   ${file_source} > ${test_current}literal-whole-line-9-at-0-name-b-total.expect &&
  generate_operate_run +n -Lwlant 10 0 b  ${file_source} > ${test_current}literal-whole-line-10-at-0-name-b-total.expect &&
  generate_operate_run +n -Lwlant 20 0 b  ${file_source} > ${test_current}literal-whole-line-20-at-0-name-b-total.expect &&
  generate_operate_run +n -Lwlant 100 0 b ${file_source} > ${test_current}literal-whole-line-100-at-0-name-b-total.expect &&
  generate_operate_run +n -Lwlant 0 1 b   ${file_source} > ${test_current}literal-whole-line-0-at-1-name-b-total.expect &&
  generate_operate_run +n -Lwlant 1 1 b   ${file_source} > ${test_current}literal-whole-line-1-at-1-name-b-total.expect &&
  generate_operate_run +n -Lwlant 2 1 b   ${file_source} > ${test_current}literal-whole-line-2-at-1-name-b-total.expect &&
  generate_operate_run +n -Lwlant 3 1 b   ${file_source} > ${test_current}literal-whole-line-3-at-1-name-b-total.expect &&
  generate_operate_run +n -Lwlant 4 1 b   ${file_source} > ${test_current}literal-whole-line-4-at-1-name-b-total.expect &&
  generate_operate_run +n -Lwlant 5 1 b   ${file_source} > ${test_current}literal-whole-line-5-at-1-name-b-total.expect &&
  generate_operate_run +n -Lwlant 6 1 b   ${file_source} > ${test_current}literal-whole-line-6-at-1-name-b-total.expect &&
  generate_operate_run +n -Lwlant 7 1 b   ${file_source} > ${test_current}literal-whole-line-7-at-1-name-b-total.expect &&
  generate_operate_run +n -Lwlant 8 1 b   ${file_source} > ${test_current}literal-whole-line-8-at-1-name-b-total.expect &&
  generate_operate_run +n -Lwlant 9 1 b   ${file_source} > ${test_current}literal-whole-line-9-at-1-name-b-total.expect &&
  generate_operate_run +n -Lwlant 10 1 b  ${file_source} > ${test_current}literal-whole-line-10-at-1-name-b-total.expect &&
  generate_operate_run +n -Lwlant 20 1 b  ${file_source} > ${test_current}literal-whole-line-20-at-1-name-b-total.expect &&
  generate_operate_run +n -Lwlant 100 1 b ${file_source} > ${test_current}literal-whole-line-100-at-1-name-b-total.expect &&

  generate_operate_run +n -Lwnt is         ${file_source} > ${test_current}literal-whole-name-is-total.expect &&
  generate_operate_run +n -Lwlnt 0 is      ${file_source} > ${test_current}literal-whole-line-0-name-is-total.expect &&
  generate_operate_run +n -Lwlnt 1 is      ${file_source} > ${test_current}literal-whole-line-1-name-is-total.expect &&
  generate_operate_run +n -Lwlnt 2 is      ${file_source} > ${test_current}literal-whole-line-2-name-is-total.expect &&
  generate_operate_run +n -Lwlnt 3 is      ${file_source} > ${test_current}literal-whole-line-3-name-is-total.expect &&
  generate_operate_run +n -Lwlnt 4 is      ${file_source} > ${test_current}literal-whole-line-4-name-is-total.expect &&
  generate_operate_run +n -Lwlnt 5 is      ${file_source} > ${test_current}literal-whole-line-5-name-is-total.expect &&
  generate_operate_run +n -Lwlnt 6 is      ${file_source} > ${test_current}literal-whole-line-6-name-is-total.expect &&
  generate_operate_run +n -Lwlnt 7 is      ${file_source} > ${test_current}literal-whole-line-7-name-is-total.expect &&
  generate_operate_run +n -Lwlnt 8 is      ${file_source} > ${test_current}literal-whole-line-8-name-is-total.expect &&
  generate_operate_run +n -Lwlnt 9 is      ${file_source} > ${test_current}literal-whole-line-9-name-is-total.expect &&
  generate_operate_run +n -Lwlnt 10 is     ${file_source} > ${test_current}literal-whole-line-10-name-is-total.expect &&
  generate_operate_run +n -Lwlnt 20 is     ${file_source} > ${test_current}literal-whole-line-20-name-is-total.expect &&
  generate_operate_run +n -Lwlnt 100 is    ${file_source} > ${test_current}literal-whole-line-100-name-is-total.expect &&
  generate_operate_run +n -Lwlant 0 0 is   ${file_source} > ${test_current}literal-whole-line-0-at-0-name-is-total.expect &&
  generate_operate_run +n -Lwlant 1 0 is   ${file_source} > ${test_current}literal-whole-line-1-at-0-name-is-total.expect &&
  generate_operate_run +n -Lwlant 2 0 is   ${file_source} > ${test_current}literal-whole-line-2-at-0-name-is-total.expect &&
  generate_operate_run +n -Lwlant 3 0 is   ${file_source} > ${test_current}literal-whole-line-3-at-0-name-is-total.expect &&
  generate_operate_run +n -Lwlant 4 0 is   ${file_source} > ${test_current}literal-whole-line-4-at-0-name-is-total.expect &&
  generate_operate_run +n -Lwlant 5 0 is   ${file_source} > ${test_current}literal-whole-line-5-at-0-name-is-total.expect &&
  generate_operate_run +n -Lwlant 6 0 is   ${file_source} > ${test_current}literal-whole-line-6-at-0-name-is-total.expect &&
  generate_operate_run +n -Lwlant 7 0 is   ${file_source} > ${test_current}literal-whole-line-7-at-0-name-is-total.expect &&
  generate_operate_run +n -Lwlant 8 0 is   ${file_source} > ${test_current}literal-whole-line-8-at-0-name-is-total.expect &&
  generate_operate_run +n -Lwlant 9 0 is   ${file_source} > ${test_current}literal-whole-line-9-at-0-name-is-total.expect &&
  generate_operate_run +n -Lwlant 10 0 is  ${file_source} > ${test_current}literal-whole-line-10-at-0-name-is-total.expect &&
  generate_operate_run +n -Lwlant 20 0 is  ${file_source} > ${test_current}literal-whole-line-20-at-0-name-is-total.expect &&
  generate_operate_run +n -Lwlant 100 0 is ${file_source} > ${test_current}literal-whole-line-100-at-0-name-is-total.expect &&
  generate_operate_run +n -Lwlant 0 1 is   ${file_source} > ${test_current}literal-whole-line-0-at-1-name-is-total.expect &&
  generate_operate_run +n -Lwlant 1 1 is   ${file_source} > ${test_current}literal-whole-line-1-at-1-name-is-total.expect &&
  generate_operate_run +n -Lwlant 2 1 is   ${file_source} > ${test_current}literal-whole-line-2-at-1-name-is-total.expect &&
  generate_operate_run +n -Lwlant 3 1 is   ${file_source} > ${test_current}literal-whole-line-3-at-1-name-is-total.expect &&
  generate_operate_run +n -Lwlant 4 1 is   ${file_source} > ${test_current}literal-whole-line-4-at-1-name-is-total.expect &&
  generate_operate_run +n -Lwlant 5 1 is   ${file_source} > ${test_current}literal-whole-line-5-at-1-name-is-total.expect &&
  generate_operate_run +n -Lwlant 6 1 is   ${file_source} > ${test_current}literal-whole-line-6-at-1-name-is-total.expect &&
  generate_operate_run +n -Lwlant 7 1 is   ${file_source} > ${test_current}literal-whole-line-7-at-1-name-is-total.expect &&
  generate_operate_run +n -Lwlant 8 1 is   ${file_source} > ${test_current}literal-whole-line-8-at-1-name-is-total.expect &&
  generate_operate_run +n -Lwlant 9 1 is   ${file_source} > ${test_current}literal-whole-line-9-at-1-name-is-total.expect &&
  generate_operate_run +n -Lwlant 10 1 is  ${file_source} > ${test_current}literal-whole-line-10-at-1-name-is-total.expect &&
  generate_operate_run +n -Lwlant 20 1 is  ${file_source} > ${test_current}literal-whole-line-20-at-1-name-is-total.expect &&
  generate_operate_run +n -Lwlant 100 1 is ${file_source} > ${test_current}literal-whole-line-100-at-1-name-is-total.expect &&

  generate_operate_run +n -Lwnt yup         ${file_source} > ${test_current}literal-whole-name-yup-total.expect &&
  generate_operate_run +n -Lwlnt 0 yup      ${file_source} > ${test_current}literal-whole-line-0-name-yup-total.expect &&
  generate_operate_run +n -Lwlnt 1 yup      ${file_source} > ${test_current}literal-whole-line-1-name-yup-total.expect &&
  generate_operate_run +n -Lwlnt 2 yup      ${file_source} > ${test_current}literal-whole-line-2-name-yup-total.expect &&
  generate_operate_run +n -Lwlnt 3 yup      ${file_source} > ${test_current}literal-whole-line-3-name-yup-total.expect &&
  generate_operate_run +n -Lwlnt 4 yup      ${file_source} > ${test_current}literal-whole-line-4-name-yup-total.expect &&
  generate_operate_run +n -Lwlnt 5 yup      ${file_source} > ${test_current}literal-whole-line-5-name-yup-total.expect &&
  generate_operate_run +n -Lwlnt 6 yup      ${file_source} > ${test_current}literal-whole-line-6-name-yup-total.expect &&
  generate_operate_run +n -Lwlnt 7 yup      ${file_source} > ${test_current}literal-whole-line-7-name-yup-total.expect &&
  generate_operate_run +n -Lwlnt 8 yup      ${file_source} > ${test_current}literal-whole-line-8-name-yup-total.expect &&
  generate_operate_run +n -Lwlnt 9 yup      ${file_source} > ${test_current}literal-whole-line-9-name-yup-total.expect &&
  generate_operate_run +n -Lwlnt 10 yup     ${file_source} > ${test_current}literal-whole-line-10-name-yup-total.expect &&
  generate_operate_run +n -Lwlnt 20 yup     ${file_source} > ${test_current}literal-whole-line-20-name-yup-total.expect &&
  generate_operate_run +n -Lwlnt 100 yup    ${file_source} > ${test_current}literal-whole-line-100-name-yup-total.expect &&
  generate_operate_run +n -Lwlant 0 0 yup   ${file_source} > ${test_current}literal-whole-line-0-at-0-name-yup-total.expect &&
  generate_operate_run +n -Lwlant 1 0 yup   ${file_source} > ${test_current}literal-whole-line-1-at-0-name-yup-total.expect &&
  generate_operate_run +n -Lwlant 2 0 yup   ${file_source} > ${test_current}literal-whole-line-2-at-0-name-yup-total.expect &&
  generate_operate_run +n -Lwlant 3 0 yup   ${file_source} > ${test_current}literal-whole-line-3-at-0-name-yup-total.expect &&
  generate_operate_run +n -Lwlant 4 0 yup   ${file_source} > ${test_current}literal-whole-line-4-at-0-name-yup-total.expect &&
  generate_operate_run +n -Lwlant 5 0 yup   ${file_source} > ${test_current}literal-whole-line-5-at-0-name-yup-total.expect &&
  generate_operate_run +n -Lwlant 6 0 yup   ${file_source} > ${test_current}literal-whole-line-6-at-0-name-yup-total.expect &&
  generate_operate_run +n -Lwlant 7 0 yup   ${file_source} > ${test_current}literal-whole-line-7-at-0-name-yup-total.expect &&
  generate_operate_run +n -Lwlant 8 0 yup   ${file_source} > ${test_current}literal-whole-line-8-at-0-name-yup-total.expect &&
  generate_operate_run +n -Lwlant 9 0 yup   ${file_source} > ${test_current}literal-whole-line-9-at-0-name-yup-total.expect &&
  generate_operate_run +n -Lwlant 10 0 yup  ${file_source} > ${test_current}literal-whole-line-10-at-0-name-yup-total.expect &&
  generate_operate_run +n -Lwlant 20 0 yup  ${file_source} > ${test_current}literal-whole-line-20-at-0-name-yup-total.expect &&
  generate_operate_run +n -Lwlant 100 0 yup ${file_source} > ${test_current}literal-whole-line-100-at-0-name-yup-total.expect &&
  generate_operate_run +n -Lwlant 0 1 yup   ${file_source} > ${test_current}literal-whole-line-0-at-1-name-yup-total.expect &&
  generate_operate_run +n -Lwlant 1 1 yup   ${file_source} > ${test_current}literal-whole-line-1-at-1-name-yup-total.expect &&
  generate_operate_run +n -Lwlant 2 1 yup   ${file_source} > ${test_current}literal-whole-line-2-at-1-name-yup-total.expect &&
  generate_operate_run +n -Lwlant 3 1 yup   ${file_source} > ${test_current}literal-whole-line-3-at-1-name-yup-total.expect &&
  generate_operate_run +n -Lwlant 4 1 yup   ${file_source} > ${test_current}literal-whole-line-4-at-1-name-yup-total.expect &&
  generate_operate_run +n -Lwlant 5 1 yup   ${file_source} > ${test_current}literal-whole-line-5-at-1-name-yup-total.expect &&
  generate_operate_run +n -Lwlant 6 1 yup   ${file_source} > ${test_current}literal-whole-line-6-at-1-name-yup-total.expect &&
  generate_operate_run +n -Lwlant 7 1 yup   ${file_source} > ${test_current}literal-whole-line-7-at-1-name-yup-total.expect &&
  generate_operate_run +n -Lwlant 8 1 yup   ${file_source} > ${test_current}literal-whole-line-8-at-1-name-yup-total.expect &&
  generate_operate_run +n -Lwlant 9 1 yup   ${file_source} > ${test_current}literal-whole-line-9-at-1-name-yup-total.expect &&
  generate_operate_run +n -Lwlant 10 1 yup  ${file_source} > ${test_current}literal-whole-line-10-at-1-name-yup-total.expect &&
  generate_operate_run +n -Lwlant 20 1 yup  ${file_source} > ${test_current}literal-whole-line-20-at-1-name-yup-total.expect &&
  generate_operate_run +n -Lwlant 100 1 yup ${file_source} > ${test_current}literal-whole-line-100-at-1-name-yup-total.expect &&
  generate_operate_run +n -Lwlant 100 1 yup ${file_source} > ${test_current}literal-whole-line-100-at-1-name-yup-total.expect &&

  #### REASSIGN ####
  generate_operate_run +n -cR a c sea             ${file_source} > ${test_current}content-reassign-a-c-sea.expect &&
  generate_operate_run +n -cR b c sea             ${file_source} > ${test_current}content-reassign-b-c-sea.expect &&
  generate_operate_run +n -cRR a c sea b c bee    ${file_source} > ${test_current}content-reassign-a-c-sea-b-c-bee.expect &&
  generate_operate_run +n -cR hello world friend  ${file_source} > ${test_current}content-reassign-hello-world-friend.expect &&
  generate_operate_run +n -cR yup yup yes         ${file_source} > ${test_current}content-reassign-yup-yup-yes.expect &&
  generate_operate_run +n -cR yup nope no         ${file_source} > ${test_current}content-reassign-yup-nope-no.expect &&
  generate_operate_run +n -cRw a c sea            ${file_source} > ${test_current}content-reassign-a-c-sea-whole.expect &&
  generate_operate_run +n -cRw b c sea            ${file_source} > ${test_current}content-reassign-b-c-sea-whole.expect &&
  generate_operate_run +n -cRRw a c sea b c bee   ${file_source} > ${test_current}content-reassign-a-c-sea-b-c-bee-whole.expect &&
  generate_operate_run +n -cRw hello world friend ${file_source} > ${test_current}content-reassign-hello-world-friend-whole.expect &&
  generate_operate_run +n -cRw yup yup yes        ${file_source} > ${test_current}content-reassign-yup-yup-yes-whole.expect &&
  generate_operate_run +n -cRw yup nope no        ${file_source} > ${test_current}content-reassign-yup-nope-no-whole.expect &&

  #### REPLACE ####
  generate_operate_run +n -cr a sea         ${file_source} > ${test_current}content-replace-a-sea.expect &&
  generate_operate_run +n -cr b sea         ${file_source} > ${test_current}content-replace-b-sea.expect &&
  generate_operate_run +n -crr a sea b bee  ${file_source} > ${test_current}content-reassign-a-sea-b-bee.expect &&
  generate_operate_run +n -cr hello friend  ${file_source} > ${test_current}content-replace-hello-friend.expect &&
  generate_operate_run +n -cr yup yes       ${file_source} > ${test_current}content-replace-yup-yes.expect &&
  generate_operate_run +n -crw a sea        ${file_source} > ${test_current}content-replace-a-sea-whole.expect &&
  generate_operate_run +n -crw b sea        ${file_source} > ${test_current}content-replace-b-sea-whole.expect &&
  generate_operate_run +n -crrw a sea b bee ${file_source} > ${test_current}content-reassign-a-sea-b-bee-whole.expect &&
  generate_operate_run +n -crw hello friend ${file_source} > ${test_current}content-replace-hello-friend-whole.expect &&
  generate_operate_run +n -crw yup yes      ${file_source} > ${test_current}content-replace-yup-yes-whole.expect &&

  #### SUBSTITUTE ####
  generate_operate_run +n -cS a c sea             ${file_source} > ${test_current}content-substitute-a-c-sea.expect &&
  generate_operate_run +n -cS b c sea             ${file_source} > ${test_current}content-substitute-b-c-sea.expect &&
  generate_operate_run +n -cSS a c sea b c bee    ${file_source} > ${test_current}content-substitute-a-c-sea-b-c-bee.expect &&
  generate_operate_run +n -cS hello world friend  ${file_source} > ${test_current}content-substitute-hello-world-friend.expect &&
  generate_operate_run +n -cS yup yup yes         ${file_source} > ${test_current}content-substitute-yup-yup-yes.expect &&
  generate_operate_run +n -cS yup nope no         ${file_source} > ${test_current}content-substitute-yup-nope-no.expect &&
  generate_operate_run +n -cSw a c sea            ${file_source} > ${test_current}content-substitute-a-c-sea-whole.expect &&
  generate_operate_run +n -cSw b c sea            ${file_source} > ${test_current}content-substitute-b-c-sea-whole.expect &&
  generate_operate_run +n -cSSw a c sea b c bee   ${file_source} > ${test_current}content-substitute-a-c-sea-b-c-bee-whole.expect &&
  generate_operate_run +n -cSw hello world friend ${file_source} > ${test_current}content-substitute-hello-world-friend-whole.expect &&
  generate_operate_run +n -cSw yup yup yes        ${file_source} > ${test_current}content-substitute-yup-yup-yes-whole.expect &&
  generate_operate_run +n -cSw yup nope no        ${file_source} > ${test_current}content-substitute-yup-nope-no-whole.expect &&

  #### WRAP ####
  generate_operate_run +n -cW a '[' ']'                   ${file_source} > ${test_current}content-wrap-a-lbracket-rbracket.expect &&
  generate_operate_run +n -cW b '[' ']'                   ${file_source} > ${test_current}content-wrap-b-lbracket-rbracket.expect &&
  generate_operate_run +n -cWW a '[' ']' b '{' '}'        ${file_source} > ${test_current}content-wrap-a-lbracket-rbracket-b-lbrace-rbrace.expect &&
  generate_operate_run +n -cW hello bold_open bold_close  ${file_source} > ${test_current}content-wrap-hello-bold_open-bold_close.expect &&
  generate_operate_run +n -cW yup '[' ']'                 ${file_source} > ${test_current}content-wrap-yup-lbracket-rbracket.expect &&
  generate_operate_run +n -cW nope '[' ']'                ${file_source} > ${test_current}content-wrap-yup-lbracket-rbracket.expect &&
  generate_operate_run +n -cWw a '[' ']'                  ${file_source} > ${test_current}content-wrap-a-lbracket-rbracket-whole.expect &&
  generate_operate_run +n -cWw b '[' ']'                  ${file_source} > ${test_current}content-wrap-b-lbracket-rbracket-whole.expect &&
  generate_operate_run +n -cWWw a '[' ']' b '{' '}'       ${file_source} > ${test_current}content-wrap-a-lbracket-rbracket-b-lbrace-rbrace-whole.expect &&
  generate_operate_run +n -cWw hello bold_open bold_close ${file_source} > ${test_current}content-wrap-hello-bold_open-bold_close-whole.expect &&
  generate_operate_run +n -cWw yup '[' ']'                ${file_source} > ${test_current}content-wrap-yup-lbracket-rbracket-whole.expect &&
  generate_operate_run +n -cWw nope '[' ']'               ${file_source} > ${test_current}content-wrap-yup-lbracket-rbracket-whole.expect &&

  #### WRAP - REASSIGN ####
  generate_operate_run +n -cWR a '[' ']' a c sea                              ${file_source} > ${test_current}content-wrap-a-lbracket-rbracket-reassign-a-c-sea.expect &&
  generate_operate_run +n -cWR b '[' ']' b c sea                              ${file_source} > ${test_current}content-wrap-b-lbracket-rbracket-reassign-b-c-sea.expect &&
  generate_operate_run +n -cWWRR a '[' ']' b '{' '}' a c sea b c bee          ${file_source} > ${test_current}content-wrap-a-lbracket-rbracket-b-lbrace-rbrace-reassign-a-c-sea-b-c-bee.expect &&
  generate_operate_run +n -cWR hello bold_open bold_close hello world friend  ${file_source} > ${test_current}content-wrap-hello-bold_open-bold_close-reassign-hello-world-friend.expect &&
  generate_operate_run +n -cWR yup '[' ']' yup yup yes                        ${file_source} > ${test_current}content-wrap-yup-lbracket-rbracket-reassign-yup-yup-yes.expect &&
  generate_operate_run +n -cWR nope '[' ']' yup nope no                       ${file_source} > ${test_current}content-wrap-yup-lbracket-rbracket-yup-nope-no.expect &&
  generate_operate_run +n -cWRw a '[' ']' a c sea                             ${file_source} > ${test_current}content-wrap-a-lbracket-rbracket-reassign-a-c-sea-whole.expect &&
  generate_operate_run +n -cWRw b '[' ']' b c sea                             ${file_source} > ${test_current}content-wrap-b-lbracket-rbracket-reassign-b-c-sea-whole.expect &&
  generate_operate_run +n -cWWRRw a '[' ']' b '{' '}' a c sea b c bee         ${file_source} > ${test_current}content-wrap-a-lbracket-rbracket-b-lbrace-rbrace-reassign-a-c-sea-b-c-bee-whole.expect &&
  generate_operate_run +n -cWRw hello bold_open bold_close hello world friend ${file_source} > ${test_current}content-wrap-hello-bold_open-bold_close-reassign-hello-world-friend-whole.expect &&
  generate_operate_run +n -cWRw yup '[' ']' yup yup yes                       ${file_source} > ${test_current}content-wrap-yup-lbracket-rbracket-reassign-yup-yup-yes-whole.expect &&
  generate_operate_run +n -cWRw nope '[' ']' yup nope no                      ${file_source} > ${test_current}content-wrap-yup-lbracket-rbracket-yup-nope-no-whole.expect &&

  #### WRAP - REPLACE ####
  generate_operate_run +n -cWr a '[' ']' a sea                          ${file_source} > ${test_current}content-wrap-a-lbracket-rbracket-replace-a-sea.expect &&
  generate_operate_run +n -cWr b '[' ']' b sea                          ${file_source} > ${test_current}content-wrap-b-lbracket-rbracket-replace-b-sea.expect &&
  generate_operate_run +n -cWWrr a '[' ']' b '{' '}' a sea b bee        ${file_source} > ${test_current}content-wrap-a-lbracket-rbracket-b-lbrace-rbrace-replace-a-sea-b-bee.expect &&
  generate_operate_run +n -cWr hello bold_open bold_close hello friend  ${file_source} > ${test_current}content-wrap-hello-bold_open-bold_close-replace-hello-friend.expect &&
  generate_operate_run +n -cWr yup '[' ']' yup yes                      ${file_source} > ${test_current}content-wrap-yup-lbracket-rbracket-replace-yup-yes.expect &&
  generate_operate_run +n -cWrw a '[' ']' a sea                         ${file_source} > ${test_current}content-wrap-a-lbracket-rbracket-replace-a-sea-whole.expect &&
  generate_operate_run +n -cWrw b '[' ']' b sea                         ${file_source} > ${test_current}content-wrap-b-lbracket-rbracket-replace-b-sea-whole.expect &&
  generate_operate_run +n -cWWrrw a '[' ']' b '{' '}' a sea b bee       ${file_source} > ${test_current}content-wrap-a-lbracket-rbracket-b-lbrace-rbrace-replace-a-sea-b-bee-whole.expect &&
  generate_operate_run +n -cWrw hello bold_open bold_close hello friend ${file_source} > ${test_current}content-wrap-hello-bold_open-bold_close-replace-hello-friend-whole.expect &&
  generate_operate_run +n -cWrw yup '[' ']' yup yes                     ${file_source} > ${test_current}content-wrap-yup-lbracket-rbracket-replace-yup-yes-whole.expect &&

  #### WRAP - SUBSTITUTE ####
  generate_operate_run +n -cWS a '[' ']' a c sea                              ${file_source} > ${test_current}content-wrap-a-lbracket-rbracket-substitute-a-c-sea.expect &&
  generate_operate_run +n -cWS b '[' ']' b c sea                              ${file_source} > ${test_current}content-wrap-b-lbracket-rbracket-substitute-b-c-sea.expect &&
  generate_operate_run +n -cWWSS a '[' ']' b '{' '}' a c sea b c bee          ${file_source} > ${test_current}content-wrap-a-lbracket-rbracket-b-lbrace-rbrace-substitute-a-c-sea-b-c-bee.expect &&
  generate_operate_run +n -cWS hello bold_open bold_close hello world friend  ${file_source} > ${test_current}content-wrap-hello-bold_open-bold_close-substitute-hello-world-friend.expect &&
  generate_operate_run +n -cWS yup '[' ']' yup yup yes                        ${file_source} > ${test_current}content-wrap-yup-lbracket-rbracket-substitute-yup-yup-yes.expect &&
  generate_operate_run +n -cWS nope '[' ']' yup nope no                       ${file_source} > ${test_current}content-wrap-yup-lbracket-rbracket-yup-nope-no.expect &&
  generate_operate_run +n -cWSw a '[' ']' a c sea                             ${file_source} > ${test_current}content-wrap-a-lbracket-rbracket-substitute-a-c-sea-whole.expect &&
  generate_operate_run +n -cWSw b '[' ']' b c sea                             ${file_source} > ${test_current}content-wrap-b-lbracket-rbracket-substitute-b-c-sea-whole.expect &&
  generate_operate_run +n -cWWSSw a '[' ']' b '{' '}' a c sea b c bee         ${file_source} > ${test_current}content-wrap-a-lbracket-rbracket-b-lbrace-rbrace-substitute-a-c-sea-b-c-bee-whole.expect &&
  generate_operate_run +n -cWSw hello bold_open bold_close hello world friend ${file_source} > ${test_current}content-wrap-hello-bold_open-bold_close-substitute-hello-world-friend-whole.expect &&
  generate_operate_run +n -cWSw yup '[' ']' yup yup yes                       ${file_source} > ${test_current}content-wrap-yup-lbracket-rbracket-substitute-yup-yup-yes-whole.expect &&
  generate_operate_run +n -cWSw nope '[' ']' yup nope no                      ${file_source} > ${test_current}content-wrap-yup-lbracket-rbracket-yup-nope-no-whole.expect &&

  echo "Generation Complete"

  if [[ $? -ne 0 ]] ; then
    let failure=1

    echo
    echo "Failure: ${last_command}"
    echo
  fi

  return $failure
}

generate_operate_run() {

  if [[ ${test_command} == "eki" ]] ; then
    last_command="eki_read ${@}"

    eki_read "$@"
  else
    last_command="iki_read ${@}"

    iki_read "$@"
  fi

  return $?
}

generate_cleanup() {

  unset generate_main
  unset generate_operate_iki
  unset generate_operate_run
  unset generate_cleanup
}

generate_main "$@"
